@@ -1,6 +1,10 @@
 #ifndef PRIVATE_H
 #define PRIVATE_H
 
+#include <inttypes.h>
+#include <lqt_atoms.h>
+#include <stdio.h>
+
 /* ================================= structures */
 
 
@@ -12,7 +16,8 @@
 
 #define HEADER_LENGTH 8
 #define MAXTRACKS 1024
- 
+#define MAXNODES 1
+
 /* Crazy Mich R. Soft constants */
 #define AVI_HASINDEX       0x00000010  // Index at end of file?
 #define AVI_MUSTUSEINDEX   0x00000020
@@ -28,17 +33,12 @@
 #define AVI_FRAME_RATE_BASE 10000
 #define MAX_RIFFS  0x100
 
-#define QTVR_OBJ 1
-#define QTVR_PAN 2
 
-#define QTVR_STANDARD_OBJECT 1
-#define QTVR_OLD_NAVIGABLE_MOVIE_SCENE 2 
-#define QTVR_OBJECT_IN_SCENE 3
-
-
-//#include "codecs.h"
-#include <stdio.h>
-#include <inttypes.h>
+#define QTVR_GRABBER_SCROLLER_UI 1
+#define QTVR_OLD_JOYSTICK_UI 2 
+#define QTVR_JOYSTICK_UI 3
+#define QTVR_GRABBER_UI 4
+#define QTVR_ABSOLUTE_UI 5
 
 /* Forward declarations */
 
@@ -55,22 +55,41 @@ typedef struct
 	unsigned char type[4];
 } quicktime_atom_t;
 
+
+typedef struct
+{
+	int64_t start;
+	int64_t end;        /* byte endpoint in file */
+	int64_t size;       /* byte size for writing */
+	unsigned char type[4];
+	int child_count;
+	int use_64;
+	long ID;
+} quicktime_qtatom_t;
+
 typedef struct
 {
 	float values[9];
 } quicktime_matrix_t;
 
+typedef struct
+  {
+  uint32_t major_brand;
+  uint32_t minor_version;
+  int num_compatible_brands;
+  uint32_t * compatible_brands;
+  } quicktime_ftyp_t;
 
 typedef struct
 {
 	int version;
 	long flags;
-	unsigned long creation_time;
-	unsigned long modification_time;
+        uint64_t creation_time;
+	uint64_t modification_time;
 	int track_id;
 	long reserved1;
-	long duration;
-	char reserved2[8];
+	uint64_t duration;
+        uint8_t reserved2[8];
 	int layer;
 	int alternate_group;
 	float volume;
@@ -111,31 +130,25 @@ typedef struct
 	int motion_jpeg_huffman_table;
 } quicktime_mjht_t;
 
-typedef struct
-{
-	int32_t hSpacing;
-	int32_t vSpacing;
-} quicktime_pasp_t;
 
 typedef struct
 {
-	int32_t colorParamType;
-	int16_t primaries;
-	int16_t transferFunction;
-	int16_t matrix;
-} quicktime_colr_t;
+    int version;
+    int revision;
+    long imagingMode;
+    long imagingValidFlags;
+    long correction;
+    long quality;
+    long directdraw;
+    long imagingProperties[5];
+    long reserved1;
+    long reserved2;
+} quicktime_impn_t;
 
 typedef struct
 {
-	int32_t cleanApertureWidthN;
-	int32_t cleanApertureWidthD;
-	int32_t cleanApertureHeightN;
-	int32_t cleanApertureHeightD;
-	int32_t horizOffN;
-	int32_t horizOffD;
-	int32_t vertOffN;
-	int32_t vertOffD;
-} quicktime_clap_t;
+    quicktime_impn_t impn;
+} quicktime_imgp_t;
 
 typedef struct
 {
@@ -146,9 +159,9 @@ typedef struct
 	
 	long STrack; /* Prefix 'S' == Scene */
 	long LowResSTrack;
-	long reserved3[6];
+	uint32_t reserved3[6];
 	long HSTrack; /* Prefix 'HS' == HotSpot */
-	long reserved4[9];
+	uint32_t reserved4[9];
 	
 	float HPanStart;
 	float HPanEnd;
@@ -173,10 +186,172 @@ typedef struct
 	int HSDepth;
 } quicktime_pano_t;
 
+typedef struct
+{
+	int version;
+	int revision;
+	char nodeType[4];
+	long locationFlags;
+	long locationData;
+	long reserved1;
+	long reserved2;
+} quicktime_nloc_t;
+
+typedef struct
+{
+	int version;
+	int revision;
+	char nodeType[4];
+	long nodeID;
+	long nameAtomID;
+	long commentAtomID;
+	long reserved1;
+	long reserved2;
+} quicktime_ndhd_t;
+
+typedef struct
+{
+	quicktime_nloc_t nloc;
+	int ID;
+} quicktime_vrni_t;
+
+typedef struct
+{
+	quicktime_vrni_t vrni[MAXNODES];
+	int children;
+} quicktime_vrnp_t;
+
+typedef struct
+{
+	int version;
+	int revision;
+	long NameAtomID;
+	long DefaultNodeID;
+	long flags;
+	long reserved1;
+	long reserved2;
+	
+} quicktime_vrsc_t;
+
+typedef struct
+{
+	quicktime_vrsc_t vrsc;
+	quicktime_imgp_t imgp;
+	quicktime_vrnp_t vrnp;
+} quicktime_qtvr_t;
+
+
+/* MPEG-4 esds (elementary stream descriptor) */
+
+typedef struct
+  {
+  int version;
+  long flags;
+
+  uint16_t esid;
+  uint8_t stream_priority;
+  
+  uint8_t  objectTypeId;
+  uint8_t  streamType;
+  uint32_t bufferSizeDB;
+  uint32_t maxBitrate;
+  uint32_t avgBitrate;
+
+  int      decoderConfigLen;
+  uint8_t* decoderConfig;
+
+  } quicktime_esds_t;
+
+/* MPEG-4 iods */
+
+typedef struct
+  {
+  int version;
+  long flags;
+
+  uint16_t ObjectDescriptorID;
+  uint8_t  ODProfileLevel;
+  uint8_t  sceneProfileLevel;
+  uint8_t  audioProfileId;
+  uint8_t  videoProfileId;
+  uint8_t  graphicsProfileLevel;
+
+  struct
+    {
+    uint8_t ES_ID_IncTag;
+    uint8_t length;
+    uint32_t track_id;
+    } * tracks;
+  int num_tracks;
+  } quicktime_iods_t;
+
+/* User atoms: These can be either inside a wave atom (for audio) or
+   in the sample description (for video) */
+
+typedef struct
+  {
+  int num_atoms;
+  uint8_t ** atoms;
+  } quicktime_user_atoms_t;
+
+/* wave atom and subatoms */
+
+typedef struct
+  {
+  char codec[4];
+  /* Remainder could be a WAVEFORMATEX structure */
+  } quicktime_frma_t;
+
+typedef struct
+  {
+  int16_t littleEndian;
+  } quicktime_enda_t;
+
+typedef struct
+  {
+  quicktime_frma_t frma;
+  int has_frma;
+  quicktime_enda_t enda;
+  int has_enda;
+
+  quicktime_esds_t esds;
+  int has_esds;
+  
+  quicktime_user_atoms_t user_atoms;
+  } quicktime_wave_t;
+
+typedef struct
+  {
+  int version;
+  long flags;
+  
+  uint32_t mChannelLayoutTag;
+  uint32_t mChannelBitmap;
+  uint32_t mNumberChannelDescriptions;
+
+  struct
+    {
+    uint32_t mChannelLabel;
+    uint32_t mChannelFlags;
+    float    mCoordinates[3];
+    } * ChannelDescriptions;
+  } quicktime_chan_t;
+
+typedef struct
+  {
+  int fields;    /* 0, 1, or 2 */
+  int dominance;   /* 0 - unknown     1 - top first     2 - bottom first */
+  } quicktime_fiel_t;
+
+typedef struct
+  {
+  float gamma;
+  } quicktime_gama_t;
+
 typedef struct
 {
 	char format[4];
-	char reserved[6];
+	uint8_t reserved[6];
 	int data_reference;
 
 /* common to audio and video */
@@ -197,30 +372,61 @@ typedef struct
 	int depth;
 	int ctab_id;
 	quicktime_ctab_t ctab;
-	float gamma;
-	int fields;    /* 0, 1, or 2 */
-	int field_dominance;   /* 0 - unknown     1 - top first     2 - bottom first */
-	quicktime_mjqt_t mjqt;
-	quicktime_mjht_t mjht;
-	quicktime_pasp_t pasp;
-	quicktime_colr_t colr;
-	quicktime_clap_t clap;
-	quicktime_pano_t pano;
 
+        quicktime_pasp_t pasp;
+        int has_pasp;
+	quicktime_colr_t colr;
+	int has_colr;
+	quicktime_clap_t clap;
+        int has_clap;
+
+        quicktime_fiel_t fiel;
+        int has_fiel;
+
+        quicktime_gama_t gama;
+        int has_gama;
+
+        quicktime_pano_t pano;
+	quicktime_qtvr_t qtvr;
+
+        int has_wave;
+        quicktime_wave_t wave;
 /* audio description */
-	int channels;
-	int sample_size;
+	uint32_t channels;
+	uint32_t sample_size;
+/* Audio extension for version == 2 */
+        uint32_t formatSpecificFlags;
+        uint32_t constBytesPerAudioPacket;
+        uint32_t constLPCMFramesPerAudioPacket;
 /* LQT: We have int16_t for the compression_id, because otherwise negative
    values don't show up correctly */
         int16_t compression_id;
 	int packet_size;
-	float sample_rate;
+        double samplerate;
 
-/* LQT: We store the complete atom (starting witht he fourcc)
+/* Audio extension for version == 1 */
+
+        uint32_t audio_samples_per_packet;
+        uint32_t audio_bytes_per_packet;
+        uint32_t audio_bytes_per_frame;
+        uint32_t audio_bytes_per_sample;
+
+
+        quicktime_esds_t esds;
+        int has_esds;
+
+        quicktime_chan_t chan;
+        int has_chan;
+
+        quicktime_user_atoms_t user_atoms;
+
+/* LQT: We store the complete atom (starting with the fourcc)
    here, because this must be passed to the Sorenson 3 decoder */
 
-        unsigned char * extradata;
-        int extradata_size;
+        unsigned char * table_raw;
+        int table_raw_size;
+
+        
 
 } quicktime_stsd_table_t;
 
@@ -251,6 +457,22 @@ typedef struct
 	quicktime_stts_table_t *table;
 } quicktime_stts_t;
 
+/* Composition time to sample */
+
+typedef struct
+{
+	long sample_count;
+	long sample_duration;
+} quicktime_ctts_table_t;
+
+typedef struct
+{
+	int version;
+	long flags;
+	long total_entries;
+        long entries_allocated;
+	quicktime_ctts_table_t *table;
+} quicktime_ctts_t;
 
 /* sync sample */
 typedef struct
@@ -333,7 +555,15 @@ typedef struct
 	quicktime_stsc_t stsc;
 	quicktime_stsz_t stsz;
 	quicktime_stco_t stco;
-} quicktime_stbl_t;
+	quicktime_ctts_t ctts;
+        int has_ctts;
+  } quicktime_stbl_t;
+
+typedef struct
+{
+    	char refType[4];
+	long trackIndex;
+} quicktime_tref_t;
 
 /* data reference */
 
@@ -412,7 +642,7 @@ typedef struct
 	long flags;
 	char component_type[4];
 	char component_subtype[4];
-	long component_manufacturer;
+        char component_manufacturer[4];
 	long component_flags;
 	long component_flag_mask;
 	char component_name[256];
@@ -424,8 +654,11 @@ typedef struct
 {
 	int is_video;
 	int is_audio;
+        int is_audio_vbr;   /* Special flag indicating VBR audio */
 	int has_baseheader;
 	int is_panorama;
+	int is_qtvr;
+	int is_object;
 	quicktime_vmhd_t vmhd;
 	quicktime_smhd_t smhd;
 	quicktime_gmhd_t gmhd;
@@ -515,11 +748,13 @@ typedef struct
 	quicktime_tkhd_t tkhd;
 	quicktime_mdia_t mdia;
 	quicktime_edts_t edts;
+	quicktime_tref_t tref;
         quicktime_strl_t * strl; // != NULL for AVI files during reading and writing
         int chunk_sizes_alloc;
         int64_t * chunk_sizes; /* This contains the chunk sizes for audio
                                   tracks. They can not so easily be obtained
                                   during decoding */
+	int has_tref;
 } quicktime_trak_t;
 
 
@@ -527,13 +762,13 @@ typedef struct
 {
 	int version;
 	long flags;
-	unsigned long creation_time;
-	unsigned long modification_time;
+	uint64_t creation_time;
+	uint64_t modification_time;
 	long time_scale;
-	long duration;
+	uint64_t duration;
 	float preferred_rate;
 	float preferred_volume;
-	char reserved[10];
+	uint8_t reserved[10];
 	quicktime_matrix_t matrix;
 	long preview_time;
 	long preview_duration;
@@ -581,6 +816,9 @@ typedef struct
 	quicktime_trak_t *trak[MAXTRACKS];
 	quicktime_udta_t udta;
 	quicktime_ctab_t ctab;
+        int has_iods;
+        quicktime_iods_t iods;
+
 } quicktime_moov_t;
 
 typedef struct
@@ -593,7 +831,7 @@ typedef struct
 /* Offset of end of 8 byte chunk header relative to ix->base_offset */
         int relative_offset;
 /* size of data without 8 byte header */
-        int size;
+        uint32_t size;
 } quicktime_ixtable_t;
                                                                                                                      
 typedef struct
@@ -615,10 +853,6 @@ typedef struct
 {
         quicktime_atom_t atom;
                                                                                                                      
-/* Partial index */
-/* For writing only, there are multiple movi objects with multiple ix tables. */
-/* This is not used for reading.  Instead an ix_t object in indx_t is used. */
-        quicktime_ix_t *ix[MAXTRACKS];
 } quicktime_movi_t;
                                                                                                                      
 typedef struct
@@ -647,37 +881,154 @@ typedef struct
         int table_size;
         int table_allocation;
         quicktime_indxtable_t *table;
-} quicktime_indx_t;
+
+/* The following are used internally only. They are
+   set by quicktime_finalize_strl and used by quicktime_finalize_indx */
+  uint32_t offset;
+  uint32_t size;
+  } quicktime_indx_t;
+
+/* AVI structures */
+
+/* strh */
+
+typedef struct
+  {
+  char fccType[4];
+  char fccHandler[4];
+  uint32_t dwFlags;
+  uint32_t dwReserved1;
+  uint32_t dwInitialFrames;
+  uint32_t dwScale;
+  uint32_t dwRate;
+  uint32_t dwStart;
+  uint32_t dwLength;
+  uint32_t dwSuggestedBufferSize;
+  uint32_t dwQuality;
+  uint32_t dwSampleSize;
+
+  struct {
+  uint16_t left;
+  uint16_t top;
+  uint16_t right;
+  uint16_t bottom;
+  }  rcFrame;
+  
+  } quicktime_strh_t;
+
+typedef struct
+  {
+  uint32_t dwMicroSecPerFrame;
+  uint32_t dwMaxBytesPerSec;
+  uint32_t dwReserved1;
+  uint32_t dwFlags;
+  uint32_t dwTotalFrames;
+  uint32_t dwInitialFrames;
+  uint32_t dwStreams;
+  uint32_t dwSuggestedBufferSize;
+  uint32_t dwWidth;
+  uint32_t dwHeight;
+  uint32_t dwScale;
+  uint32_t dwRate;
+  uint32_t dwStart;
+  uint32_t dwLength;
+  } quicktime_avih_t;
+
+typedef struct
+  {
+  uint32_t  biSize;  /* sizeof(BITMAPINFOHEADER) */
+  uint32_t  biWidth;
+  uint32_t  biHeight;
+  uint16_t  biPlanes; /* unused */
+  uint16_t  biBitCount;
+  char biCompression[5]; /* fourcc of image */
+  uint32_t  biSizeImage;   /* size of image. For uncompressed images */
+                           /* ( biCompression 0 or 3 ) can be zero.  */
+
+  uint32_t  biXPelsPerMeter; /* unused */
+  uint32_t  biYPelsPerMeter; /* unused */
+  uint32_t  biClrUsed;       /* valid only for palettized images. */
+  /* Number of colors in palette. */
+  uint32_t  biClrImportant;
+
+  /* Additional stuff */
+  int ext_size;
+  uint8_t * ext_data;
+  } quicktime_BITMAPINFOHEADER_t;
+
+typedef struct
+  {
+  uint32_t v1;
+  uint16_t v2;
+  uint16_t v3;
+  uint8_t  v4[8];
+  } quicktime_GUID_t;
+
+typedef enum
+  {
+    LQT_WAVEFORMAT_WAVEFORMAT,
+    LQT_WAVEFORMAT_PCMWAVEFORMAT,
+    LQT_WAVEFORMAT_WAVEFORMATEX,
+    LQT_WAVEFORMAT_WAVEFORMATEXTENSIBLE,
+  } quicktime_WAVEFORMAT_type_t;
+
+typedef struct
+  {
+  quicktime_WAVEFORMAT_type_t type;
+
+  struct
+    {
+    struct
+      {
+      uint16_t  wFormatTag;     /* value that identifies compression format */
+      uint16_t  nChannels;
+      uint32_t  nSamplesPerSec;
+      uint32_t  nAvgBytesPerSec;
+      uint16_t  nBlockAlign;    /* size of a data sample */
+      } WAVEFORMAT;
+    struct
+      {
+      uint16_t  wBitsPerSample;
+      } PCMWAVEFORMAT;
+    struct
+      {
+      uint16_t  cbSize;         /* size of format-specific data */
+      uint8_t * ext_data;
+      int       ext_size; /* Can be different from cbSize for WAVEFORMATEXTENSIBLE */
+      } WAVEFORMATEX;
+    struct
+      {
+      union
+        {
+        uint16_t wValidBitsPerSample;
+        uint16_t wSamplesPerBlock;
+        uint16_t wReserved;
+        } Samples;
+      uint32_t    dwChannelMask;
+      quicktime_GUID_t SubFormat;
+      } WAVEFORMATEXTENSIBLE;
+    } f;
+  } quicktime_WAVEFORMAT_t;
+
+typedef union
+  {
+  quicktime_BITMAPINFOHEADER_t bh;
+  quicktime_WAVEFORMAT_t       wf;
+  } quicktime_strf_t;
 
 struct quicktime_strl_s
 {
         quicktime_atom_t atom;
 /* Super index for reading */
         quicktime_indx_t indx;
-/* LIBQUICKTIME: These are the important values to make proper audio
-   headers. They are set by the ENCODER in the encode-functions before
-   writing the first audio chunk (i.e. after a call to quicktime_set_avi) */
 
-        /* strh stuff */
-
-        int64_t  dwScaleOffset;
-        uint32_t dwScale;
-
-        uint32_t dwRate;
-
-        int64_t  dwLengthOffset;
-
-        int64_t dwSampleSizeOffset;
-        uint32_t dwSampleSize;
-
-        /* WAVEFORMATEX stuff */
-
-        uint16_t nBlockAlign;
-
-        int64_t nAvgBytesPerSecOffset;
-        uint32_t nAvgBytesPerSec;
-
-        uint16_t wBitsPerSample;
+     /* strh stuff */
+        
+        int64_t  strh_offset;
+        int64_t  end_pos;
+        quicktime_strh_t strh;
+        /* Format */
+        quicktime_strf_t strf;
 
         /* The next ones will be used for generating index tables
            with the lowest possible error */
@@ -686,10 +1037,8 @@ struct quicktime_strl_s
         int64_t total_samples;
         int64_t total_blocks;
 
-/* Start of indx header for later writing */
-        int64_t indx_offset;
-/* Size of JUNK without 8 byte header which is to be replaced by indx */
-        int64_t padding_size;
+        uint32_t size; /* The size after the strl was first written */
+
 /* Tag for writer with NULL termination: 00wb, 00dc   Not available in reader.*/
         char tag[5];
 /* Flags for reader.  Not available in writer. */
@@ -701,10 +1050,9 @@ struct quicktime_strl_s
 
 typedef struct
 {
+        int64_t avih_offset;
+        quicktime_avih_t avih;
         quicktime_atom_t atom;
-        int64_t frames_offset;
-        int64_t bitrate_offset;
-/* Offsets to be written during file closure */
         int64_t total_frames_offset;
                                                                                                                      
 /* AVI equivalent for each trak.  Use file->moov.total_tracks */
@@ -721,6 +1069,53 @@ typedef struct
 /* Size of chunk less the 8 byte header */
         int32_t size;
 } quicktime_idx1table_t;
+
+typedef struct
+  {
+  char * IARL; // Archival Location. Indicates where the subject of the file is archived.
+  char * IART; // Artist. Lists the artist of the original subject of the file. For example, "Michaelangelo."
+  char * ICMS; // Commissioned. Lists the name of the person or organization that commissioned the subject of
+               // the file. For example, "Pope Julian II."
+  char * ICMT; // Comments. Provides general comments about the file or the subject of the file. If the
+               // comment is several sentences long, end each sentence with a period. Do not include newline
+               // characters.
+  char * ICOP; // Copyright. Records the copyright information for the file. For example,
+               // "Copyright Encyclopedia International 1991." If there are multiple copyrights, separate them
+               // by a semicolon followed by a space.
+  char * ICRD; // Creation date. Specifies the date the subject of the file was created. List dates in
+               // year-month-day format, padding one-digit months and days with a zero on the left. For example,
+               // "1553-05-03" for May 3, 1553.
+  char * ICRP; // Cropped. Describes whether an image has been cropped and, if so, how it was cropped. For example,
+               // "lower right corner."
+  char * IDIM; // Dimensions. Specifies the size of the original subject of the file. For example,
+               // "8.5 in h, 11 in w."
+  char * IDPI; // Dots Per Inch. Stores dots per inch setting of the digitizer used to produce the file, such as
+               // "300."
+  char * IENG; // Engineer. Stores the name of the engineer who worked on the file. If there are multiple engineers,
+               // separate the names by a semicolon and a blank. For example, "Smith, John; Adams, Joe."
+  char * IGNR; // Genre. Describes the original work, such as, "landscape," "portrait," "still life," etc.
+  char * IKEY; // Keywords. Provides a list of keywords that refer to the file or subject of the file. Separate
+               // multiple keywords with a semicolon and a blank. For example, "Seattle; aerial view; scenery."
+  char * ILGT; // Lightness. Describes the changes in lightness settings on the digitizer required to produce the
+               // file. Note that the format of this information depends on hardware used.
+  char * IMED; // Medium. Describes the original subject of the file, such as, "computer image," "drawing,"
+               // "lithograph," and so forth.
+  char * INAM; // Name. Stores the title of the subject of the file, such as, "Seattle From Above."
+  char * IPLT; // Palette Setting. Specifies the number of colors requested when digitizing an image, such as "256."
+  char * IPRD; // Product. Specifies the name of the title the file was originally intended for, such as
+               // "Encyclopedia of Pacific Northwest Geography."
+  char * ISBJ; // Subject. Describes the conbittents of the file, such as "Aerial view of Seattle."
+  char * ISFT; // Software. Identifies the name of the software package used to create the file, such as
+               // "Microsoft WaveEdit."
+  char * ISHP; // Sharpness. Identifies the changes in sharpness for the digitizer required to produce the file
+               // (the format depends on the hardware used).
+  char * ISRC; // Source. Identifies the name of the person or organization who supplied the original subject of the
+               // file. For example, "Trey Research."
+  char * ISRF; // Source Form. Identifies the original form of the material that was digitized, such as "slide,"
+               // "paper," "map," and so forth. This is not necessarily the same as IMED.
+  char * ITCH; // Technician. Identifies the technician who digitized the subject file. For example, "Smith, John."
+  } quicktime_riffinfo_t;
+
 typedef struct
 {
         quicktime_atom_t atom;
@@ -734,12 +1129,13 @@ typedef struct
         quicktime_atom_t atom;
         quicktime_movi_t movi;
         quicktime_hdrl_t hdrl;
-                                                                                                                     
+        quicktime_riffinfo_t info;
 /* Full index */
         quicktime_idx1_t idx1;
 /* Notify reader the idx1 table is valid */
         int have_idx1;
         int have_hdrl;
+        int have_info;
 } quicktime_riff_t;
 
 
@@ -747,15 +1143,42 @@ typedef struct
 typedef struct
 {
 	quicktime_trak_t *track; /* real quicktime track corresponding to this table */
-	int channels;            /* number of audio channels in the track */
-	int64_t current_position;   /* current sample in output file */
+	int channels;
+
+        int samplerate;
+
+        /* number of audio channels in the track */
+        lqt_channel_t * channel_setup;
+        int64_t current_position;   /* current sample in output file */
 	int64_t current_chunk;      /* current chunk in output file */
 
         /* Last position if set by the codec. If last position and current position
            differ, the codec knows, that a seek happened since the last decode call */
-        long last_position;
+        int64_t last_position;
 	void *codec;
 
+        int eof; /* This is set to 1 by the core if one tries to read beyond EOF */
+
+/* Another API enhancement: Codecs only deliver samples in the format specified by
+   sample_format. The usual decode() functions will convert them to int16_t or float */
+   
+        lqt_sample_format_t sample_format; /* Set by the codec */
+
+        uint8_t * sample_buffer;
+        int sample_buffer_alloc;  /* Allocated size in SAMPLES of the sample buffer */
+
+/* VBR stuff */
+        int vbr_num_frames; /* Frames written since start of chunk */
+        int64_t vbr_frame_start;
+        int64_t vbr_frames_written;
+
+        /* The total samples are calculated while initializing, but they MIGHT
+           be changed by the faad decoder, in the case resampling happens. */
+        int64_t total_samples;
+
+/* WAV ID: Taken from the codec_info and saved here for writing the AVI header */
+        int wav_id;
+
 } quicktime_audio_map_t;
 
 typedef struct
@@ -768,11 +1191,56 @@ typedef struct
 /* Timestamp of the NEXT frame decoded. Unit is the timescale of the track */
 /* For Encoding: Timestamp of the LAST encoded frame */
         int64_t timestamp;
+
+/* For codecs with B-frames: Timestamp of the just written frame */
+        int has_b_frames; /* Probably needed for encoding only */
+        int64_t coded_timestamp; /* Set by the codec during encoding */
+
         int64_t stts_index;
         int64_t stts_count;
-        int color_model, row_span, row_span_uv;
+        int stream_cmodel;  /* Colormodel, which is read/written natively by the codec  */
+        int stream_row_span, stream_row_span_uv;
 
-} quicktime_video_map_t;
+        int io_cmodel;      /* Colormodel, which is used by the encode/decode functions */
+        int io_row_span, io_row_span_uv;
+
+        /* This is used by the core to do implicit colorspace conversion and scaling (NOT recommended!!) */
+        uint8_t ** temp_frame;
+        lqt_chroma_placement_t chroma_placement;
+        lqt_interlace_mode_t interlace_mode;
+
+  } quicktime_video_map_t;
+
+/* obji */
+
+typedef struct
+{
+    int version;
+    int revision;
+    int movieType;
+    int viewStateCount;
+    int defaultViewState;
+    int mouseDownViewState;
+    long viewDuration;
+    long columns;
+    long rows;
+    float mouseMotionScale;
+    float minPan;
+    float maxPan;
+    float defaultPan;
+    float minTilt;
+    float maxTilt;
+    float defaultTilt;
+    float minFOV;
+    float FOV;
+    float defaultFOV;
+    float defaultViewCenterH;
+    float defaultViewCenterV;
+    float viewRate;
+    float frameRate;
+    long animSettings;
+    long controlSettings;
+} quicktime_obji_t;
 
 /* pHdr */
 
@@ -797,15 +1265,31 @@ typedef struct
     long        commentStrOffset;    // offset into string table atom
 } quicktime_pHdr_t;
 
-
-/* file descriptor passed to all routines */
+/* qtvr node */
 
 typedef struct
+{
+    	int node_type;
+	int64_t node_start; /* start frame */ 
+	int64_t node_size; /* size of node in frames */
+	quicktime_ndhd_t ndhd;
+	quicktime_obji_t obji;
+	} quicktime_qtvr_node_t;
+
+/* file descriptor passed to all routines */
+
+struct quicktime_s
 {
 	FILE *stream;
 	int64_t total_length;
+        int encoding_started;
 	quicktime_mdat_t mdat;
 	quicktime_moov_t moov;
+        quicktime_ftyp_t ftyp;
+        int has_ftyp;
+
+        lqt_file_type_t file_type;
+
 	int rd;
 	int wr;
 
@@ -818,20 +1302,11 @@ typedef struct
  */
         int64_t moov_end;
         int64_t moov_size;
-        int64_t old_preload_size;
-        char *old_preload_buffer;
-        int64_t old_preload_start;
-        int64_t old_preload_end;
-        int64_t old_preload_ptr;
 
-/* ASF section */
-        int use_asf;
-
-        int use_avi;
 /* AVI tree */
         quicktime_riff_t *riff[MAX_RIFFS];
         int total_riffs;
-
+        uint32_t max_riff_size;
 
 /* for begining and ending frame writes where the user wants to write the  */
 /* file descriptor directly */
@@ -845,7 +1320,7 @@ typedef struct
 
 /* Read ahead buffer */
 	int64_t preload_size;      /* Enables preload when nonzero. */
-	char *preload_buffer;
+	uint8_t *preload_buffer;
 	int64_t preload_start;     /* Start of preload_buffer in file */
 	int64_t preload_end;       /* End of preload buffer in file */
 	int64_t preload_ptr;       /* Offset of preload_start in preload_buffer */
@@ -855,7 +1330,7 @@ typedef struct
         int64_t presave_size;
 /* Next presave byte's position in file */
         int64_t presave_position;
-        char *presave_buffer;
+        uint8_t *presave_buffer;
 /* Presave doesn't matter a whole lot, so its size is fixed */
 #define QUICKTIME_PRESAVE 0x100000
 
@@ -868,15 +1343,13 @@ typedef struct
 	int total_vtracks;
 	quicktime_video_map_t *vtracks;
 
-/* Number of processors at our disposal */
-	int cpus;
-
 /* Parameters for frame currently being decoded */
-	int do_scaling;
 	int in_x, in_y, in_w, in_h, out_w, out_h;
 /*      Libquicktime change: color_model and row_span are now saved per track */
 /*	int color_model, row_span; */
-} quicktime_t;
+
+	quicktime_qtvr_node_t qtvr_node[MAXNODES];
+};
 
 typedef struct
 {
@@ -890,39 +1363,23 @@ typedef struct
 				int track);
         /* API Change: Return value is the number of samples */
         int (*decode_audio)(quicktime_t *file, 
-                            int16_t ** output_i, 
-                            float ** output_f,
+                            void * output,
                             long samples, 
                             int track);
 	int (*encode_audio)(quicktime_t *file, 
-				int16_t **input_i, 
-				float **input_f, 
-				int track, 
-                            long samples);
-	int (*reads_colormodel)(quicktime_t *file, 
-		int colormodel, 
-		int track);
-	int (*writes_colormodel)(quicktime_t *file, 
-		int colormodel, 
-		int track);
+                            void * input,
+                            long samples,
+                            int track);
 	int (*set_parameter)(quicktime_t *file, 
 		int track, 
 		char *key, 
 		void *value);
-	void (*flush)(quicktime_t *file, 
-		int track);
-
-        /* AVI codec ID for audio.  AVI codec ID's are based on WAV files, by the way. */
-        int wav_id;
-                                                                                   
-/* Pointer to static character code for identifying the codec. */
-        char *fourcc;
-                                                                                   
-/* English title of codec.  Optional. */
-        char *title;
-                                                                                   
-/* English description of codec.  Optional. */
-        char *desc;
+        /* Set the encoding pass */
+        int (*set_pass)(quicktime_t *file, 
+		int track, int pass, int total_passes,
+                              const char * stats_file);
+	int (*flush)(quicktime_t *file, 
+                     int track);
 
 	void *priv;
 
@@ -931,52 +1388,4 @@ typedef struct
         char *codec_name; /* Needed by libquicktime */
 } quicktime_codec_t;
 
-
-typedef struct 
-{
-  quicktime_codec_t codec;
-  char fourcc[5];
-
-  void(*init)(quicktime_video_map_t *);
-
-  int(*decode)(quicktime_t *file,
-	       int track,
-	       unsigned long inputsize,
-	       unsigned char *input,
-	       unsigned char *output);
-
-  int(*encode)(quicktime_t *file,
-	       int track,
-	       unsigned char *input,
-	       unsigned char *output);
-  
-  int(*delete_codec)(quicktime_video_map_t *vtrack);
-
-} quicktime_extern_video_t;
-
-
-
-
-typedef struct 
-{
-  quicktime_codec_t codec;
-  char fourcc[5];
-
-  void(*init)(quicktime_audio_map_t *);
-
-  int(*decode)(quicktime_t *file,
-	       int track,
-	       unsigned long inputsize,
-	       unsigned char *input,
-	       unsigned char *output);
-
-  int(*encode)(quicktime_t *file,
-	       int track,
-	       unsigned char *input,
-	       unsigned char *output);
-  
-  int(*delete_codec)(quicktime_audio_map_t *atrack);
-} quicktime_extern_audio_t;
-
-
 #endif
