@@ -12,11 +12,6 @@
 static char * fourccs_dv_ntsc[] = { "dvc ", (char*)0 };
 static char * fourccs_dv_pal[]  = { "dvcp", (char*)0 };
 
-static int encoding_colormodels_dv[] =
-  {
-    BC_YUV422,
-    LQT_COLORMODEL_NONE
-  };
 
 #if 0  
 static lqt_parameter_info_static_t decode_parameters_dv[] =
@@ -67,47 +62,42 @@ static lqt_parameter_info_static_t encode_parameters_dv[] =
       name:               "dv_anamorphic16x9",
       real_name:          "Is Anamorphic 16x9",
       type:               LQT_PARAMETER_INT,
-      val_default:        {0},
-      val_min:            0,
-      val_max:            1,
-      stringlist_options: (char**)0
+      val_default:        {val_int: 0},
+      val_min:            {val_int: 0},
+      val_max:            {val_int: 1},
     },
     { 
       name:               "dv_vlc_encode_passes",
       real_name:          "VLC Encode Passes",
       type:               LQT_PARAMETER_INT,
-      val_default:        {3},
-      val_min:            1,
-      val_max:            3,
-      stringlist_options: (char**)0
+      val_default:        {val_int: 3},
+      val_min:            {val_int: 1},
+      val_max:            {val_int: 3},
     },
-	{ 
+    { 
       name:               "dv_clamp_luma",
       real_name:          "Clamp Luma Values",
       type:               LQT_PARAMETER_INT,
-      val_default:        {0},
-      val_min:            0,
-      val_max:            1,
-      stringlist_options: (char**)0
-	},
-	{ 
+      val_default:        {val_int: 0},
+      val_min:            {val_int: 0},
+      val_max:            {val_int: 1},
+    },
+    { 
       name:               "dv_clamp_chroma",
       real_name:          "Clamp Chroma Values",
       type:               LQT_PARAMETER_INT,
-      val_default:        {0},
-      val_min:            0,
-      val_max:            1,
-      stringlist_options: (char**)0
-	},
-	{ 
+      val_default:        {val_int: 0},
+      val_min:            {val_int: 0},
+      val_max:            {val_int: 1},
+    },
+    { 
       name:               "dv_rem_ntsc_setup",
       real_name:          "Compensate for 7.5IRE NTSC setup",
       type:               LQT_PARAMETER_INT,
-      val_default:        {0},
-      val_min:            0,
-      val_max:            1,
-      stringlist_options: (char**)0
-	},
+      val_default:        {val_int: 0},
+      val_min:            {val_int: 0},
+      val_max:            {val_int: 1},
+    },
     { /* End of parameters */ }
   };
 
@@ -121,9 +111,8 @@ static lqt_codec_info_static_t codec_info_dv_pal =
     type:        LQT_CODEC_VIDEO,
     direction:   LQT_DIRECTION_ENCODE,
     encoding_parameters: encode_parameters_dv,
-    //    decoding_parameters: decode_parameters_dv,
-    encoding_colormodels: encoding_colormodels_dv,
-    decoding_colormodel:  BC_YUV422
+    compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
+
   };
 
 static lqt_codec_info_static_t codec_info_dv_ntsc =
@@ -136,9 +125,7 @@ static lqt_codec_info_static_t codec_info_dv_ntsc =
     type:        LQT_CODEC_VIDEO,
     direction:   LQT_DIRECTION_ENCODE,
     encoding_parameters: encode_parameters_dv,
-    //    decoding_parameters: decode_parameters_dv,
-    encoding_colormodels: encoding_colormodels_dv,
-    decoding_colormodel:  BC_YUV422
+    compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
   };
 
 
