@@ -31,6 +31,8 @@
 
 #include "ffmpeg.h"
 
+
+
 /* The following code was ported from gmerlin_avdecoder (http://gmerlin.sourceforge.net) */
 
 /* MPEG Audio header parsing code */
@@ -266,8 +268,12 @@ static int decode_header(mpeg_header * h, uint8_t * ptr, const mpeg_header * ref
 
 typedef struct
   {
-  quicktime_ffmpeg_codec_common_t com;
+  AVCodecContext * avctx;
+  AVCodec * encoder;
+  AVCodec * decoder;
 
+  int initialized;
+  
   /* Interleaved samples as avcodec needs them */
     
   int16_t * sample_buffer;
@@ -287,100 +293,97 @@ typedef struct
 
   mpeg_header mph;
   int have_mpeg_header;
+
+  uint8_t * extradata;
     
   } quicktime_ffmpeg_audio_codec_t;
 
 
-int lqt_ffmpeg_delete_audio(quicktime_audio_map_t *vtrack)
+static int lqt_ffmpeg_delete_audio(quicktime_audio_map_t *vtrack)
   {
   quicktime_ffmpeg_audio_codec_t *codec = ((quicktime_codec_t*)vtrack->codec)->priv;
   
-  if(codec->com.init_enc)
-    avcodec_close(codec->com.ffcodec_enc);
-  if(codec->com.init_dec)
-    avcodec_close(codec->com.ffcodec_dec);
-  
+  if(codec->avctx)
+    {
+    if(codec->initialized)
+      avcodec_close(codec->avctx);
+    else
+      free(codec->avctx);
+    }
   if(codec->sample_buffer) free(codec->sample_buffer);
   if(codec->chunk_buffer)  free(codec->chunk_buffer);
-  
+  if(codec->extradata) free(codec->extradata); 
   free(codec);
   return 0;
   }
 
-void quicktime_init_audio_codec_ffmpeg(quicktime_audio_map_t *atrack, AVCodec *encoder,
-                                       AVCodec *decoder)
-{
-	quicktime_ffmpeg_audio_codec_t *codec;
-
-	avcodec_init();
-        fprintf(stderr, "quicktime_init_audio_codec_ffmpeg 1\n");
-	codec = calloc(1, sizeof(quicktime_ffmpeg_audio_codec_t));
-	if(!codec)
-          return;
-
-	codec->com.ffc_enc = encoder;
-	codec->com.ffc_dec = decoder;
-	
-	((quicktime_codec_t*)atrack->codec)->priv = (void *)codec;
-	((quicktime_codec_t*)atrack->codec)->delete_acodec = lqt_ffmpeg_delete_audio;
-	if(encoder)
-          ((quicktime_codec_t*)atrack->codec)->encode_audio = lqt_ffmpeg_encode_audio;
-	if(decoder)
-          ((quicktime_codec_t*)atrack->codec)->decode_audio = lqt_ffmpeg_decode_audio;
-	((quicktime_codec_t*)atrack->codec)->set_parameter = lqt_ffmpeg_set_parameter_audio;
-
-}
-
-int lqt_ffmpeg_set_parameter_audio(quicktime_t *file, 
-                               int track, 
-                               char *key, 
-                               void *value)
-{
-	quicktime_ffmpeg_audio_codec_t *codec = ((quicktime_codec_t*)file->atracks[track].codec)->priv;
-
-        if(!strcasecmp(key, "bit_rate_audio"))
-          {
-          codec->com.params.bit_rate = (*(int *)value) * 1000;
-          return 0;
-          }
-        //	fprintf(stderr, "Unknown key: %s\n", key);
-          return -1;
-}
-
-
-static void deinterleave(int16_t ** dst_i, float ** dst_f, int16_t * src,
-                         int channels, int samples)
+static int set_parameter(quicktime_t *file, 
+                  int track, 
+                  char *key, 
+                  void *value)
   {
-  int i, j;
-  if(dst_f)
-    {
-    for(i = 0; i < channels; i++)
-      {
-      if(dst_f[i])
-        {
-        for(j = 0; j < samples; j++)
-          {
-          dst_f[i][j] = (float)src[j*channels + i]/32767.0;
-          }
-        }
-      }
-    }
-  if(dst_i)
-    {
-    for(i = 0; i < channels; i++)
-      {
-      if(dst_i[i])
-        {
-        for(j = 0; j < samples; j++)
-          {
-          dst_i[i][j] = src[j*channels + i];
-          }
-        }
-      }
-
-    }
+  quicktime_ffmpeg_audio_codec_t *codec = ((quicktime_codec_t*)file->atracks[track].codec)->priv;
+  lqt_ffmpeg_set_parameter(codec->avctx, key, value);
+  return 0;
   }
 
+
+/* Decode VBR chunk into the sample buffer */
+
+static int decode_chunk_vbr(quicktime_t * file, int track)
+  {
+  int chunk_packets, i, num_samples, bytes_decoded;
+  int packet_size, packet_samples;
+  int frame_bytes;
+  quicktime_audio_map_t *track_map = &(file->atracks[track]);
+  quicktime_ffmpeg_audio_codec_t *codec = ((quicktime_codec_t*)track_map->codec)->priv;
+
+  chunk_packets = lqt_audio_num_vbr_packets(file, track, track_map->current_chunk, &num_samples);
+
+  if(!chunk_packets)
+    return 0;
+  
+  if(codec->sample_buffer_alloc < codec->sample_buffer_end - codec->sample_buffer_start + num_samples)
+    {
+    
+    codec->sample_buffer_alloc = codec->sample_buffer_end - codec->sample_buffer_start + num_samples;
+    //    fprintf(stderr, "codec->sample_buffer_alloc: %d\n", codec->sample_buffer_alloc);
+    codec->sample_buffer = realloc(codec->sample_buffer, 2 * codec->sample_buffer_alloc *
+                                   track_map->channels);
+    }
+
+  for(i = 0; i < chunk_packets; i++)
+    {
+    packet_size = lqt_audio_read_vbr_packet(file, track, track_map->current_chunk, i,
+                                            &(codec->chunk_buffer), &(codec->chunk_buffer_alloc),
+                                            &packet_samples);
+    if(!packet_size)
+      return 0;
+    
+    frame_bytes =
+      avcodec_decode_audio(codec->avctx,
+                           &(codec->sample_buffer[track_map->channels *
+                                                  (codec->sample_buffer_end - codec->sample_buffer_start)]),
+                           &bytes_decoded,
+                           codec->chunk_buffer,
+                           packet_size + FF_INPUT_BUFFER_PADDING_SIZE);
+    if(frame_bytes < 0)
+      {
+      fprintf(stderr, "avcodec_decode_audio error\n");
+      break;
+      }
+#if 0
+    fprintf(stderr, "decode_chunk_vbr: Samples decoded: %d, Bytes used: %d\n",
+            bytes_decoded / (track_map->channels * 2),
+            frame_bytes);
+#endif
+    codec->sample_buffer_end += (bytes_decoded / (track_map->channels * 2));
+    }
+  track_map->current_chunk++;
+  return num_samples;
+  }
+
+
 /* Decode the current chunk into the sample buffer */
 
 static int decode_chunk(quicktime_t * file, int track)
@@ -403,17 +406,23 @@ static int decode_chunk(quicktime_t * file, int track)
                                       &(codec->chunk_buffer),
                                       &(codec->chunk_buffer_alloc),
                                       codec->bytes_in_chunk_buffer);
+
+  //  fprintf(stderr, "Got chunk:\n");
+  //  lqt_hexdump(codec->chunk_buffer, 16, 16);
+  
   if(!chunk_size)
     {
     //    fprintf(stderr, "audio_ffmpeg: EOF 1 (%d bytes left)\n", codec->bytes_in_chunk_buffer);
     /* If the codec is mp3, make sure to decode the very last frame */
 
-    if((codec->com.ffcodec_dec->codec_id == CODEC_ID_MP3) &&
+    if((codec->avctx->codec_id == CODEC_ID_MP3) &&
        (codec->bytes_in_chunk_buffer >= 4))
       {
       if(!decode_header(&mph, codec->chunk_buffer, (const mpeg_header*)0))
+        {
+        fprintf(stderr, "Decode header failed\n");
         return 0;
-
+        }
       if(mph.frame_bytes <= codec->bytes_in_chunk_buffer)
         {
         fprintf(stderr, "Huh, frame not decoded?\n");
@@ -493,7 +502,7 @@ static int decode_chunk(quicktime_t * file, int track)
     
     /* Some really broken mp3 files have the header bytes split across 2 chunks */
 
-    if(codec->com.ffcodec_dec->codec_id == CODEC_ID_MP3)
+    if(codec->avctx->codec_id == CODEC_ID_MP3)
       {
       if(codec->bytes_in_chunk_buffer < 4)
         {
@@ -559,7 +568,7 @@ static int decode_chunk(quicktime_t * file, int track)
      */
     
     frame_bytes =
-      avcodec_decode_audio(codec->com.ffcodec_dec,
+      avcodec_decode_audio(codec->avctx,
                            &(codec->sample_buffer[track_map->channels *
                                                   (codec->sample_buffer_end - codec->sample_buffer_start)]),
                            &bytes_decoded,
@@ -581,7 +590,7 @@ static int decode_chunk(quicktime_t * file, int track)
     
     if(bytes_decoded < 0)
       {
-      if(codec->com.ffcodec_dec->codec_id == CODEC_ID_MP3)
+      if(codec->avctx->codec_id == CODEC_ID_MP3)
         {
         /* For mp3, bytes_decoded < 0 means, that the frame should be muted */
         memset(&(codec->sample_buffer[track_map->channels * (codec->sample_buffer_end - codec->sample_buffer_start)]),
@@ -616,7 +625,7 @@ static int decode_chunk(quicktime_t * file, int track)
     if(codec->bytes_in_chunk_buffer < 0)
       codec->bytes_in_chunk_buffer = 0;
 
-    if(bytes_decoded <= 0)
+    if(bytes_decoded < 0)
       {
       if(codec->bytes_in_chunk_buffer > 0)
         codec->bytes_in_chunk_buffer = 0;
@@ -644,10 +653,11 @@ static int decode_chunk(quicktime_t * file, int track)
   return samples_decoded;
   }
 
-int lqt_ffmpeg_decode_audio(quicktime_t *file, int16_t **output_i,
-                            float **output_f, long samples,
-                            int track)
+static int lqt_ffmpeg_decode_audio(quicktime_t *file, void * output, long samples, int track)
   {
+  uint8_t * header;
+  uint32_t header_len;
+  
   int samples_decoded;
   //  int result = 0;
   int64_t chunk_sample; /* For seeking only */
@@ -659,22 +669,74 @@ int lqt_ffmpeg_decode_audio(quicktime_t *file, int16_t **output_i,
   int samples_to_skip;
   int samples_to_move;
 
-  //  fprintf(stderr, "ffmpeg decode audio %lld\n", track_map->current_position);
+  //  fprintf(stderr, "ffmpeg decode audio %lld %d\n", track_map->current_position, samples);
+
+  if(!output) /* Global initialization */
+    {
+    return 0;
+    }
   
   /* Initialize codec */
-  if(!codec->com.init_dec)
+  if(!codec->initialized)
     {
-    codec->com.ffcodec_dec = avcodec_alloc_context();
+
+    /* Set some mandatory variables */
+    codec->avctx->channels        = quicktime_track_channels(file, track);
+    codec->avctx->sample_rate     = quicktime_sample_rate(file, track);
+
+    if(track_map->track->mdia.minf.stbl.stsd.table[0].version == 1)
+      {
+      if(track_map->track->mdia.minf.stbl.stsd.table[0].audio_bytes_per_frame)
+        codec->avctx->block_align =
+          track_map->track->mdia.minf.stbl.stsd.table[0].audio_bytes_per_frame;
+      }
+    
+    //  priv->ctx->block_align     = s->data.audio.block_align;
+    //  priv->ctx->bit_rate        = s->codec_bitrate;
+
+    codec->avctx->bits_per_sample = quicktime_audio_bits(file, track);
+
+    /* Some codecs need extra stuff */
+
+    if(codec->decoder->id == CODEC_ID_ALAC)
+      {
+      header = quicktime_wave_get_user_atom(track_map->track, "alac", &header_len);
+      if(header)
+        {
+        codec->avctx->extradata = header;
+        codec->avctx->extradata_size = header_len;
+        }
+      }
+    if(codec->decoder->id == CODEC_ID_QDM2)
+      {
+      header = quicktime_wave_get_user_atom(track_map->track, "QDCA", &header_len);
+      if(header)
+        {
+        codec->extradata = malloc(header_len + 12);
+        /* frma atom */
+        codec->extradata[0] = 0x00;
+        codec->extradata[1] = 0x00;
+        codec->extradata[2] = 0x00;
+        codec->extradata[3] = 0x0C;
+        memcpy(codec->extradata + 4, "frmaQDM2", 8);
+        /* QDCA atom */
+        memcpy(codec->extradata + 12, header, header_len);
+        codec->avctx->extradata = codec->extradata;
+        codec->avctx->extradata_size = header_len + 12;
+        }
+
+      }
+
     //    memcpy(&(codec->com.ffcodec_enc), &(codec->com.params), sizeof(AVCodecContext));
     
-    if(avcodec_open(codec->com.ffcodec_dec, codec->com.ffc_dec) != 0)
+    if(avcodec_open(codec->avctx, codec->decoder) != 0)
       {
       fprintf(stderr, "Avcodec open failed\n");
       return 0;
       }
     
     //    codec->sample_buffer_offset = 0;
-    codec->com.init_dec = 1;
+    codec->initialized = 1;
     }
 
   /* Check if we have to reposition the stream */
@@ -685,10 +747,16 @@ int lqt_ffmpeg_decode_audio(quicktime_t *file, int16_t **output_i,
     if((track_map->current_position < codec->sample_buffer_start) || 
        (track_map->current_position + samples >= codec->sample_buffer_end))
       {
-      quicktime_chunk_of_sample(&chunk_sample,
+      if(lqt_audio_is_vbr(file, track))
+        lqt_chunk_of_sample_vbr(&chunk_sample,
                                 &(track_map->current_chunk),
                                 track_map->track,
                                 track_map->current_position);
+      else
+        quicktime_chunk_of_sample(&chunk_sample,
+                                  &(track_map->current_chunk),
+                                  track_map->track,
+                                  track_map->current_position);
 #if 0
       fprintf(stderr, "Seek: last_pos: %ld, pos: %lld, chunk: %lld, chunk_sample: %lld\n",
               track_map->last_position,
@@ -698,7 +766,11 @@ int lqt_ffmpeg_decode_audio(quicktime_t *file, int16_t **output_i,
       codec->sample_buffer_start = chunk_sample;
       codec->sample_buffer_end   = chunk_sample;
       codec->bytes_in_chunk_buffer = 0;
-      decode_chunk(file, track);
+
+      if(lqt_audio_is_vbr(file, track))
+        decode_chunk_vbr(file, track);
+      else
+        decode_chunk(file, track);
       }
     }
   
@@ -708,7 +780,7 @@ int lqt_ffmpeg_decode_audio(quicktime_t *file, int16_t **output_i,
     {
     samples_to_skip = track_map->current_position - codec->sample_buffer_start;
 #if 0
-    fprintf(stderr, "Flush samples_to_skip: %d, %d", samples_to_skip,
+    fprintf(stderr, "** Flush samples_to_skip: %d, %d\n", samples_to_skip,
             (int)(codec->sample_buffer_end - codec->sample_buffer_start));
 #endif
     if(samples_to_skip > (int)(codec->sample_buffer_end - codec->sample_buffer_start))
@@ -742,9 +814,17 @@ int lqt_ffmpeg_decode_audio(quicktime_t *file, int16_t **output_i,
     
     //    if(track_map->current_chunk >= track_map->track->mdia.minf.stbl.stco.total_entries)
     //      return 0;
-    
-    if(!decode_chunk(file, track))
-      break;
+
+    if(lqt_audio_is_vbr(file, track))
+      {
+      if(!decode_chunk_vbr(file, track))
+        break;
+      }
+    else
+      {
+      if(!decode_chunk(file, track))
+        break;
+      }
     }
   samples_decoded = codec->sample_buffer_end - codec->sample_buffer_start - samples_to_skip;
 
@@ -754,8 +834,12 @@ int lqt_ffmpeg_decode_audio(quicktime_t *file, int16_t **output_i,
   //  fprintf(stderr, "Samples_decoded: %d\n", samples_decoded);
   /* Deinterleave into the buffer */
   
-  deinterleave(output_i, output_f, codec->sample_buffer + (track_map->channels * samples_to_skip),
-               channels, samples_decoded);
+  
+  //  deinterleave(output_i, output_f, codec->sample_buffer + (track_map->channels * samples_to_skip),
+  //               channels, samples_decoded);
+
+  memcpy(output, codec->sample_buffer + (channels * samples_to_skip),
+         channels * samples_decoded * 2);
   
   track_map->last_position = track_map->current_position + samples_decoded;
 
@@ -766,36 +850,9 @@ int lqt_ffmpeg_decode_audio(quicktime_t *file, int16_t **output_i,
 /*
  *   Encoding part
  */
-        
-static void interleave(int16_t * sample_buffer, int16_t ** input_i, float ** input_f,
-                       int num_samples, int channels)
-  {
-  int i, j;
-  if(input_i)
-    {
-    for(i = 0; i < num_samples; i++)
-      {
-      for(j = 0; j < channels; j++)
-        *(sample_buffer++) = input_i[j][i];
-      }
-    }
-  else if(input_f)
-    {
-    for(i = 0; i < num_samples; i++)
-      {
-      for(j = 0; j < channels; j++)
-        *(sample_buffer++) = (int16_t)(input_f[j][i] * 16383.0);
-      }
-    }
-  }
 
-
-/*
-  Untested, but it should work...   
-*/
-
-int lqt_ffmpeg_encode_audio(quicktime_t *file, int16_t **input_i, float **input_f,
-                            int track, long samples)
+static int lqt_ffmpeg_encode_audio(quicktime_t *file, void * input,
+                            long samples, int track)
   {
   int result = -1;
   quicktime_audio_map_t *track_map = &(file->atracks[track]);
@@ -808,31 +865,27 @@ int lqt_ffmpeg_encode_audio(quicktime_t *file, int16_t **input_i, float **input_
   int samples_encoded;
   /* Initialize encoder */
     
-  if(!codec->com.init_enc)
+  if(!codec->initialized)
     {
-    codec->com.ffcodec_enc = avcodec_alloc_context();
-    codec->com.ffcodec_enc->sample_rate = trak->mdia.minf.stbl.stsd.table[0].sample_rate;
-    codec->com.ffcodec_enc->channels = channels;
+    codec->avctx->sample_rate = track_map->samplerate;
+    codec->avctx->channels = channels;
 
-#define SP(x) codec->com.ffcodec_enc->x = codec->com.params.x
-    SP(bit_rate);
-#undef SP
-    if(avcodec_open(codec->com.ffcodec_enc, codec->com.ffc_enc) != 0)
+    if(avcodec_open(codec->avctx, codec->encoder) != 0)
       {
       fprintf(stderr, "Avcodec open failed\n");
       return -1;
       }
     
-    codec->com.init_enc = 1;
+    codec->initialized = 1;
 
     /* One frame is: bitrate * frame_samples / (samplerate * 8) + 1024 */
 #if 0		// PATCH 1
-    codec->chunk_buffer_alloc = (codec->com.ffcodec_enc->bit_rate * codec->com.ffcodec_enc->frame_size /
-                                (codec->com.ffcodec_enc->sample_rate * 8)) + 1024;
+    codec->chunk_buffer_alloc = (codec->avctx->bit_rate * codec->avctx->frame_size /
+                                (codec->avctx->sample_rate * 8)) + 1024;
 #else
-    codec->chunk_buffer_alloc = ( codec->com.ffcodec_enc->frame_size
+    codec->chunk_buffer_alloc = ( codec->avctx->frame_size
 									* sizeof( int16_t )
-									* codec->com.ffcodec_enc->channels
+									* codec->avctx->channels
 								);
 #endif
     codec->chunk_buffer = malloc(codec->chunk_buffer_alloc);
@@ -850,31 +903,34 @@ int lqt_ffmpeg_encode_audio(quicktime_t *file, int16_t **input_i, float **input_
 
   /* Interleave */
 
-  interleave(&(codec->sample_buffer[codec->samples_in_buffer * channels]),
-             input_i, input_f, samples, channels);
+  //  interleave(&(codec->sample_buffer[codec->samples_in_buffer * channels]),
+  //             input_i, input_f, samples, channels);
 
+  memcpy(codec->sample_buffer + codec->samples_in_buffer * channels,
+         input, samples * channels * 2);
+  
   codec->samples_in_buffer += samples;
   
   /* Encode */
   
-  //  fprintf(stderr, "codec->samples_in_buffer: %d, codec->com.ffcodec_enc->frame_size %d\n",
-  //          codec->samples_in_buffer, codec->com.ffcodec_enc->frame_size);
-  while(codec->samples_in_buffer >= codec->com.ffcodec_enc->frame_size)
+  //  fprintf(stderr, "codec->samples_in_buffer: %d, codec->avctx->frame_size %d\n",
+  //          codec->samples_in_buffer, codec->avctx->frame_size);
+  while(codec->samples_in_buffer >= codec->avctx->frame_size)
     {
     //    fprintf(stderr, "avcodec_encode_audio %d...", samples_done);
     
-    frame_bytes = avcodec_encode_audio(codec->com.ffcodec_enc, codec->chunk_buffer,
+    frame_bytes = avcodec_encode_audio(codec->avctx, codec->chunk_buffer,
                                        codec->chunk_buffer_alloc,
                                        &(codec->sample_buffer[samples_done*channels]));
     if(frame_bytes > 0)
       {
       quicktime_write_chunk_header(file, trak, &chunk_atom);
 #if 0    // PATCH 3  
-      if(codec->com.ffcodec_enc->frame_size == 1)
+      if(codec->avctx->frame_size == 1)
         samples_encoded = codec->samples_in_buffer;
       else
 #endif
-        samples_encoded = codec->com.ffcodec_enc->frame_size;
+        samples_encoded = codec->avctx->frame_size;
 
       //      fprintf(stderr, "Done %d->%d\n", samples_encoded, frame_bytes);
 
@@ -895,3 +951,27 @@ int lqt_ffmpeg_encode_audio(quicktime_t *file, int16_t **input_i, float **input_
             codec->samples_in_buffer * sizeof(int16_t) * channels);
   return result;
   }
+
+void quicktime_init_audio_codec_ffmpeg(quicktime_audio_map_t *atrack, AVCodec *encoder,
+                                       AVCodec *decoder)
+{
+	quicktime_ffmpeg_audio_codec_t *codec;
+
+	avcodec_init();
+        fprintf(stderr, "quicktime_init_audio_codec_ffmpeg 1\n");
+	codec = calloc(1, sizeof(quicktime_ffmpeg_audio_codec_t));
+	if(!codec)
+          return;
+
+	codec->encoder = encoder;
+	codec->decoder = decoder;
+	codec-> avctx = avcodec_alloc_context();
+	((quicktime_codec_t*)atrack->codec)->priv = (void *)codec;
+	((quicktime_codec_t*)atrack->codec)->delete_acodec = lqt_ffmpeg_delete_audio;
+	if(encoder)
+          ((quicktime_codec_t*)atrack->codec)->encode_audio = lqt_ffmpeg_encode_audio;
+	if(decoder)
+          ((quicktime_codec_t*)atrack->codec)->decode_audio = lqt_ffmpeg_decode_audio;
+	((quicktime_codec_t*)atrack->codec)->set_parameter = set_parameter;
+        atrack->sample_format = LQT_SAMPLE_INT16;
+}
