@@ -30,325 +30,225 @@
 #include <quicktime/lqt_codecapi.h>
 #include <quicktime/colormodels.h>
 #include "ffmpeg.h"
+#include "params.h"
 
 #define MAX_FOURCCS 30
 #define MAX_WAV_IDS 4
 
 int ffmpeg_num_audio_codecs = -1;
 int ffmpeg_num_video_codecs = -1;
-
-
-static int encoding_colormodels_ffmpeg[] = {
-	BC_YUV420P, 
-	LQT_COLORMODEL_NONE
-};
-
-static int encoding_colormodels_dvc[] = {
-	BC_YUV411P, 
-	LQT_COLORMODEL_NONE
-};
-
-
-
-#define ENCODE_PARAM_AUDIO \
-	{\
-	  "bit_rate_audio",\
-          "Bit rate (kbps)",\
-	  LQT_PARAMETER_INT,\
-          { 128 },\
-          0,\
-          0,\
-          (char**)0\
-	}
-
-#define ENCODE_PARAM_VIDEO_GENERAL \
-        { \
-        "general", \
-        "General", \
-        LQT_PARAMETER_SECTION, \
-        }, \
-        {\
-          "flags_gray",\
-          "Gray scale only mode",\
-          LQT_PARAMETER_INT,\
-          { 0 },\
-          0,\
-          1,\
-          (char**)0\
-        }, \
-	{ \
-	"strict_std_compliance", \
-	"Standard compilance", \
-	LQT_PARAMETER_INT, \
-	{ 0 }, \
-	0, \
-	2, \
-	(char**)0 \
-	}, \
-        {  \
-        "aspect_ratio_info", \
-        "Aspect Ratio", \
-	LQT_PARAMETER_STRINGLIST, \
-        {val_string:  "Square" }, \
-        0, \
-        0, \
-        (char*[]){ "Square", "4:3", "16:9", (char*)0 } \
-        }
   
-#define ENCODE_PARAM_VIDEO_BITRATE \
-        { \
-        "bitrate", \
-        "Bitrate", \
-        LQT_PARAMETER_SECTION, \
-        }, \
-        {\
-	  "bit_rate_video",\
-          "Bit rate (kbps)",\
-	  LQT_PARAMETER_INT,\
-          { 800 },\
-          0,\
-          0,\
-          (char**)0\
-	},\
-        { \
-          "bit_rate_tolerance",\
-          "Bitrate Tolerance (kbps)\n",\
-          LQT_PARAMETER_INT,\
-          { 4000 } \
-        }, \
-        { \
-          "rc_min_rate",\
-          "Minimum bitrate (kbps)\n",\
-          LQT_PARAMETER_INT,\
-          { 0 } \
-        }, \
-        { \
-          "rc_max_rate",\
-          "Maximum bitrate (kbps)\n",\
-          LQT_PARAMETER_INT,\
-          { 0 } \
-        }, \
-        { \
-        "qcompress", \
-        "Qscale change between easy and hard scenes", \
-        LQT_PARAMETER_INT, \
-        { 50 }, \
-        0, \
-        100, \
-        (char**)0 \
-        }, \
-        { \
-        "qblur", \
-        "Qscale smoothing over time", \
-        LQT_PARAMETER_INT, \
-        { 50 }, \
-        0, \
-        100, \
-        (char**)0 \
-        }
-  
-#define ENCODE_PARAM_VIDEO_VBR \
-        { \
-        "vbr", \
-        "VBR Options", \
-        LQT_PARAMETER_SECTION, \
-        }, \
-        {\
-          "qscale", \
-          "VBR quantizer scale (0 = CBR)", \
-          LQT_PARAMETER_INT, \
-          { 0 }, \
-          0, \
-          31 \
-        }, \
-        {\
-          "qmin", \
-          "min quantiser scale (VBR)", \
-          LQT_PARAMETER_INT, \
-          { 2 }, \
-          0, \
-          31 \
-        }, \
-        {\
-          "qmax", \
-          "max quantiser scale (VBR)", \
-          LQT_PARAMETER_INT, \
-          { 31 }, \
-          0, \
-          31 \
-        }, \
-        {\
-          "mb_qmin", \
-          "min macroblock quantiser scale (VBR)", \
-          LQT_PARAMETER_INT, \
-          { 2 }, \
-          0, \
-          31 \
-        }, \
-        {\
-          "mb_qmax", \
-          "max macroblock quantiser scale (VBR)", \
-          LQT_PARAMETER_INT, \
-          { 31 }, \
-          0, \
-          31 \
-        }, \
-        {\
-          "max_qdiff", \
-          "max difference between the quantiser scale (VBR)", \
-          LQT_PARAMETER_INT, \
-          { 3 }, \
-          0, \
-          31 \
-        } \
+#define ENCODE_PARAM_AUDIO                     \
+  {                                            \
+    name:      "bit_rate_audio",                 \
+    real_name: "Bit rate (kbps)",                \
+    type:      LQT_PARAMETER_INT,                \
+    val_default: { val_int: 128 },               \
+  }
 
-#define ENCODE_PARAM_VIDEO_TEMPORAL \
-        { \
-        "temporal_compression", \
-        "Temporal Compression", \
-        LQT_PARAMETER_SECTION \
-        }, \
-	{\
-	"me_method",\
-	"Motion estimation method",\
-	LQT_PARAMETER_STRINGLIST,\
-	{val_string: "Zero"},\
-        0, \
-        0, \
-	((char *[]){"Zero", "Phods", "Log", "X1", "Epzs", "Full", (char *)0})\
-	},\
-	{\
-	"mb_decision",\
-	"MB decision mode",\
-	LQT_PARAMETER_STRINGLIST,\
-	{val_string: "Simple"},\
-        0, \
-        0, \
-	((char *[]){"Simple", "Fewest bits", "Rate distoration", (char *)0})\
-	},\
-        { \
-        "gop_size", \
-        "GOP size (0 = intra only)", \
-        LQT_PARAMETER_INT, \
-        { val_int: 250 }, \
-        0, \
-        300, \
-        (char**)0 \
-        } \
- 
-#define ENCODE_PARAM_VIDEO_MPEG4 \
-        { \
-        "mpeg4", \
-        "MPEG-4 Options", \
-        LQT_PARAMETER_SECTION \
-        }, \
-	{\
-	"flags_4mv",\
-	"Use four motion vector by macroblock",\
-	LQT_PARAMETER_INT,\
-	{ 0 },\
-	0,\
-	1,\
-	(char**)0\
-	}, \
-        { \
-        "flags_part", \
-        "Data partitioning mode", \
-        LQT_PARAMETER_INT, \
-        { 0 }, \
-        0, \
-        1, \
-        (char**)0 \
-        }
+#define ENCODE_PARAM_VIDEO_RATECONTROL \
+  {                                           \
+    name:      "rate_control",                       \
+    real_name: "Rate control",                     \
+    type:      LQT_PARAMETER_SECTION,         \
+  },                                        \
+    PARAM_BITRATE_VIDEO,                    \
+    PARAM_BITRATE_TOLERANCE,                \
+    PARAM_RC_MIN_RATE,                      \
+    PARAM_RC_MAX_RATE,                      \
+    PARAM_RC_BUFFER_SIZE,                   \
+    PARAM_RC_INITIAL_COMPLEX,               \
+    PARAM_RC_INITIAL_BUFFER_OCCUPANCY
+
+#define ENCODE_PARAM_VIDEO_QUANTIZER_I \
+  {                                           \
+    name:      "quantizer",                       \
+    real_name: "Quantizer",                     \
+    type:      LQT_PARAMETER_SECTION,         \
+  },                                        \
+    PARAM_QMIN,                             \
+    PARAM_QMAX,                             \
+    PARAM_MAX_QDIFF,                        \
+    PARAM_FLAG_QSCALE,                      \
+    PARAM_QSCALE,                      \
+    PARAM_QCOMPRESS,                        \
+    PARAM_QBLUR,                            \
+    PARAM_QUANTIZER_NOISE_SHAPING,          \
+    PARAM_FLAG_TRELLIS_QUANT
+
+#define ENCODE_PARAM_VIDEO_QUANTIZER_IP \
+  ENCODE_PARAM_VIDEO_QUANTIZER_I,               \
+  PARAM_I_QUANT_FACTOR,                         \
+  PARAM_I_QUANT_OFFSET
+
+#define ENCODE_PARAM_VIDEO_QUANTIZER_IPB \
+  ENCODE_PARAM_VIDEO_QUANTIZER_IP,        \
+  PARAM_B_QUANT_FACTOR,                   \
+  PARAM_B_QUANT_OFFSET
+
+#define ENCODE_PARAM_VIDEO_FRAMETYPES_IP \
+  {                                           \
+    name:      "frame_types",                       \
+    real_name: "Frame types",                     \
+    type:      LQT_PARAMETER_SECTION,         \
+  },                                        \
+  PARAM_GOP_SIZE,                      \
+  PARAM_SCENE_CHANGE_THESHOLD,       \
+  PARAM_SCENECHANGE_FACTOR,        \
+  PARAM_FLAG_CLOSED_GOP,         \
+  PARAM_FLAG2_STRICT_GOP
+
+#define ENCODE_PARAM_VIDEO_FRAMETYPES_IPB \
+  ENCODE_PARAM_VIDEO_FRAMETYPES_IP,        \
+  PARAM_MAX_B_FRAMES,                 \
+  PARAM_B_FRAME_STRATEGY
+
+#define ENCODE_PARAM_VIDEO_ME \
+  {                                           \
+    name:      "motion_estimation",                       \
+    real_name: "Motion estimation",                     \
+    type:      LQT_PARAMETER_SECTION,         \
+  },                                        \
+    PARAM_ME_METHOD,                        \
+    PARAM_ME_CMP,\
+    PARAM_ME_CMP_CHROMA,\
+    PARAM_ME_RANGE,\
+    PARAM_ME_THRESHOLD,\
+    PARAM_MB_DECISION,\
+    PARAM_DIA_SIZE
+
+#define ENCODE_PARAM_VIDEO_ME_PRE \
+  {                                           \
+    name:      "motion_estimation",                       \
+    real_name: "ME pre-pass",                     \
+    type:      LQT_PARAMETER_SECTION,         \
+  },                                        \
+    PARAM_PRE_ME,\
+    PARAM_ME_PRE_CMP,\
+    PARAM_ME_PRE_CMP_CHROMA,\
+    PARAM_PRE_DIA_SIZE
+
+#define ENCODE_PARAM_VIDEO_QPEL                 \
+  {                                           \
+    name:      "qpel_motion_estimation",                       \
+    real_name: "Qpel ME",                     \
+    type:      LQT_PARAMETER_SECTION,         \
+  },                                        \
+    PARAM_FLAG_QPEL, \
+    PARAM_ME_SUB_CMP,\
+    PARAM_ME_SUB_CMP_CHROMA,\
+    PARAM_ME_SUBPEL_QUALITY
 
 
-#define ENCODE_PARAM_VIDEO_H263P \
-        { \
-        "h263p", \
-        "H263+ Options", \
-        LQT_PARAMETER_SECTION \
-        }, \
-        { \
-        "flags_h263p_aic", \
-        "Advanced intra coding", \
-        LQT_PARAMETER_INT, \
-        { 0 }, \
-        0, \
-        1 \
-        }, \
-        { \
-        "flags_h263p_umv", \
-        "Unlimited Motion Vector", \
-        LQT_PARAMETER_INT, \
-        { 0 }, \
-        0, \
-        1 \
-        }
+#define ENCODE_PARAM_VIDEO_MASKING \
+  {                                \
+    name:      "masking",                       \
+    real_name: "Masking",                     \
+    type:      LQT_PARAMETER_SECTION,         \
+  },                                        \
+    PARAM_LUMI_MASKING, \
+    PARAM_DARK_MASKING, \
+    PARAM_TEMPORAL_CPLX_MASKING, \
+    PARAM_SPATIAL_CPLX_MASKING, \
+    PARAM_BORDER_MASKING, \
+    PARAM_P_MASKING,                                       \
+    PARAM_FLAG_NORMALIZE_AQP
+
+#define ENCODE_PARAM_VIDEO_MISC \
+  {                                           \
+    name:      "misc",                       \
+    real_name: "Misc",                     \
+    type:      LQT_PARAMETER_SECTION,         \
+  },                                        \
+    PARAM_STRICT_STANDARD_COMPLIANCE,       \
+    PARAM_NOISE_REDUCTION, \
+    PARAM_FLAG_GRAY, \
+    PARAM_FLAG_BITEXACT
+    
+
 
 #define DECODE_PARAM_AUDIO
 
 #define DECODE_PARAM_VIDEO \
-        {\
-	  "workaround_bugs",\
-	  "Enable bug worarounds",\
-	  LQT_PARAMETER_INT,\
-	  { 1 },\
-	  0,\
-	  1,\
-	  (char**)0\
-	},\
-        {\
-          "flags_gray",\
-          "Gray scale only mode",\
-          LQT_PARAMETER_INT,\
-          { 0 },\
-          0,\
-          1,\
-          (char**)0\
-        }, \
-        { \
-          "error_resilience", \
-          "Error resilience", \
-          LQT_PARAMETER_STRINGLIST, \
-          { val_string: "Careful" }, \
-          0, \
-          0, \
-          (char *[]){"None", "Careful", "Compilant", "Agressive", "Very Agressive", (char *)0 } \
-        }
-
-static lqt_parameter_info_static_t encode_parameters_video[] = {
-  ENCODE_PARAM_VIDEO_GENERAL,
-  ENCODE_PARAM_VIDEO_BITRATE,
-  ENCODE_PARAM_VIDEO_VBR,
-  { /* End of parameters */ }
-};
-
-static lqt_parameter_info_static_t encode_parameters_mpegvideo[] = {
-  ENCODE_PARAM_VIDEO_GENERAL,
-  ENCODE_PARAM_VIDEO_BITRATE,
-  ENCODE_PARAM_VIDEO_VBR,
-  ENCODE_PARAM_VIDEO_TEMPORAL,
-  { /* End of parameters */ }
-};
+  PARAM_FLAG_GRAY
 
 static lqt_parameter_info_static_t encode_parameters_mpeg4[] = {
-  ENCODE_PARAM_VIDEO_GENERAL,
-  ENCODE_PARAM_VIDEO_BITRATE,
-  ENCODE_PARAM_VIDEO_VBR,
-  ENCODE_PARAM_VIDEO_TEMPORAL,
-  ENCODE_PARAM_VIDEO_MPEG4,
+  ENCODE_PARAM_VIDEO_FRAMETYPES_IPB,
+  ENCODE_PARAM_VIDEO_RATECONTROL,
+  ENCODE_PARAM_VIDEO_QUANTIZER_IPB,
+  ENCODE_PARAM_VIDEO_ME,
+  PARAM_FLAG_GMC,
+  PARAM_FLAG_4MV,
+  ENCODE_PARAM_VIDEO_ME_PRE,
+  ENCODE_PARAM_VIDEO_QPEL,
+  ENCODE_PARAM_VIDEO_MASKING,
+  ENCODE_PARAM_VIDEO_MISC,
+  { /* End of parameters */ }
+};
+
+static lqt_parameter_info_static_t encode_parameters_dx50[] = {
+  ENCODE_PARAM_VIDEO_FRAMETYPES_IP,
+  ENCODE_PARAM_VIDEO_RATECONTROL,
+  ENCODE_PARAM_VIDEO_QUANTIZER_IP,
+  ENCODE_PARAM_VIDEO_ME,
+  ENCODE_PARAM_VIDEO_ME_PRE,
+  ENCODE_PARAM_VIDEO_MASKING,
+  ENCODE_PARAM_VIDEO_MISC,
+  { /* End of parameters */ }
+};
+
+
+static lqt_parameter_info_static_t encode_parameters_h263[] = {
+  ENCODE_PARAM_VIDEO_FRAMETYPES_IP,
+  ENCODE_PARAM_VIDEO_RATECONTROL,
+  ENCODE_PARAM_VIDEO_QUANTIZER_IP,
+  ENCODE_PARAM_VIDEO_ME,
+  PARAM_FLAG_4MV,
+  ENCODE_PARAM_VIDEO_ME_PRE,
+  ENCODE_PARAM_VIDEO_MASKING,
+  ENCODE_PARAM_VIDEO_MISC,
+  PARAM_FLAG_OBMC,
   { /* End of parameters */ }
 };
 
 static lqt_parameter_info_static_t encode_parameters_h263p[] = {
-  ENCODE_PARAM_VIDEO_GENERAL,
-  ENCODE_PARAM_VIDEO_BITRATE,
-  ENCODE_PARAM_VIDEO_VBR,
-  ENCODE_PARAM_VIDEO_TEMPORAL,
-  ENCODE_PARAM_VIDEO_H263P,
+  ENCODE_PARAM_VIDEO_FRAMETYPES_IP,
+  ENCODE_PARAM_VIDEO_RATECONTROL,
+  ENCODE_PARAM_VIDEO_QUANTIZER_IP,
+  ENCODE_PARAM_VIDEO_ME,
+  PARAM_FLAG_4MV,
+  ENCODE_PARAM_VIDEO_ME_PRE,
+  ENCODE_PARAM_VIDEO_MASKING,
+  ENCODE_PARAM_VIDEO_MISC,
+  PARAM_FLAG_H263P_AIV,
+  PARAM_FLAG_OBMC,
   { /* End of parameters */ }
 };
 
+static lqt_parameter_info_static_t encode_parameters_msmpeg4v3[] = {
+  ENCODE_PARAM_VIDEO_FRAMETYPES_IP,
+  ENCODE_PARAM_VIDEO_RATECONTROL,
+  ENCODE_PARAM_VIDEO_QUANTIZER_IP,
+  ENCODE_PARAM_VIDEO_ME,
+  ENCODE_PARAM_VIDEO_ME_PRE,
+  ENCODE_PARAM_VIDEO_MASKING,
+  ENCODE_PARAM_VIDEO_MISC,
+  { /* End of parameters */ }
+};
+
+
+
+static lqt_parameter_info_static_t encode_parameters_dvvideo[] = {
+  { /* End of parameters */ }
+};
+
+static lqt_parameter_info_static_t encode_parameters_mjpeg[] = {
+  ENCODE_PARAM_VIDEO_RATECONTROL,
+  ENCODE_PARAM_VIDEO_QUANTIZER_I,
+  ENCODE_PARAM_VIDEO_MASKING,
+  ENCODE_PARAM_VIDEO_MISC,
+  { /* End of parameters */ }
+};
 
 static lqt_parameter_info_static_t encode_parameters_audio[] = {
   ENCODE_PARAM_AUDIO,
@@ -360,19 +260,7 @@ static lqt_parameter_info_static_t decode_parameters_video[] = {
   { /* End of parameters */ }
 };
 
-static lqt_parameter_info_static_t decode_parameters_mpeg4[] = {
-  DECODE_PARAM_VIDEO,
-  { /* End of parameters */ }
-};
-#if 0
-static lqt_parameter_info_static_t decode_parameters_rle[] = {
-  DECODE_PARAM_VIDEO,
-  { /* End of parameters */ }
-};
-#endif
-
 static lqt_parameter_info_static_t decode_parameters_audio[] = {
-  //  DECODE_PARAM_AUDIO,
   { /* End of parameters */ }
 };
 
@@ -387,7 +275,7 @@ struct CODECIDMAP {
 	char *name;
 	char *fourccs[MAX_FOURCCS];
         int   wav_ids[MAX_WAV_IDS];
-        int * encoding_colormodels;
+        int compatibility_flags;
 /*
  *  We explicitely allow, if encoding is allowed.
  *  This prevents the spreading of broken files
@@ -403,58 +291,65 @@ struct CODECIDMAP codecidmap_v[] = {
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          //          encode_parameters: encode_parameters_mpegvideo,
           decode_parameters: decode_parameters_video,
 	  short_name: "mpg1",
 	  name: "Mpeg 1 Video",
-	  fourccs: {"mpg1", "MPG1", "pim1", "PIM1", (char *)0} },
+	  fourccs: {"mpg1", "MPG1", "pim1", "PIM1", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE }
+
+        },
         {
           id: CODEC_ID_MPEG4,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
           encode_parameters: encode_parameters_mpeg4,
-          decode_parameters: decode_parameters_mpeg4,
+          decode_parameters: decode_parameters_video,
 	  short_name: "mpg4",
-	  name: "Mpeg 4 Video (DivX)",
+	  name: "MPEG-4",
 	  fourccs: {"mp4v", "divx", "DIV1", "div1", "MP4S", "mp4s", "M4S2",
                     "m4s2", "xvid", "XVID", "XviD", "DX50", "dx50", "DIVX",
                     "MP4V", (char *)0 },
-          encoding_colormodels: encoding_colormodels_ffmpeg,
-          do_encode: 1 },
+          wav_ids: { LQT_WAV_ID_NONE },
+          compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT | LQT_FILE_MP4,
+          do_encode: 1
+        },
 	{
           id: CODEC_ID_MSMPEG4V1,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          //          encode_parameters: encode_parameters_mpeg4,
-          decode_parameters: decode_parameters_mpeg4,
+          decode_parameters: decode_parameters_video,
 	  short_name: "msmpeg4v1",
 	  name: "MSMpeg 4v1",
-	  fourccs: {"DIV1", "div1", "MPG4", "mpg4", (char *)0} },
+	  fourccs: {"DIV1", "div1", "MPG4", "mpg4", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+        },
 	{
           id: CODEC_ID_MSMPEG4V2,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          //          encode_parameters: encode_parameters_mpeg4,
-          decode_parameters: decode_parameters_mpeg4,
+          decode_parameters: decode_parameters_video,
 	  short_name: "msmpeg4v2",
 	  name: "MSMpeg 4v2",
-	  fourccs: {"DIV2", "div2", "MP42", "mp42", (char *)0} },
+	  fourccs: {"DIV2", "div2", "MP42", "mp42", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+        },
 	{
           id: CODEC_ID_MSMPEG4V3,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_mpeg4,
-          decode_parameters: decode_parameters_mpeg4,
+          encode_parameters: encode_parameters_msmpeg4v3,
+          decode_parameters: decode_parameters_video,
 	  short_name: "msmpeg4v3",
-	  name: "MSMpeg 4v3",
+	  name: "MSMpeg 4v3 (DivX 3 compatible)",
 	  fourccs: {"DIV3", "mpg3", "MP43", "mp43", "DIV5", "div5", "DIV6",
                     "MPG3", "div6", "div3", "DIV4", "div4", "AP41", "ap41",
                     (char *)0},
-          encoding_colormodels: encoding_colormodels_ffmpeg,
+          wav_ids: { LQT_WAV_ID_NONE },
+          compatibility_flags: LQT_FILE_AVI,
           do_encode: 1,
         },
 #if 0
@@ -467,21 +362,36 @@ struct CODECIDMAP codecidmap_v[] = {
           decode_parameters: decode_parameters_video,
 	  short_name: "wmv1",
 	  name: "WMV1",
-	  fourccs: {"WMV1", "wmv1", (char *)0} },
+	  fourccs: {"WMV1", "wmv1", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+        },
 #endif
 	{
           id: CODEC_ID_H263,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_mpegvideo,
+          encode_parameters: encode_parameters_h263,
           decode_parameters: decode_parameters_video,
 	  short_name: "h263",
 	  name: "H263",
 	  fourccs: {"H263", "h263", "U263", "u263", (char *)0},
-          encoding_colormodels: encoding_colormodels_ffmpeg,
-          do_encode: 1, },
+          wav_ids: { LQT_WAV_ID_NONE },
+          compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
+          do_encode: 1,
+        },
 	{
+          id: CODEC_ID_H264,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          decode_parameters: decode_parameters_video,
+	  short_name: "h264",
+	  name: "H264",
+	  fourccs: {"avc1", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+        },
+        {
           id: CODEC_ID_H263P,
 	  index: -1,
           encoder: NULL,
@@ -491,168 +401,197 @@ struct CODECIDMAP codecidmap_v[] = {
 	  short_name: "h263p",
 	  name: "H263+",
 	  fourccs: {"U263", "u263", (char *)0},
-          encoding_colormodels: encoding_colormodels_ffmpeg,
-          do_encode: 1, },
+          wav_ids: { LQT_WAV_ID_NONE },
+          compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
+          do_encode: 1,
+        },
 	{
           id: CODEC_ID_H263I,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          //          encode_parameters: encode_parameters_mpegvideo,
           decode_parameters: decode_parameters_video,
 	  short_name: "i263",
 	  name: "I263",
-          encoding_colormodels: encoding_colormodels_ffmpeg,
 	  fourccs: {"I263", "i263", "viv1", "VIV1", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
         },
-#if 0
-	{
-          id: CODEC_ID_RV10,
-	  index: -1,
-          encoder: NULL,
-          decoder: NULL,
-          encode_parameters: encode_parameters_mpegvideo,
-          decode_parameters: decode_parameters_video,
-	  short_name: "rv10",
-	  name: "Real Video 10",
-	  fourccs: {"RV10", "rv10", "RV13", "rv13", (char *)0} },
-#endif
 	{
           id: CODEC_ID_SVQ1,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          //          encode_parameters: encode_parameters_mpegvideo,
           decode_parameters: decode_parameters_video,
 	  short_name: "svq1",
 	  name: "Sorenson Video 1",
-	  fourccs: {"SVQ1", (char *)0} },
+	  fourccs: {"SVQ1", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+        },
 	{
           id: CODEC_ID_SVQ3,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          //          encode_parameters: encode_parameters_mpegvideo,
           decode_parameters: decode_parameters_video,
 	  short_name: "svq3",
 	  name: "Sorenson Video 3",
-	  fourccs: {"SVQ3", (char *)0} },
+	  fourccs: {"SVQ3", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+        },
 	{
           id: CODEC_ID_MJPEG,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_video,
+          encode_parameters: encode_parameters_mjpeg,
           decode_parameters: decode_parameters_video,
 	  short_name: "mjpg",
 	  name: "MJPEG",
 	  fourccs: {"MJPG", "mjpg", "JPEG", "jpeg", "dmb1", "AVDJ", (char *)0},
-          encoding_colormodels: encoding_colormodels_ffmpeg,
-          do_encode: 1, },
+          wav_ids: { LQT_WAV_ID_NONE },
+          compatibility_flags: LQT_FILE_AVI,
+          do_encode: 1,
+        },
 	{
           id: CODEC_ID_8BPS,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          //          encode_parameters: encode_parameters_video,
-          decode_parameters: decode_parameters_video,
 	  short_name: "8BPS",
 	  name: "Quicktime Planar RGB (8BPS)",
-	  fourccs: {"8BPS", (char *)0} },
-	{
+	  fourccs: {"8BPS", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+        },
+        {
           id: CODEC_ID_INDEO3,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          //          encode_parameters: encode_parameters_video,
           decode_parameters: decode_parameters_video,
 	  short_name: "indeo",
 	  name: "Intel Indeo 3",
-	  fourccs: {"IV31", "IV32", (char *)0} },
-	{
+	  fourccs: {"IV31", "IV32", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+        },
+        {
           id: CODEC_ID_RPZA,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          //          encode_parameters: encode_parameters_video,
-          decode_parameters: decode_parameters_video,
 	  short_name: "rpza",
 	  name: "Apple Video",
-	  fourccs: {"rpza", (char *)0} },
+	  fourccs: {"rpza", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+        },
 	{
           id: CODEC_ID_SMC,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          //          encode_parameters: encode_parameters_video,
-          decode_parameters: decode_parameters_video,
 	  short_name: "smc",
 	  name: "Apple Graphics",
-	  fourccs: {"smc ", (char *)0} },
-	{
+	  fourccs: {"smc ", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+        },
+        {
           id: CODEC_ID_CINEPAK,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          //          encode_parameters: encode_parameters_video,
           decode_parameters: decode_parameters_video,
 	  short_name: "cinepak",
 	  name: "Cinepak",
-	  fourccs: {"cvid", (char *)0} },
+	  fourccs: {"cvid", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+        },
 	{
           id: CODEC_ID_CYUV,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          //          encode_parameters: encode_parameters_video,
           decode_parameters: decode_parameters_video,
 	  short_name: "cyuv",
 	  name: "Creative YUV",
-	  fourccs: {"CYUV", (char *)0} },
+	  fourccs: {"CYUV", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+        },
 	{
           id: CODEC_ID_QTRLE,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          //          encode_parameters: encode_parameters_video,
-          decode_parameters: decode_parameters_video,
 	  short_name: "rle",
 	  name: "RLE",
-	  fourccs: {"rle ", (char *)0} },
+	  fourccs: {"rle ", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+        },
         {
           id: CODEC_ID_MSRLE,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          //          encode_parameters: encode_parameters_video,
-          decode_parameters: decode_parameters_video,
 	  short_name: "wrle",
 	  name: "Microsoft RLE",
-	  fourccs: {"WRLE", (char *)0} },
+	  fourccs: {"WRLE", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+        },
         {
           id: CODEC_ID_DVVIDEO,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_video,
+          encode_parameters: encode_parameters_dvvideo,
           decode_parameters: decode_parameters_video,
 	  short_name: "dv_ntsc",
 	  name: "DV (NTSC)",
 	  fourccs: {"dvc ", (char *)0 },
-          encoding_colormodels: encoding_colormodels_dvc,
-          do_encode: 1 },
+          wav_ids: { LQT_WAV_ID_NONE },
+          compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
+          do_encode: 1,
+
+        },
         {
           id: CODEC_ID_DVVIDEO,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_video,
+          encode_parameters: encode_parameters_dvvideo,
           decode_parameters: decode_parameters_video,
 	  short_name: "dv_pal",
 	  name: "DV (PAL)",
 	  fourccs: {"dvcp", "dvpp", (char *)0 },
-          encoding_colormodels: encoding_colormodels_ffmpeg,
-          do_encode: 1 },
+          wav_ids: { LQT_WAV_ID_NONE },
+          compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
+          do_encode: 1
+        },
+        {
+          id: CODEC_ID_DVVIDEO,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          encode_parameters: encode_parameters_dvvideo,
+          decode_parameters: decode_parameters_video,
+	  short_name: "dv50_pal",
+	  name: "DVCPRO50 (PAL)",
+	  fourccs: {"dv5p", (char *)0 },
+          wav_ids: { LQT_WAV_ID_NONE },
+          compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
+          do_encode: 1
+        },
+        {
+          id: CODEC_ID_DVVIDEO,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          encode_parameters: encode_parameters_dvvideo,
+          decode_parameters: decode_parameters_video,
+	  short_name: "dv50_ntsc",
+	  name: "DVCPRO50 (NTSC)",
+	  fourccs: {"dv5n", (char *)0 },
+          wav_ids: { LQT_WAV_ID_NONE },
+          compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
+          do_encode: 1
+        },
 };
 
 struct CODECIDMAP codecidmap_a[] = {
@@ -677,8 +616,9 @@ struct CODECIDMAP codecidmap_a[] = {
           decode_parameters: decode_parameters_audio,
 	  short_name: "mp2",
 	  name: "Mpeg Layer 2 Audio",
-	  fourccs: {".mp2", (char *)0 },
+	  fourccs: {".mp2", ".MP2", (char *)0 },
           wav_ids: { 0x55, LQT_WAV_ID_NONE },
+          compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT | LQT_FILE_AVI,
           do_encode: 1,
         },
 	{
@@ -692,9 +632,35 @@ struct CODECIDMAP codecidmap_a[] = {
 	  name: "AC3 Audio",
 	  fourccs: {".ac3", ".AC3", (char *)0},
           wav_ids: { 0x2000, LQT_WAV_ID_NONE },
+          compatibility_flags: LQT_FILE_AVI,
           do_encode: 1,
         },
-#if 0 /* Sounds ugly */
+        {
+          id: CODEC_ID_QDM2,
+          index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          encode_parameters: encode_parameters_audio,
+          decode_parameters: decode_parameters_audio,
+          short_name: "qdm2",
+          name: "QDM2 Audio",
+          fourccs: {"QDM2", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+          //          do_encode: 1,
+        },
+#if 1 /* Doesn't work as long as audio chunks are not split into VBR "Samples" */
+	{
+          id: CODEC_ID_ALAC,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          short_name: "alac",
+	  name: "Apple lossless",
+	  fourccs: { "alac", (char*)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+        },
+#endif
+#if 1 /* Sounds ugly */
 	{
           id: CODEC_ID_ADPCM_MS,
 	  index: -1,
@@ -706,7 +672,7 @@ struct CODECIDMAP codecidmap_a[] = {
           wav_ids: { 0x02, LQT_WAV_ID_NONE },
         },
 #endif
-#if 0 /* Sounds ugly */
+#if 1 /* Sounds ugly */
         {
           id: CODEC_ID_ADPCM_IMA_WAV,
 	  index: -1,
@@ -719,16 +685,6 @@ struct CODECIDMAP codecidmap_a[] = {
           
         },
 #endif
-	{
-          id: CODEC_ID_PCM_ALAW,
-	  index: -1,
-          encoder: NULL,
-          decoder: NULL,
-          short_name: "alaw",
-	  name: "Alaw",
-	  fourccs: { "alaw", (char*)0},
-          wav_ids: { 0x06, LQT_WAV_ID_NONE },
-        },
 };
 
 
@@ -806,7 +762,6 @@ static lqt_codec_info_static_t codec_info_ffmpeg = {
 	direction:   0,
 	encoding_parameters: NULL,
 	decoding_parameters: NULL,
-	decoding_colormodel:  LQT_COLORMODEL_NONE,
 };
 
 /* These are called from the plugin loader */
@@ -824,22 +779,22 @@ static void set_codec_info(struct CODECIDMAP * map)
 
   codec_info_ffmpeg.fourccs = map->fourccs;
   codec_info_ffmpeg.wav_ids = map->wav_ids;
-
+  //  fprintf(stderr, "Short name: %s, wav_ids: %p\n", map->short_name, map->wav_ids);
   if(map->encoder && map->decoder)
     {
     codec_info_ffmpeg.direction = LQT_DIRECTION_BOTH;
     codec_info_ffmpeg.encoding_parameters = map->encode_parameters;
     codec_info_ffmpeg.decoding_parameters = map->decode_parameters;
-    codec_info_ffmpeg.encoding_colormodels = map->encoding_colormodels;
     //    capabilities = "Codec";
+    codec_info_ffmpeg.compatibility_flags = map->compatibility_flags;
     }
   else if(map->encoder)
     {
     codec_info_ffmpeg.direction = LQT_DIRECTION_ENCODE;
     codec_info_ffmpeg.encoding_parameters = map->encode_parameters;
     codec_info_ffmpeg.decoding_parameters = NULL;
-    codec_info_ffmpeg.encoding_colormodels = map->encoding_colormodels;
-      
+    codec_info_ffmpeg.compatibility_flags = map->compatibility_flags;
+
     //    capabilities = "Encoder";
     }
   else if(map->decoder)
@@ -888,7 +843,7 @@ extern lqt_codec_info_static_t * get_codec_info(int index)
 	struct CODECIDMAP * map;
         ffmpeg_map_init();
 	map = find_codec(index);
-
+        //        memset(&codec_info_ffmpeg, 0, sizeof(codec_info_ffmpeg));
         if(map)
           {
           set_codec_info(map);
