@@ -98,7 +98,12 @@ typedef struct
   {
   mpeg_version_t version;
   int layer;
-  int bitrate;    /* -1: VBR */
+
+  int bitrate_mode;
+  int bitrate;
+  
+  int min_bitrate; /* ABR only */
+  int max_bitrate; /* ABR only  */
   int samplerate;
   int frame_bytes;
   int channel_mode;
@@ -239,11 +244,8 @@ typedef struct
   lame_global_flags *lame_global;
   //        mpeg3_layer_t *encoded_header;
   int encode_initialized;
-  float **input;
   int input_size;
   int input_allocated;
-  int bitrate;
-  int quality;
   
   unsigned char *encoder_output;
   int encoder_output_alloc;
@@ -252,10 +254,19 @@ typedef struct
   int samples_per_frame;
   int stereo;
 
-  /* Floating point input must be scaled to +/- 32768 */
-  
-  float * input_buffer[2];
+  int16_t * input_buffer[2];
   int input_buffer_alloc;
+
+  int64_t samples_read;    /* samples passed to lame_encode_buffer */
+  int64_t samples_written; /* samples written to the file          */
+
+  /* Configuration stuff */
+  int bitrate_mode;
+  int bitrate;
+  int bitrate_min;
+  int bitrate_max;
+  int quality;
+  int quality_vbr;
   
   } quicktime_mp3_codec_t;
 
@@ -278,16 +289,6 @@ static int delete_codec(quicktime_audio_map_t *atrack)
     {
     lame_close(codec->lame_global);
     }
-  if(codec->input)
-    {
-    int i;
-    for(i = 0; i < atrack->channels; i++)
-      {
-      free(codec->input[i]);
-      }
-    free(codec->input);
-    }
-
   if(codec->input_buffer[0])
     free(codec->input_buffer[0]);
   if(codec->input_buffer[1])
@@ -301,20 +302,21 @@ static int delete_codec(quicktime_audio_map_t *atrack)
   return 0;
   }
 
-static int write_data(quicktime_t *file, quicktime_audio_map_t *track_map,
-                       quicktime_mp3_codec_t *codec)
+
+static int write_data(quicktime_t *file, int track,
+                      quicktime_mp3_codec_t *codec, int samples)
   {
   mpeg_header h;
   quicktime_atom_t chunk_atom;
   int result = 0;
   int chunk_bytes = 0, chunk_samples = 0;
-
+  quicktime_audio_map_t *track_map;
   uint8_t * chunk_ptr;
 
+  track_map = &file->atracks[track];
+  
   memset(&h, 0, sizeof(h));
   
-  /* We write each mp3 frame into an own chunk. This increases the chance,
-     that M$ software will be able to play our files */
   chunk_ptr = codec->encoder_output;
   while(codec->encoder_output_size > 4)
     {
@@ -329,6 +331,7 @@ static int write_data(quicktime_t *file, quicktime_audio_map_t *track_map,
       chunk_ptr += h.frame_bytes;
       chunk_bytes += h.frame_bytes;
       chunk_samples += h.samples_per_frame;
+      //      fprintf(stderr, "Created frame %d bytes\n", h.frame_bytes);
       }
     else
       break;
@@ -336,13 +339,51 @@ static int write_data(quicktime_t *file, quicktime_audio_map_t *track_map,
 
   if(chunk_ptr > codec->encoder_output)
     {
-    quicktime_write_chunk_header(file, track_map->track, &chunk_atom);
-    result = !quicktime_write_data(file, codec->encoder_output, chunk_bytes);
-    quicktime_write_chunk_footer(file, 
-                                 track_map->track, 
-                                 track_map->current_chunk,
-                                 &chunk_atom,
-                                 chunk_samples);
+    if(track_map->track->strl)
+      {
+      quicktime_write_chunk_header(file, track_map->track, &chunk_atom);
+      result = !quicktime_write_data(file, codec->encoder_output, chunk_bytes);
+      quicktime_write_chunk_footer(file, 
+                                   track_map->track, 
+                                   track_map->current_chunk,
+                                   &chunk_atom,
+                                   chunk_samples);
+      if(file->total_riffs == 1)
+        {
+        track_map->track->strl->strh.dwLength += chunk_bytes;
+        }
+      }
+    else
+      {
+      lqt_start_audio_vbr_chunk(file, track);
+      quicktime_write_chunk_header(file, track_map->track, &chunk_atom);
+
+      lqt_start_audio_vbr_frame(file, track);
+      result = !quicktime_write_data(file, codec->encoder_output, chunk_bytes);
+
+    
+      if(samples < 0)
+        {
+        lqt_finish_audio_vbr_frame(file, track, chunk_samples);
+        quicktime_write_chunk_footer(file, 
+                                     track_map->track, 
+                                     track_map->current_chunk,
+                                     &chunk_atom,
+                                     track_map->vbr_num_frames);
+        codec->samples_written += chunk_samples;
+        }
+      else
+        {
+        lqt_finish_audio_vbr_frame(file, track, samples);
+        quicktime_write_chunk_footer(file, 
+                                     track_map->track, 
+                                     track_map->current_chunk,
+                                     &chunk_atom,
+                                     track_map->vbr_num_frames);
+        codec->samples_written += samples;
+        }
+      }
+    
     track_map->current_chunk++;
 
     /* Adjust mp3 buffer */
@@ -351,26 +392,43 @@ static int write_data(quicktime_t *file, quicktime_audio_map_t *track_map,
       memmove(codec->encoder_output, chunk_ptr, codec->encoder_output_size);
     
 #if 0
-    fprintf(stderr, "Encoded %d samples, %d bytes %d remaining\n",
+    fprintf(stderr, "Encoded %d samples %d bytes %d remaining\n",
             chunk_samples, chunk_bytes, codec->encoder_output_size);
 #endif   
-
-
     }
   
   
   return result;
   }
 
+#define PUT_16_LE(num, ptr) \
+ptr[0] = num & 0xff; \
+ptr[1] = (num >> 8) & 0xff;\
+ptr+=2;
 
+#define PUT_32_LE(num, ptr) \
+ptr[0] = num & 0xff; \
+ptr[1] = (num >> 8) & 0xff;\
+ptr[2] = (num >> 16) & 0xff;\
+ptr[3] = (num >> 24) & 0xff;\
+ptr+=4;
+
+/* nanosoft style mp3 definitions */
+
+#define MPEGLAYER3_ID_UNKNOWN            0
+#define MPEGLAYER3_ID_MPEG               1
+#define MPEGLAYER3_ID_CONSTANTFRAMESIZE  2
+
+#define MPEGLAYER3_FLAG_PADDING_ISO      0x00000000
+#define MPEGLAYER3_FLAG_PADDING_ON       0x00000001
+#define MPEGLAYER3_FLAG_PADDING_OFF      0x00000002
 
 static int encode(quicktime_t *file, 
-                  int16_t **input_i, 
-                  float **input_f, 
-                  int track, 
-                  long samples)
+                  void * _input, 
+                  long samples,
+                  int track)
   {
-
+  int16_t * input;
   int result = 0;
   int encoded_size;
   quicktime_audio_map_t *track_map = &(file->atracks[track]);
@@ -378,18 +436,40 @@ static int encode(quicktime_t *file,
   quicktime_mp3_codec_t *codec = ((quicktime_codec_t*)track_map->codec)->priv;
   int i;
 
+  uint8_t extradata[12];
+  uint8_t * extradata_ptr;
+  uint32_t tmp;
+  int mpeg_version;
+  
   if(!codec->encode_initialized)
     {
+    if(!trak->strl)
+      lqt_init_vbr_audio(file, track);
+
     codec->encode_initialized = 1;
     codec->lame_global = lame_init();
-    lame_set_brate(codec->lame_global, codec->bitrate / 1000);
-    lame_set_VBR(codec->lame_global, vbr_off);
-    lame_set_quality(codec->lame_global, codec->quality);
-    lame_set_in_samplerate(codec->lame_global, 
-                           trak->mdia.minf.stbl.stsd.table[0].sample_rate);
-    lame_set_out_samplerate(codec->lame_global, 
-                            trak->mdia.minf.stbl.stsd.table[0].sample_rate);
 
+    if(trak->strl || (codec->bitrate_mode == vbr_off))
+      {
+      lame_set_VBR(codec->lame_global, vbr_off);
+      lame_set_brate(codec->lame_global, codec->bitrate / 1000);
+      }
+    else if(codec->bitrate_mode == vbr_default)
+      {
+      lame_set_VBR(codec->lame_global, vbr_default);
+      lame_set_VBR_q(codec->lame_global, codec->quality_vbr);
+      }
+    else if(codec->bitrate_mode == vbr_abr)
+      {
+      lame_set_VBR(codec->lame_global, vbr_abr);
+      lame_set_VBR_min_bitrate_kbps(codec->lame_global, codec->bitrate_min/1000);
+      lame_set_VBR_max_bitrate_kbps(codec->lame_global, codec->bitrate_max/1000);
+      }
+    
+    lame_set_quality(codec->lame_global, codec->quality);
+    lame_set_in_samplerate(codec->lame_global, track_map->samplerate);
+    lame_set_out_samplerate(codec->lame_global, track_map->samplerate);
+    
     lame_set_bWriteVbrTag(codec->lame_global, 0);
     
     /* Also support Mono streams */
@@ -409,15 +489,49 @@ static int encode(quicktime_t *file,
     if(trak->strl)
       {
       /* strh stuff */
-      trak->strl->dwRate = codec->bitrate / 8;
-      trak->strl->dwScale = 1;
-      trak->strl->dwSampleSize = 1;
-      
+      trak->strl->strh.dwRate = codec->bitrate / 8;
+      trak->strl->strh.dwScale = 1;
+      trak->strl->strh.dwSampleSize = 1;
       /* WAVEFORMATEX stuff */
       
-      trak->strl->nBlockAlign = 1;
-      trak->strl->nAvgBytesPerSec =  codec->bitrate / 8;
-      trak->strl->wBitsPerSample = 0;
+      trak->strl->strf.wf.f.WAVEFORMAT.nBlockAlign = 1;
+      trak->strl->strf.wf.f.WAVEFORMAT.nAvgBytesPerSec =  codec->bitrate / 8;
+      trak->strl->strf.wf.f.PCMWAVEFORMAT.wBitsPerSample = 0;
+
+      /* Extradata (completely unneccesary for decoding) */
+
+      mpeg_version = lame_get_version(codec->lame_global);
+      
+      extradata_ptr = extradata;
+
+      tmp = MPEGLAYER3_ID_MPEG; // WORD          wID
+      PUT_16_LE(tmp, extradata_ptr);
+
+      tmp = MPEGLAYER3_FLAG_PADDING_ISO; // DWORD         fdwFlags;
+      PUT_32_LE(tmp, extradata_ptr);
+      
+      switch(mpeg_version)
+        {
+        case 0: // MPEG-2
+          tmp = ((144000 / 2) * (codec->bitrate/1000)) / track_map->samplerate; // WORD nBlockSize;
+          break;
+        case 1: // MPEG-1
+          tmp = ((144000 / 1) * (codec->bitrate/1000)) / track_map->samplerate; // WORD nBlockSize;
+          break;
+        case 2: // MPEG-2.5
+          tmp = ((144000 / 4) * (codec->bitrate/1000)) / track_map->samplerate; // WORD nBlockSize;
+          break;
+        }
+      
+      PUT_16_LE(tmp, extradata_ptr);
+
+      tmp = 1; // WORD nFramesPerBlock;
+      PUT_16_LE(tmp, extradata_ptr);
+
+      tmp = 1393; //  WORD nCodecDelay;
+      PUT_16_LE(tmp, extradata_ptr);
+      
+      quicktime_strf_set_audio_extradata(&trak->strl->strf, extradata, 12);
       }
     }
 
@@ -430,49 +544,46 @@ static int encode(quicktime_t *file,
     codec->encoder_output_alloc = encoded_size + codec->encoder_output_size + 16;
     codec->encoder_output       = realloc(codec->encoder_output, codec->encoder_output_alloc);
     }
-  if(input_f)
+
+  if(codec->input_buffer_alloc < samples)
     {
-    if(codec->input_buffer_alloc < samples)
-      {
-      codec->input_buffer_alloc = samples + 16;
-      codec->input_buffer[0] =
-        realloc(codec->input_buffer[0], codec->input_buffer_alloc * sizeof(float));
-      
-      if(codec->stereo)
-        codec->input_buffer[1] =
-          realloc(codec->input_buffer[1], codec->input_buffer_alloc * sizeof(float));
-      }
-
-    for(i = 0; i < samples; i++)
-      codec->input_buffer[0][i] = input_f[0][i] * 32767.0;
-
+    codec->input_buffer_alloc = samples + 16;
+    codec->input_buffer[0] =
+      realloc(codec->input_buffer[0], codec->input_buffer_alloc * sizeof(*codec->input_buffer[0]));
+    
     if(codec->stereo)
-      {
-      for(i = 0; i < samples; i++)
-        codec->input_buffer[1][i] = input_f[1][i] * 32767.0;
-      }
-
-    result = lame_encode_buffer_float(codec->lame_global,
-                                      codec->input_buffer[0],
-                                      codec->stereo ? codec->input_buffer[1] : codec->input_buffer[0],
-                                      samples,
-                                      codec->encoder_output + codec->encoder_output_size,
-                                      codec->encoder_output_alloc - codec->encoder_output_size);
+      codec->input_buffer[1] =
+        realloc(codec->input_buffer[1], codec->input_buffer_alloc * sizeof(*codec->input_buffer[1]));
     }
-  else if(input_i)
+
+  input = (int16_t*)_input;
+    
+  if(codec->stereo)
     {
-    result = lame_encode_buffer(codec->lame_global,
-                                input_i[0],
-                                codec->stereo ? input_i[1] : input_i[0],
-                                samples,
-                                codec->encoder_output + codec->encoder_output_size,
-                                codec->encoder_output_alloc - codec->encoder_output_size);
+    for(i = 0; i < samples; i++)
+      {
+      codec->input_buffer[0][i] = *(input++);
+      codec->input_buffer[1][i] = *(input++);
+      }
     }
-
+  else
+    {
+    for(i = 0; i < samples; i++)
+      codec->input_buffer[0][i] = *(input++);
+    }
+  
+  result = lame_encode_buffer(codec->lame_global,
+                              codec->input_buffer[0],
+                              codec->stereo ? codec->input_buffer[1] : codec->input_buffer[0],
+                              samples,
+                              codec->encoder_output + codec->encoder_output_size,
+                              codec->encoder_output_alloc - codec->encoder_output_size);
+  codec->samples_read += samples;
+    
   if(result > 0)
     {
     codec->encoder_output_size += result;
-    result = write_data(file, track_map, codec);
+    result = write_data(file, track, codec, -1);
     }
   
   return result;
@@ -486,16 +597,33 @@ static int set_parameter(quicktime_t *file, int track,
   quicktime_audio_map_t *atrack = &(file->atracks[track]);
   quicktime_mp3_codec_t *codec =
     ((quicktime_codec_t*)atrack->codec)->priv;
-
-  if(!strcasecmp(key, "mp3_bitrate"))
+  
+  if(!strcasecmp(key, "mp3_bitrate_mode"))
+    {
+    if(!strcmp((char*)value, "CBR"))
+      codec->bitrate_mode = vbr_off;
+    else if(strcmp((char*)value, "ABR"))
+      codec->bitrate_mode = vbr_abr;
+    else if(strcmp((char*)value, "VBR"))
+      codec->bitrate_mode = vbr_default;
+    }
+  else if(!strcasecmp(key, "mp3_bitrate"))
     codec->bitrate = *(int*)value;
+  else if(!strcasecmp(key, "mp3_bitrate_min"))
+    codec->bitrate_min = *(int*)value;
+  else if(!strcasecmp(key, "mp3_bitrate_max"))
+    codec->bitrate_max = *(int*)value;
   else if(!strcasecmp(key, "mp3_quality"))
     codec->quality = *(int*)value;
-
+  else if(!strcasecmp(key, "mp3_quality_vbr"))
+    codec->quality_vbr = *(int*)value;
+  else
+    fprintf(stderr, "set_parameter: %s\n", key);
+  
   return 0;
   }
 
-static void flush(quicktime_t *file, int track)
+static int flush(quicktime_t *file, int track)
   {
   int result = 0;
   quicktime_audio_map_t *track_map = &(file->atracks[track]);
@@ -505,17 +633,21 @@ static void flush(quicktime_t *file, int track)
     {
     //    samples_left = lame_get_mf_samples_to_encode(codec->lame_global);
     
-    result = lame_encode_flush_nogap(codec->lame_global,
-                                     codec->encoder_output + codec->encoder_output_size, 
-                                     codec->encoder_output_alloc);
+    result = lame_encode_flush(codec->lame_global,
+                               codec->encoder_output + codec->encoder_output_size, 
+                               codec->encoder_output_alloc);
     /* Check if more frames arrived */
 
     if(result > 0)
       {
+      //      fprintf(stderr, "Flush: %d\n", result);
       codec->encoder_output_size += result;
-      write_data(file, track_map, codec);
+      write_data(file, track, codec, codec->samples_read - codec->samples_written);
+      return 1;
       }
     }
+  return 0;
+  //  fprintf(stderr, "Samples read: %lld, Samples written: %lld\n", codec->samples_read, codec->samples_written);
   }
 
 void quicktime_init_codec_lame(quicktime_audio_map_t *atrack)
@@ -533,4 +665,5 @@ void quicktime_init_codec_lame(quicktime_audio_map_t *atrack)
   codec = ((quicktime_codec_t*)atrack->codec)->priv;
   codec->bitrate = 256000;
   codec->quality = 0;
+  atrack->sample_format = LQT_SAMPLE_INT16;
   }
