@@ -72,23 +72,6 @@ static int reads_colormodel(quicktime_t *file,
 	return (colormodel == BC_YUV420P);
 }
 #endif
-static int writes_colormodel(quicktime_t *file, 
-		int colormodel, 
-		int track)
-{
-	return (colormodel == BC_RGB888 ||
-		colormodel == BC_RGBA8888 ||
-		colormodel == BC_RGB161616 ||
-		colormodel == BC_RGBA16161616 ||
-		colormodel == BC_YUV888 ||
-		colormodel == BC_YUVA8888 ||
-		colormodel == BC_YUV161616 ||
-		colormodel == BC_YUVA16161616 ||
-		colormodel == BC_YUV420P ||
-		colormodel == BC_YUV422 ||
-		colormodel == BC_COMPRESSED);
-}
-
 
 
 
@@ -452,6 +435,7 @@ int quicktime_divx_write_vol(unsigned char *data_start,
 static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
 {
 //printf("encode 1\n");
+        int i;
 	quicktime_video_map_t *vtrack = &(file->vtracks[track]);
 	quicktime_divx_codec_t *codec = ((quicktime_codec_t*)vtrack->codec)->priv;
 	quicktime_trak_t *trak = vtrack->track;
@@ -464,7 +448,14 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
 	ENC_FRAME encore_input;
 	ENC_RESULT encore_result;
 //printf("encode 1 %d %d\n", width_i, height_i);
-
+        int row_span, row_span_uv;
+        
+        if(!row_pointers)
+          {
+          file->vtracks[track].stream_cmodel = BC_YUV420P;
+          return 0;
+          }
+        
 	init_mutex();
 	pthread_mutex_lock(&encode_mutex);
 
@@ -492,49 +483,32 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
 //printf("encode 1\n");
 
 // Assume planes are contiguous
-	if(file->vtracks[track].color_model == BC_YUV420P &&
-		width == width_i &&
-		height == height_i)
-	{
-//printf("encode 1.1\n");
-		encore_input.image = row_pointers[0];
-	}
-// Convert to YUV420P
-	else
-	{
-		if(!codec->temp_frame)
-		{
-			codec->temp_frame = malloc(width_i * height_i * 3 / 2);
-		}
+        if(!codec->temp_frame)
+          {
+          codec->temp_frame = malloc((width_i * height_i * 3) / 2);
+          }
+        
+        row_span = file->vtracks[track].stream_row_span ? file->vtracks[track].stream_row_span : width;
+        row_span_uv = file->vtracks[track].stream_row_span_uv ? file->vtracks[track].stream_row_span_uv : row_span/2;
+        
+        for(i = 0; i < height; i++)
+          {
+          memcpy(codec->temp_frame + width_i * i, row_pointers[0] + row_span * i, width);
+          }
+        
+        for(i = 0; i < height/2; i++)
+          {
+          //          fprintf(stderr, "i: %d (%d)\n", i, row_span_uv);
+          memcpy(codec->temp_frame +      width_i * height_i      + i * width_i/2, row_pointers[1] + row_span_uv * i, width/2);
+          memcpy(codec->temp_frame + 5 * (width_i * height_i) / 4 + i * width_i/2, row_pointers[2] + row_span_uv * i, width/2);
+          }
+        
 //printf("encode 2 %d %d %d %d %d %d\n", file->color_model, width, height, width_i, height_i);
 		
 //printf("encode 1.5\n");
-		cmodel_transfer(0, /* Leave NULL if non existent */
-			row_pointers,
-			codec->temp_frame, /* Leave NULL if non existent */
-			codec->temp_frame + width_i * height_i,
-			codec->temp_frame + width_i * height_i + width_i * height_i / 4,
-			row_pointers[0], /* Leave NULL if non existent */
-			row_pointers[1],
-			row_pointers[2],
-			0,        /* Dimensions to capture from input frame */
-			0, 
-			width, 
-			height,
-			0,       /* Dimensions to project on output frame */
-			0, 
-			width, 
-			height,
-			file->vtracks[track].color_model, 
-			BC_YUV420P,
-			0,         /* When transfering BC_RGBA8888 to non-alpha this is the background color in 0xRRGGBB hex */
-			width,       /* For planar use the luma rowspan */
-			width_i);
-		
-//printf("encode 2\n");
 	
-		encore_input.image = codec->temp_frame;
-	}
+        encore_input.image = codec->temp_frame;
+	
 //printf("encode 3\n");
 
 	if(!codec->work_buffer)
@@ -633,7 +607,6 @@ void quicktime_init_codec_divx(quicktime_video_map_t *vtrack)
 	((quicktime_codec_t*)vtrack->codec)->priv = calloc(1, sizeof(quicktime_divx_codec_t));
 	((quicktime_codec_t*)vtrack->codec)->delete_vcodec = delete_codec;
 	((quicktime_codec_t*)vtrack->codec)->encode_video = encode;
-	((quicktime_codec_t*)vtrack->codec)->writes_colormodel = writes_colormodel;
 	((quicktime_codec_t*)vtrack->codec)->set_parameter = set_parameter;
 	
 	codec = ((quicktime_codec_t*)vtrack->codec)->priv;
