@@ -1,6 +1,23 @@
 #include "funcprotos.h"
+
+#include <quicktime/lqt.h>
 #include <quicktime/colormodels.h>
-#include "qtpng.h"
+
+#include <png.h>
+
+typedef struct
+{
+	int compression_level;
+	unsigned char *buffer;
+// Read position
+	long buffer_position;
+// Frame size
+	long buffer_size;
+// Buffer allocation
+	long buffer_allocated;
+	unsigned char *temp_frame;
+} quicktime_png_codec_t;
+
 
 static int delete_codec(quicktime_video_map_t *vtrack)
 {
@@ -69,43 +86,18 @@ static void flush_function(png_structp png_ptr)
 static int decode(quicktime_t *file, unsigned char **row_pointers, int track)
 {
 	int result = 0;
-	int64_t i;
 	quicktime_video_map_t *vtrack = &(file->vtracks[track]);
-	quicktime_trak_t *trak = vtrack->track;
 	png_structp png_ptr;
 	png_infop info_ptr;
 	png_infop end_info = 0;	
-	int height = trak->tkhd.track_height;
-	int width = trak->tkhd.track_width;
 	quicktime_png_codec_t *codec = ((quicktime_codec_t*)vtrack->codec)->priv;
-	int cmodel = source_cmodel(file, track);
-	int use_temp = (cmodel != file->vtracks[track].color_model ||
-		file->in_x != 0 ||
-		file->in_y != 0 ||
-		file->in_w != width ||
-		file->in_h != height ||
-		file->out_w != width ||
-		file->out_h != height);
-	unsigned char **temp_rows = malloc(sizeof(unsigned char*) * height);
 
-	if(use_temp)
-	{
-		if(!codec->temp_frame)
-		{
-			codec->temp_frame = malloc(cmodel_calculate_datasize(width, 
-					height, 
-					-1, 
-					cmodel));
-		}
-		for(i = 0; i < height; i++)
-			temp_rows[i] = codec->temp_frame + 
-				cmodel_calculate_pixelsize(cmodel) * width * i;
-	}
-	else
-	{
-		for(i = 0; i < height; i++)
-			temp_rows[i] = row_pointers[i];
-	}
+        if(!row_pointers)
+          {
+          vtrack->stream_cmodel = source_cmodel(file, track);
+          fprintf(stderr, "Detected stream_cmodel: %s\n", lqt_colormodel_to_string(vtrack->stream_cmodel));
+          return 0;
+          }
 
 	quicktime_set_video_position(file, vtrack->current_position, track);
 	codec->buffer_size = quicktime_frame_size(file, vtrack->current_position, track);
@@ -126,37 +118,10 @@ static int decode(quicktime_t *file, unsigned char **row_pointers, int track)
 		png_read_info(png_ptr, info_ptr);
 
 /* read the image */
-		png_read_image(png_ptr, temp_rows);
+		png_read_image(png_ptr, row_pointers);
 		png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
 	}
 
-	if(use_temp)
-	{
-		cmodel_transfer(row_pointers, 
-			temp_rows,
-			row_pointers[0],
-			row_pointers[1],
-			row_pointers[2],
-			0,
-			0,
-			0,
-			file->in_x, 
-			file->in_y, 
-			file->in_w, 
-			file->in_h,
-			0, 
-			0, 
-			file->out_w, 
-			file->out_h,
-			cmodel, 
-			file->vtracks[track].color_model,
-			0,
-			width,
-			file->out_w);
-	}
-
-
-	free(temp_rows);
 
 	return result;
 }
@@ -171,9 +136,17 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
 	int width = trak->tkhd.track_width;
 	png_structp png_ptr;
 	png_infop info_ptr;
-	int cmodel = source_cmodel(file, track);
         quicktime_atom_t chunk_atom;
 
+        if(!row_pointers)
+          {
+          vtrack->stream_cmodel = source_cmodel(file, track);
+          fprintf(stderr, "Detected stream_cmodel: %s, %d\n",
+                  lqt_colormodel_to_string(vtrack->stream_cmodel),
+                  quicktime_video_depth(file, track));
+          return 0;
+          }
+        
 	codec->buffer_size = 0;
 	codec->buffer_position = 0;
 
@@ -188,15 +161,21 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
 		info_ptr, 
 		width, height,
     	8, 
-		cmodel == BC_RGB888 ? 
+		vtrack->stream_cmodel == BC_RGB888 ? 
 		  PNG_COLOR_TYPE_RGB : 
 		  PNG_COLOR_TYPE_RGB_ALPHA, 
 		PNG_INTERLACE_NONE, 
 		PNG_COMPRESSION_TYPE_DEFAULT, 
 		PNG_FILTER_TYPE_DEFAULT);
-	png_write_info(png_ptr, info_ptr);
+
+#if 0
+        png_write_info(png_ptr, info_ptr);
 	png_write_image(png_ptr, row_pointers);
 	png_write_end(png_ptr, info_ptr);
+#else
+        png_set_rows(png_ptr, info_ptr, row_pointers);
+        png_write_png(png_ptr, info_ptr, PNG_TRANSFORM_IDENTITY, NULL);
+#endif
 	png_destroy_write_struct(&png_ptr, &info_ptr);
 
         quicktime_write_chunk_header(file, trak, &chunk_atom);
@@ -226,22 +205,6 @@ static int set_parameter(quicktime_t *file,
   return 0;
   }
 
-static int reads_colormodel(quicktime_t *file, 
-                            int colormodel, 
-                            int track)
-  {
-  return (colormodel == source_cmodel(file, track));
-  }
-
-static int writes_colormodel(quicktime_t *file, 
-                             int colormodel, 
-                             int track)
-  {
-  return ((colormodel == BC_RGBA8888) || (colormodel == BC_RGB888));
-  }
-
-
-
 void quicktime_init_codec_png(quicktime_video_map_t *vtrack)
 {
 	quicktime_png_codec_t *codec;
@@ -255,10 +218,16 @@ void quicktime_init_codec_png(quicktime_video_map_t *vtrack)
 	((quicktime_codec_t*)vtrack->codec)->decode_audio = 0;
 	((quicktime_codec_t*)vtrack->codec)->encode_audio = 0;
 
-	((quicktime_codec_t*)vtrack->codec)->reads_colormodel = reads_colormodel;
-	((quicktime_codec_t*)vtrack->codec)->writes_colormodel = writes_colormodel;
 
 /* Init private items */
 	codec = ((quicktime_codec_t*)vtrack->codec)->priv;
 	codec->compression_level = 9;
 }
+
+void quicktime_init_codec_pngalpha(quicktime_video_map_t *vtrack)
+  {
+  /* Set depth to 32 */
+  vtrack->track->mdia.minf.stbl.stsd.table[0].depth = 32;
+  /* Proceed as normal */
+  quicktime_init_codec_png(vtrack);
+  }
