@@ -27,47 +27,32 @@
 
 static char * fourccs_rtjpeg[]  = { "RTJ0", (char*)0 };
 
-static int encoding_colormodels_rtjpeg[] = {
-	BC_YUV420P, 
-	LQT_COLORMODEL_NONE
-};
-
 static lqt_parameter_info_static_t encode_parameters_rtjpeg[] = {
 	{
-		"rtjpeg_quality",
-		"Quality setting",
-		LQT_PARAMETER_INT,
-		{ 100 },
-		0,
-		100,
-		(char**)0
+          name:        "rtjpeg_quality",
+          real_name:   "Quality setting",
+	  type:        LQT_PARAMETER_INT,
+	  val_default: { val_int: 100 },
+          val_min: { val_int: 0 },
+	  val_max: { val_int: 100 },
 	},
 	{
-		"rtjpeg_key_rate",
-		"Key frame interval",
-		LQT_PARAMETER_INT,
-		{ 25 },
-		0,
-		0,
-		(char**)0
+          name:        "rtjpeg_key_rate",
+          real_name:   "Key frame interval",
+          type:        LQT_PARAMETER_INT,
+          val_default: { val_int: 25 },
 	},
 	{
-		"rtjpeg_luma_quant",
-		"Luma quantiser",
-		LQT_PARAMETER_INT,
-		{ 1 },
-		0,
-		0,
-		(char**)0
+          name:        "rtjpeg_luma_quant",
+          real_name:   "Luma quantiser",
+          type:        LQT_PARAMETER_INT,
+          val_default: { val_int: 1 },
 	},
 	{
-		"rtjpeg_chroma_quant",
-		"Chroma quantiser",
-		LQT_PARAMETER_INT,
-		{ 1 },
-		0,
-		0,
-		(char**)0
+          name:        "rtjpeg_chroma_quant",
+          real_name:   "Chroma quantiser",
+          type:        LQT_PARAMETER_INT,
+          val_default: { val_int: 1 },
 	},
 	{ /* End of parameters */ }
 };
@@ -81,8 +66,7 @@ static lqt_codec_info_static_t codec_info_rtjpeg = {
 	direction:   LQT_DIRECTION_BOTH,
 	encoding_parameters: encode_parameters_rtjpeg,
 	decoding_parameters: (lqt_parameter_info_static_t*)0,
-	encoding_colormodels: encoding_colormodels_rtjpeg,
-	decoding_colormodel: BC_YUV420P
+        compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
 };
 
 /* These are called from the plugin loader */
@@ -107,11 +91,3 @@ extern lqt_init_video_codec_func_t get_video_codec(int index)
 	return (lqt_init_video_codec_func_t)0;
 }
 
-/* Not used if we have only one colormodel */
-#if 0
-int get_stream_colormodel(quicktime_t * file, int track, int codec_index,
-                          int * exact)
-{
-	return BC_YUV420P;
-}
-#endif
