@@ -1,10 +1,13 @@
 #include "config.h"
 #include <quicktime/lqt.h>
 #include <quicktime/lqt_codecapi.h>
-#include "raw.h"
 
 #include <quicktime/colormodels.h>
 
+void quicktime_init_codec_raw(quicktime_video_map_t *vtrack);
+void quicktime_init_codec_rawalpha(quicktime_video_map_t *vtrack);
+
+void quicktime_init_codec_v210(quicktime_video_map_t *vtrack);
 void quicktime_init_codec_v308(quicktime_video_map_t *vtrack);
 void quicktime_init_codec_v408(quicktime_video_map_t *vtrack);
 void quicktime_init_codec_v410(quicktime_video_map_t *vtrack);
@@ -15,62 +18,21 @@ void quicktime_init_codec_yv12(quicktime_video_map_t *vtrack);
 
 static char * fourccs_raw[]  = { QUICKTIME_RAW, "raw3",   (char*)0 };
 
-static char * fourccs_v308[] = { QUICKTIME_YUV444,        (char*)0 };
+static char * fourccs_v210[] = { QUICKTIME_V210,  (char *)0};
 
-static char * fourccs_v408[] = { QUICKTIME_YUVA4444 ,     (char*)0 };
+static char * fourccs_v308[] = { QUICKTIME_V308,        (char*)0 };
 
-static char * fourccs_v410[] = { QUICKTIME_YUV444_10bit , (char*)0 };
+static char * fourccs_v408[] = { QUICKTIME_V408,     (char*)0 };
 
-static char * fourccs_yuv2[] = { QUICKTIME_YUV422 ,       (char*)0 };
+static char * fourccs_v410[] = { QUICKTIME_V410 , (char*)0 };
+
+static char * fourccs_yuv2[] = { QUICKTIME_YUV2 ,       (char*)0 };
 
 static char * fourccs_2vuy[] = { QUICKTIME_2VUY ,         (char*)0 };
 
 static char * fourccs_yuv4[] = { QUICKTIME_YUV4 ,         (char*)0 };
 
-static char * fourccs_yv12[] = { QUICKTIME_YUV420 ,       (char*)0 };
-
-static int encoding_colormodels_raw[] =
-  {
-    BC_RGB888,
-    BC_ARGB8888,
-    LQT_COLORMODEL_NONE
-  };
-
-static int encoding_colormodels_v308[] =
-  {
-    BC_VYU888,
-    LQT_COLORMODEL_NONE,
-  };
-
-static int encoding_colormodels_v408[] =
-  {
-    BC_UYVA8888,
-    LQT_COLORMODEL_NONE,
-  };
-
-static int encoding_colormodels_v410[] =
-  {
-    BC_YUV101010,
-    LQT_COLORMODEL_NONE,
-  };
-
-static int encoding_colormodels_yuv2[] =
-  {
-    BC_YUV422,
-    LQT_COLORMODEL_NONE,
-  };
-
-static int encoding_colormodels_yuv4[] =
-  {
-    BC_RGB888,
-    LQT_COLORMODEL_NONE,
-  };
-
-static int encoding_colormodels_yv12[] =
-  {
-    BC_YUV420P,
-    LQT_COLORMODEL_NONE,
-  };
+static char * fourccs_yv12[] = { QUICKTIME_YUV420 , "I420", (char*)0 };
 
 // if DUMMY_PARAMETERS is defined it will cause segfaults
 #undef DUMMY_PARAMETERS
@@ -114,7 +76,7 @@ static lqt_codec_info_static_t codec_info_raw =
   {
   name:        "raw",
   long_name:   "RGB uncompressed",
-  description: "RGB uncompressed. Allows alpha",
+  description: "RGB uncompressed.",
   fourccs:     fourccs_raw,
   type:        LQT_CODEC_VIDEO,
   direction:   LQT_DIRECTION_BOTH,
@@ -125,51 +87,78 @@ static lqt_codec_info_static_t codec_info_raw =
   encoding_parameters: (lqt_parameter_info_static_t*)0,
   decoding_parameters: (lqt_parameter_info_static_t*)0,
 #endif
-  encoding_colormodels: encoding_colormodels_raw,
-  decoding_colormodel:  LQT_COLORMODEL_NONE
+  compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
+  };
+
+static lqt_codec_info_static_t codec_info_rawalpha =
+  {
+  name:        "rawalpha",
+  long_name:   "RGBA uncompressed",
+  description: "RGBA uncompressed",
+  fourccs:     fourccs_raw,
+  type:        LQT_CODEC_VIDEO,
+  direction:   LQT_DIRECTION_ENCODE,
+#ifdef DUMMY_PARAMETERS
+  encoding_parameters: dummy_parameters,
+  decoding_parameters: dummy_parameters,
+#else
+  encoding_parameters: (lqt_parameter_info_static_t*)0,
+  decoding_parameters: (lqt_parameter_info_static_t*)0,
+#endif
+  compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
+  };
+
+
+static lqt_codec_info_static_t codec_info_v210 =
+  {
+  name:        "v210",
+  long_name:   "10 bit Packed YUV 4:2:2 (v210)",
+  description: "10 bit Packed YUV 4:2:2 (v210)",
+  fourccs:     fourccs_v210,
+  type:        LQT_CODEC_VIDEO,
+  direction:   LQT_DIRECTION_BOTH,
+  encoding_parameters: (lqt_parameter_info_static_t*)0,
+  decoding_parameters: (lqt_parameter_info_static_t*)0,
+  compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
   };
 
 static lqt_codec_info_static_t codec_info_v308 =
   {
   name:        "v308",
-  long_name:   "8 bit Planar YUV 4:4:4",
+  long_name:   "8 bit Planar YUV 4:4:4 (v308)",
   description: "8 bit Planar YUV 4:4:4 (v308)",
   fourccs:     fourccs_v308,
   type:        LQT_CODEC_VIDEO,
   direction:   LQT_DIRECTION_BOTH,
   encoding_parameters: (lqt_parameter_info_static_t*)0,
   decoding_parameters: (lqt_parameter_info_static_t*)0,
-  encoding_colormodels: encoding_colormodels_v308,
-  decoding_colormodel:  BC_VYU888
+  compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
   };
 
 static lqt_codec_info_static_t codec_info_v408 =
   {
   name:              "v408",
-  long_name:         "8 bit Planar YUVA 4:4:4:4",
+  long_name:         "8 bit Planar YUVA 4:4:4:4 (v408)",
   description:       "8 bit Planar YUVA 4:4:4:4 (v408)",
   fourccs:           fourccs_v408,
   type:              LQT_CODEC_VIDEO,
   direction:         LQT_DIRECTION_BOTH,
   encoding_parameters: (lqt_parameter_info_static_t*)0,
   decoding_parameters: (lqt_parameter_info_static_t*)0,
-  encoding_colormodels: encoding_colormodels_v408,
-  decoding_colormodel:  BC_UYVA8888
-
+  compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
   };
 
 static lqt_codec_info_static_t codec_info_v410 =
   {
   name:        "v410",
-  long_name:   "10 bit Planar YUV 4:4:4 (v410)",
-  description: "10 bit Planar YUV 4:4:4 (v410)",
+  long_name:   "10 bit Packed YUV 4:4:4 (v410)",
+  description: "10 bit Packed YUV 4:4:4 (v410)",
   fourccs:     fourccs_v410,
   type:        LQT_CODEC_VIDEO,
   direction:   LQT_DIRECTION_BOTH,
   encoding_parameters: (lqt_parameter_info_static_t*)0,
   decoding_parameters: (lqt_parameter_info_static_t*)0,
-  encoding_colormodels: encoding_colormodels_v410,
-  decoding_colormodel: BC_YUV101010
+  compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
   };
 
 static lqt_codec_info_static_t codec_info_yuv2 =
@@ -182,9 +171,8 @@ static lqt_codec_info_static_t codec_info_yuv2 =
   direction:   LQT_DIRECTION_BOTH,
   encoding_parameters: (lqt_parameter_info_static_t*)0,
   decoding_parameters: (lqt_parameter_info_static_t*)0,
-  encoding_colormodels: encoding_colormodels_yuv2,
-  decoding_colormodel: BC_YUV422
 
+  compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
   };
 
 static lqt_codec_info_static_t codec_info_2vuy =
@@ -197,8 +185,7 @@ static lqt_codec_info_static_t codec_info_2vuy =
   direction:   LQT_DIRECTION_BOTH,
   encoding_parameters: (lqt_parameter_info_static_t*)0,
   decoding_parameters: (lqt_parameter_info_static_t*)0,
-  encoding_colormodels: encoding_colormodels_yuv2,
-  decoding_colormodel: BC_YUV422
+  compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
 
   };
 
@@ -212,8 +199,7 @@ static lqt_codec_info_static_t codec_info_yuv4 =
     direction:   LQT_DIRECTION_BOTH,
     encoding_parameters: (lqt_parameter_info_static_t*)0,
     decoding_parameters: (lqt_parameter_info_static_t*)0,
-    encoding_colormodels: encoding_colormodels_yuv4,
-    decoding_colormodel: BC_RGB888
+  compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
 
   };
 
@@ -227,14 +213,13 @@ static lqt_codec_info_static_t codec_info_yv12 =
     direction:   LQT_DIRECTION_BOTH,
     encoding_parameters: (lqt_parameter_info_static_t*)0,
     decoding_parameters: (lqt_parameter_info_static_t*)0,
-    encoding_colormodels: encoding_colormodels_yv12,
-    decoding_colormodel: BC_YUV420P
+  compatibility_flags: LQT_FILE_QT_OLD | LQT_FILE_QT,
     
   };
 
 /* These are called from the plugin loader */
 
-extern int get_num_codecs() { return 8; }
+extern int get_num_codecs() { return 10; }
 
 extern lqt_codec_info_static_t * get_codec_info(int index)
   {
@@ -242,28 +227,24 @@ extern lqt_codec_info_static_t * get_codec_info(int index)
     {
     case 0: /* raw */
       return &codec_info_raw;
-      break;
-    case 1: /* v308 */
+    case 1: /* raw (with alpha) */
+      return &codec_info_rawalpha;
+    case 2: /* v308 */
       return &codec_info_v308;
-      break;
-    case 2: /* v408 */
+    case 3: /* v408 */
       return &codec_info_v408;
-      break;
-    case 3: /* v410 */
+    case 4: /* v410 */
       return &codec_info_v410;
-      break;
-    case 4: /* yuv2 */
+    case 5: /* yuv2 */
       return &codec_info_yuv2;
-      break;
-    case 5: /* yuv4 */
+    case 6: /* yuv4 */
       return &codec_info_yuv4;
-      break;
-    case 6: /* vy12 */
+    case 7: /* vy12 */
       return &codec_info_yv12;
-      break;
-    case 7: /* 2vuy */
+    case 8: /* 2vuy */
       return &codec_info_2vuy;
-      break;
+    case 9: /* v210 */
+      return &codec_info_v210;
     }
   return (lqt_codec_info_static_t*)0;
   }
@@ -274,51 +255,25 @@ extern lqt_init_video_codec_func_t get_video_codec(int index)
     {
     case 0: /* raw */
       return quicktime_init_codec_raw;
-      break;
-    case 1: /* v308 */
+    case 1: /* raw (with alpha) */
+      return quicktime_init_codec_rawalpha;
+    case 2: /* v308 */
       return quicktime_init_codec_v308;
-      break;
-    case 2: /* v408 */
+    case 3: /* v408 */
       return quicktime_init_codec_v408;
-      break;
-    case 3: /* v410 */
-    return quicktime_init_codec_v410;
-      break;
-    case 4: /* yuv2 */
+    case 4: /* v410 */
+      return quicktime_init_codec_v410;
+    case 5: /* yuv2 */
       return quicktime_init_codec_yuv2;
-      break;
-    case 5: /* yuv4 */
+    case 6: /* yuv4 */
       return quicktime_init_codec_yuv4;
-      break;
-    case 6: /* vy12 */
+    case 7: /* vy12 */
       return quicktime_init_codec_yv12;
-      break;
-    case 7: /* 2vuy */
+    case 8: /* 2vuy */
       return quicktime_init_codec_2vuy;
-      break;
+    case 9: /* v210 */
+      return quicktime_init_codec_v210;
     }
   return (lqt_init_video_codec_func_t)0;
   }
 
-int get_stream_colormodel(quicktime_t * file, int track,
-                          int codec_index, int * exact)
-  {
-  int depth;
-  if(exact)
-    *exact = 1;
-  
-  if(codec_index == 0)
-    {
-    depth = quicktime_video_depth(file, track);
-    switch(depth)
-      {
-      case 32:
-        return BC_ARGB8888;
-        break;
-      default:
-        return BC_RGB888;
-        break;
-      }
-    }
-  return LQT_COLORMODEL_NONE;
-  }
