@@ -12,8 +12,8 @@ dst[2] = pal->blue[indx] >> 8;
 
 typedef struct
 {
-	unsigned char *temp_frame;  /* For changing color models and scaling */
-	unsigned char **temp_rows;
+//	unsigned char *temp_frame;  /* For changing color models and scaling */
+//	unsigned char **temp_rows;
 	unsigned char *temp_data;
         int temp_data_alloc;
 /* Support all possible depths */
@@ -171,94 +171,12 @@ static void scanline_raw_32(uint8_t * src,
     }
   }
 
-#if 0
-static void scanline_raw_2_gray(uint8_t * src,
-                                uint8_t * dst,
-                                int num_pixels,
-                                quicktime_ctab_t * pal)
-  {
-  int i;
-  int counter = 0;
-  for(i = 0; i < num_pixels; i++)
-    {
-    if(counter == 4)
-      {
-      counter = 0;
-      src++;
-      }
-    *dst = *src & 0xc0;
-                                                                                
-    /* Fill byte */
-                                                                                
-    *dst |= (*dst >> 2);
-    *dst |= (*dst >> 4);
-                                                                                
-    dst[1] = dst[0];
-    dst[2] = dst[0];
-                                                                                
-    /* Advance */
-                                                                                
-    *src <<= 2;
-    dst += 3;
-    counter++;
-    }
-  }
-
-
-static void scanline_raw_4_gray(uint8_t * src,
-                                uint8_t * dst,
-                                int num_pixels,
-                                quicktime_ctab_t * pal)
-  {
-  int i;
-                                                                                
-  int counter = 0;
-  for(i = 0; i < num_pixels; i++)
-    {
-    if(counter == 2)
-      {
-      counter = 0;
-      src++;
-      }
-    *dst = *src & 0xF0;
-                                                                                
-    /* Fill byte */
-                                                                                
-    *dst |= (*dst >> 4);
-                                                                                
-    dst[1] = dst[0];
-    dst[2] = dst[0];
-                                                                                
-    /* Advance */
-                                                                                
-    *src <<= 4;
-    dst += 3;
-    counter++;
-    }
-  }
-
-static void scanline_raw_8_gray(uint8_t * src,
-                                uint8_t * dst,
-                                int num_pixels,
-                                quicktime_ctab_t * pal)
-  {
-  int i;
-  for(i = 0; i < num_pixels; i++)
-    {
-    dst[0] = *src;
-    dst[1] = *src;
-    dst[2] = *src;
-    dst += 3;
-    src++;
-    }
-  }
-
-#endif
 /* */
 
 static int quicktime_delete_codec_raw(quicktime_video_map_t *vtrack)
 {
 	quicktime_raw_codec_t *codec = ((quicktime_codec_t*)vtrack->codec)->priv;
+#if 0
 	if(codec->temp_frame)
 	{
 		free(codec->temp_frame);
@@ -267,6 +185,7 @@ static int quicktime_delete_codec_raw(quicktime_video_map_t *vtrack)
 	{
 		free(codec->temp_rows);
 	}
+#endif
 	if(codec->temp_data)
 	{
 		free(codec->temp_data);
@@ -287,39 +206,6 @@ static int source_cmodel(quicktime_t *file, int track)
                 return BC_RGB888;
 }
 
-static int reads_colormodel(quicktime_t *file, 
-                            int colormodel, 
-                            int track)
-  {
-  
-  if(source_cmodel(file, track) == BC_RGB888)
-    return (colormodel == BC_RGB888) ||
-      (colormodel == BC_BGR8888);
-  else
-    return (colormodel == BC_RGBA8888) ||
-      (colormodel == BC_ARGB8888) ||
-      (colormodel == BC_RGB888) ||
-      (colormodel == BC_BGR8888);
-  }
-
-/* static int quicktime_reads_colormodel_raw(quicktime_t *file,  */
-/* 		int colormodel,  */
-/* 		int track) */
-/* { */
-/* 	return (colormodel == BC_RGB888 || */
-/* 		colormodel == BC_BGR8888); */
-/* } */
-
-
-static int writes_colormodel(quicktime_t *file, 
-                             int colormodel, 
-                             int track)
-  {
-  return ((colormodel == BC_RGBA8888) ||
-          (colormodel == BC_ARGB8888) ||
-          (colormodel == BC_RGB888));
-  }
-
 static int quicktime_decode_raw(quicktime_t *file, unsigned char **row_pointers, int track)
 {
 	int result = 0;
@@ -329,23 +215,18 @@ static int quicktime_decode_raw(quicktime_t *file, unsigned char **row_pointers,
 	int width = trak->tkhd.track_width;
 	long bytes;
 	int i;
-        unsigned char ** rows;
 	unsigned char *ptr;
         quicktime_ctab_t * ctab;
 	quicktime_raw_codec_t *codec = ((quicktime_codec_t*)file->vtracks[track].codec)->priv;
-	int cmodel = source_cmodel(file, track);
-	int use_temp = (cmodel != file->vtracks[track].color_model ||
-		file->in_x != 0 ||
-		file->in_y != 0 ||
-		file->in_w != width ||
-		file->in_h != height ||
-		file->out_w != width ||
-		file->out_h != height);
-
+        
+        if(!row_pointers)
+          {
+          file->vtracks[track].stream_cmodel = source_cmodel(file, track);
+          return 0;
+          }
+        
         ctab = &(trak->mdia.minf.stbl.stsd.table->ctab);
                 
-        if(!codec->temp_rows)
-          codec->temp_rows = malloc(sizeof(unsigned char*) * height);
         
         if(!codec->scanline_func)
           {
@@ -419,22 +300,6 @@ static int quicktime_decode_raw(quicktime_t *file, unsigned char **row_pointers,
             codec->bytes_per_line++;
           }
                 
-	if(use_temp)
-	{
-		if(!codec->temp_frame)
-		{
-                codec->temp_frame = malloc(codec->bytes_per_line * height);
-                for(i = 0; i < height; i++)
-                  {
-                  codec->temp_rows[i] = codec->temp_frame + i * codec->bytes_per_line;
-                  }
-		}
-                rows = codec->temp_rows;
-        }
-	else
-	{
-                rows = row_pointers;
-	}
 
 
 /* Read data */
@@ -455,36 +320,9 @@ static int quicktime_decode_raw(quicktime_t *file, unsigned char **row_pointers,
 
         for(i = 0; i < height; i++)
           {
-          codec->scanline_func(ptr, rows[i], width, ctab);
+          codec->scanline_func(ptr, row_pointers[i], width, ctab);
           ptr += codec->bytes_per_line;
           }
-        
-/* Convert colormodel */
-	if(use_temp)
-	{
-		cmodel_transfer(row_pointers, 
-			rows,
-			row_pointers[0],
-			row_pointers[1],
-			row_pointers[2],
-			0,
-			0,
-			0,
-			file->in_x, 
-			file->in_y, 
-			file->in_w, 
-			file->in_h,
-			0, 
-			0, 
-			file->out_w, 
-			file->out_h,
-			cmodel, 
-			file->vtracks[track].color_model,
-			0,
-			width,
-			file->out_w);
-	}
-
 	return result;
 }
 
@@ -492,117 +330,82 @@ static int quicktime_encode_raw(quicktime_t *file,
 	unsigned char **row_pointers, 
 	int track)
 {
+        int i, j;
+        uint8_t * in_ptr, *out_ptr;
+        uint8_t padd = 0;
 	quicktime_video_map_t *vtrack = &(file->vtracks[track]);
-	quicktime_raw_codec_t *codec = ((quicktime_codec_t*)vtrack->codec)->priv;
 	quicktime_trak_t *trak = vtrack->track;
 	int result = 0;
-	register int i;
 	int height = trak->tkhd.track_height;
 	int width = trak->tkhd.track_width;
 	int depth = quicktime_video_depth(file, track);
-	int dest_cmodel;
         quicktime_atom_t chunk_atom;
+	quicktime_raw_codec_t *codec = ((quicktime_codec_t*)file->vtracks[track].codec)->priv;
         
 //printf("quicktime_encode_raw %llx %llx\n", file->file_position, file->ftell_position);
-	if(depth == 32)
-	{
-		dest_cmodel = BC_ARGB8888;
+        if(!row_pointers)
+          {
+          if(depth == 32)
+            vtrack->stream_cmodel = BC_RGBA8888;
+          else
+            vtrack->stream_cmodel = BC_RGB888;
+          return 0;
+          }
 
-	}
-	else
-	{
-		dest_cmodel = BC_RGB888;
-	}
+        if(!codec->bytes_per_line)
+          {
+          if(depth == 32)
+            codec->bytes_per_line = width * 4;
+          else
+            codec->bytes_per_line = width * 3;
 
+          if(codec->bytes_per_line & 1)
+            codec->bytes_per_line++;
+          }
+        
+        quicktime_write_chunk_header(file, trak, &chunk_atom);
 
+        if(vtrack->stream_cmodel == BC_RGBA8888)
+          {
+          if(!codec->temp_data)
+            codec->temp_data = calloc(codec->bytes_per_line, 1);
 
-
-	if(file->vtracks[track].color_model != dest_cmodel)
-	{
-		if(!codec->temp_frame)
-		{
-			codec->temp_frame = malloc(cmodel_calculate_datasize(width,
-				height,
-				-1,
-				dest_cmodel));
-			codec->temp_rows = malloc(sizeof(unsigned char*) * height);
-			
-			for(i = 0; i < height; i++)
-			{
-				codec->temp_rows[i] = codec->temp_frame + 
-					i * cmodel_calculate_pixelsize(dest_cmodel) * width;
-			}
-		}
-		
-		
-		
-		cmodel_transfer(codec->temp_rows, /* Leave NULL if non existent */
-			row_pointers,
-			0, /* Leave NULL if non existent */
-			0,
-			0,
-			row_pointers[0], /* Leave NULL if non existent */
-			row_pointers[1],
-			row_pointers[2],
-			0,        /* Dimensions to capture from input frame */
-			0, 
-			width, 
-			height,
-			0,       /* Dimensions to project on output frame */
-			0,
-			width, 
-			height,
-                        file->vtracks[track].color_model, 
-			dest_cmodel,
-			0,         /* When transfering BC_RGBA8888 to non-alpha this is the background color in 0xRRGGBB hex */
-			width,       /* For planar use the luma rowspan */
-			width);     /* For planar use the luma rowspan */
-                quicktime_write_chunk_header(file, trak, &chunk_atom);
-		result = !quicktime_write_data(file, codec->temp_frame, 
-			cmodel_calculate_datasize(width,
-				height,
-				-1,
-				dest_cmodel));
-                quicktime_write_chunk_footer(file,
-                        trak,
-                        vtrack->current_chunk,
-                        &chunk_atom,
-                        1);
-
-	}
-	else
-	{
-                quicktime_write_chunk_header(file, trak, &chunk_atom);
-                result = !quicktime_write_data(file, row_pointers[0], 
-			cmodel_calculate_datasize(width,
-				height,
-				-1,
-                                file->vtracks[track].color_model));
-                quicktime_write_chunk_footer(file,
-                        trak,
-                        vtrack->current_chunk,
-                        &chunk_atom,
-                        1);
-	}
-
+          for(i = 0; i < height; i++)
+            {
+            in_ptr = row_pointers[i];
+            out_ptr = codec->temp_data;
+            for(j = 0; j < width; j++)
+              {
+              out_ptr[1] = in_ptr[0]; /* R */
+              out_ptr[2] = in_ptr[1]; /* G */
+              out_ptr[3] = in_ptr[2]; /* B */
+              out_ptr[0] = in_ptr[3]; /* A */
+              out_ptr += 4;
+              in_ptr  += 4;
+              }
+            result = !quicktime_write_data(file, codec->temp_data, codec->bytes_per_line);
+            }
+          }
+        else /* BC_RGB888 */
+          {
+          for(i = 0; i < height; i++)
+            {
+            result = !quicktime_write_data(file, row_pointers[i], width * 3);
+            if(width & 1)
+              result = !quicktime_write_data(file, &padd, 1);
+            }
+          }
+        
+        quicktime_write_chunk_footer(file,
+                                     trak,
+                                     vtrack->current_chunk,
+                                     &chunk_atom,
+                                     1);
+        
         vtrack->current_chunk++;
 	return result;
 }
 
-#if 0 /* Unused */
-
-int quicktime_raw_rows_consecutive(unsigned char **row_pointers, int w, int h, int depth)
-{
-	int i, result;
-/* see if row_pointers are consecutive */
-	for(i = 1, result = 1; i < h; i++)
-	{
-		if(row_pointers[i] - row_pointers[i - 1] != w * depth) result = 0;
-	}
-	return result;
-}
-#endif
-
 void quicktime_init_codec_raw(quicktime_video_map_t *vtrack)
 {
         quicktime_codec_t *codec_base = (quicktime_codec_t*)vtrack->codec;
@@ -613,9 +416,10 @@ void quicktime_init_codec_raw(quicktime_video_map_t *vtrack)
         codec_base->encode_video = quicktime_encode_raw;
         codec_base->decode_audio = 0;
         codec_base->encode_audio = 0;
-        codec_base->reads_colormodel = reads_colormodel;
-        codec_base->writes_colormodel = writes_colormodel;
-        codec_base->fourcc = QUICKTIME_RAW;
-        codec_base->title = "RGB uncompressed";
-        codec_base->desc = "RGB uncompressed";
 }
+
+void quicktime_init_codec_rawalpha(quicktime_video_map_t *vtrack)
+  {
+  vtrack->track->mdia.minf.stbl.stsd.table[0].depth = 32;
+  quicktime_init_codec_raw(vtrack);
+  }
