@@ -20,7 +20,8 @@ typedef struct
 	int nominal_bitrate;
 	int min_bitrate;
 	int use_vbr;
-	int encode_initialized;
+        int write_OVHS;
+        int encode_initialized;
 	ogg_stream_state enc_os;
 	ogg_page enc_og;
         uint8_t * enc_header;
@@ -85,6 +86,8 @@ ogg_packet enc_op;
 // Number of samples decoded in the current chunk
 	int chunk_samples;
 
+        int header_read;
+
   } quicktime_vorbis_codec_t;
 
 
@@ -130,29 +133,71 @@ static int delete_codec(quicktime_audio_map_t *atrack)
 
 static int next_chunk(quicktime_t * file, int track)
   {
+  int i;
+  int num_packets;
+  int samples;
   int chunk_size;
   char * buffer;
+  uint8_t * header;
+  uint32_t header_len;
+
   quicktime_audio_map_t *track_map = &(file->atracks[track]);
   quicktime_vorbis_codec_t *codec = ((quicktime_codec_t*)track_map->codec)->priv;
 
+  if(!codec->header_read) /* Try OVHS atom */
+    {
+    header = quicktime_wave_get_user_atom(track_map->track, "OVHS", &header_len);
+    if(header)
+      {
+      fprintf(stderr, "Using OVHS Atom, %d bytes\n", header_len-8);
+      buffer = ogg_sync_buffer(&codec->dec_oy, header_len-8);
+      memcpy(buffer, header + 8, header_len-8);
+      ogg_sync_wrote(&codec->dec_oy, header_len-8);
+      return 1;
+      }
+    }
+  
   //  fprintf(stderr, "Next chunk %lld\n", track_map->current_chunk);
 
-  chunk_size = lqt_read_audio_chunk(file,
-                                    track, track_map->current_chunk,
-                                    &(codec->chunk_buffer),
-                                    &(codec->chunk_buffer_alloc));
-  if(chunk_size <= 0)
+  if(lqt_audio_is_vbr(file, track))
     {
-    //    fprintf(stderr, "Shit\n");
-    return 0;
+    num_packets = lqt_audio_num_vbr_packets(file, track, track_map->current_chunk, &samples);
+    if(!num_packets)
+      return 0;
+
+    //    fprintf(stderr, "VBR Chunk: packets: %d, samples: %d\n", num_packets, samples);
+    
+    for(i = 0; i < num_packets; i++)
+      {
+      chunk_size = lqt_audio_read_vbr_packet(file, track, track_map->current_chunk, i,
+                                             &(codec->chunk_buffer),
+                                             &(codec->chunk_buffer_alloc), &samples);
+      //      fprintf(stderr, "Read VBR packet, chunk: %lld, packet: %d, bytes: %d, samples: %d\n",
+      //              track_map->current_chunk, i, chunk_size, samples);
+      buffer = ogg_sync_buffer(&codec->dec_oy, chunk_size);
+      memcpy(buffer, codec->chunk_buffer, chunk_size);
+      ogg_sync_wrote(&codec->dec_oy, chunk_size);
+      }
+    }
+  else
+    {
+    chunk_size = lqt_read_audio_chunk(file,
+                                      track, track_map->current_chunk,
+                                      &(codec->chunk_buffer),
+                                      &(codec->chunk_buffer_alloc), (int*)0);
+    if(chunk_size <= 0)
+      {
+      //    fprintf(stderr, "Shit\n");
+      return 0;
+      }
+    
+    buffer = ogg_sync_buffer(&codec->dec_oy, chunk_size);
+    memcpy(buffer, codec->chunk_buffer, chunk_size);
+    ogg_sync_wrote(&codec->dec_oy, chunk_size);
     }
   
   track_map->current_chunk++;
   
-  buffer = ogg_sync_buffer(&codec->dec_oy, chunk_size);
-  memcpy(buffer, codec->chunk_buffer, chunk_size);
-  ogg_sync_wrote(&codec->dec_oy, chunk_size);
-  
   return 1;
   }
 
@@ -161,6 +206,7 @@ static int next_page(quicktime_t * file, int track)
   quicktime_audio_map_t *track_map = &(file->atracks[track]);
   quicktime_vorbis_codec_t *codec = ((quicktime_codec_t*)track_map->codec)->priv;
   int result = 0;
+  
   while(result < 1)
     {
     result = ogg_sync_pageout(&codec->dec_oy, &codec->dec_og);
@@ -285,12 +331,11 @@ static int decode_frame(quicktime_t * file, int track)
   }
 
 static int decode(quicktime_t *file, 
-					int16_t **output_i, 
-					float **output_f, 
-					long samples, 
-					int track) 
+                  void * _output,
+                  long samples, 
+                  int track) 
   {
-  int i;
+  int i, j;
   int64_t chunk_sample; /* For seeking only */
   quicktime_audio_map_t *track_map = &(file->atracks[track]);
   quicktime_vorbis_codec_t *codec = ((quicktime_codec_t*)track_map->codec)->priv;
@@ -299,7 +344,13 @@ static int decode(quicktime_t *file,
   int samples_to_skip;
   int samples_to_move;
   int samples_copied;
+  float * output;
   //  fprintf(stderr, "decode audio vorbis %lld\n", track_map->current_position);
+
+  if(!_output) /* Global initialization */
+    {
+    return 0;
+    }
   
   /* Initialize codec */
   if(!codec->decode_initialized)
@@ -347,7 +398,7 @@ static int decode(quicktime_t *file,
       fprintf(stderr, "decode: vorbis_synthesis_headerin: not a vorbis header\n");
       return 0;
       }
-    
+    codec->header_read = 1;
     vorbis_synthesis_init(&codec->dec_vd, &codec->dec_vi);
     vorbis_block_init(&codec->dec_vd, &codec->dec_vb);
     //    fprintf(stderr, "Initialized decoder\n");
@@ -357,68 +408,71 @@ static int decode(quicktime_t *file,
   
   if(file->atracks[track].last_position != file->atracks[track].current_position)
     {
-    if((file->atracks[track].current_position < codec->sample_buffer_start) || 
-       (file->atracks[track].current_position + samples > codec->sample_buffer_end))
-      {
-      /* Get the next chunk */
+    /* Get the next chunk */
 
+    if(lqt_audio_is_vbr(file, track))
+      lqt_chunk_of_sample_vbr(&chunk_sample,
+                              &(track_map->current_chunk),
+                              track_map->track,
+                              track_map->current_position);
+    else
       quicktime_chunk_of_sample(&chunk_sample,
                                 &(track_map->current_chunk),
                                 track_map->track,
                                 track_map->current_position);
       
-      if(track_map->current_chunk >= file->atracks[track].track->mdia.minf.stbl.stco.total_entries)
-        {
+    if(track_map->current_chunk >= file->atracks[track].track->mdia.minf.stbl.stco.total_entries)
+      {
 #if 0
-        fprintf(stderr, "Detected EOF %lld %ld\n",
-                track_map->current_chunk,
-                file->atracks[track].track->mdia.minf.stbl.stco.total_entries);
-#endif
-        return 0;
-        }
-#if 0
-      fprintf(stderr, "Seek: pos: %lld, chunk: %lld, chunk_sample: %lld\n",
-              file->atracks[track].current_position,
-              file->atracks[track].current_chunk, chunk_sample);
-#endif
-      
-      
-      /* Reset everything */
-      
-      vorbis_dsp_clear(&codec->dec_vd);
-      vorbis_block_clear(&codec->dec_vb);
-      
-      ogg_stream_clear(&codec->dec_os);
-      ogg_sync_reset(&codec->dec_oy);
-      codec->stream_initialized = 0;
-      if(!next_page(file, track))
-        return 0;
-
-      /* Initialize again */
-      
-      ogg_sync_init(&codec->dec_oy);
-      ogg_stream_init(&codec->dec_os, ogg_page_serialno(&codec->dec_og));
-      vorbis_synthesis_init(&codec->dec_vd, &codec->dec_vi);
-      vorbis_block_init(&codec->dec_vd, &codec->dec_vb);
-
-      /* Reset sample buffer */
-            
-      codec->sample_buffer_start = chunk_sample;
-      codec->sample_buffer_end   = chunk_sample;
-
-      /* Decode frames until we have enough */
-      
-      while(track_map->current_position + samples > codec->sample_buffer_end)
-        {
-        if(!decode_frame(file, track))
-          break;
-        }
-#if 0
-      fprintf(stderr, "Cur: %lld, start: %lld end: %lld\n", track_map->current_position,
-              codec->sample_buffer_start, codec->sample_buffer_end);
+      fprintf(stderr, "Detected EOF %lld %ld\n",
+              track_map->current_chunk,
+              file->atracks[track].track->mdia.minf.stbl.stco.total_entries);
 #endif
+      return 0;
       }
+#if 1
+    fprintf(stderr, "Seek: pos: %lld, chunk: %lld, chunk_sample: %lld\n",
+            file->atracks[track].current_position,
+            file->atracks[track].current_chunk, chunk_sample);
+#endif
+      
+      
+    /* Reset everything */
+      
+    vorbis_dsp_clear(&codec->dec_vd);
+    vorbis_block_clear(&codec->dec_vb);
+      
+    ogg_stream_clear(&codec->dec_os);
+    ogg_sync_reset(&codec->dec_oy);
+    codec->stream_initialized = 0;
+
+    /* Initialize again */
+    ogg_sync_init(&codec->dec_oy);
+    // ogg_stream_init(&codec->dec_os, ogg_page_serialno(&codec->dec_og));
+    vorbis_synthesis_init(&codec->dec_vd, &codec->dec_vi);
+    vorbis_block_init(&codec->dec_vd, &codec->dec_vb);
+
+    if(!next_page(file, track))
+      return 0;
+      
+    /* Reset sample buffer */
+      
+    codec->sample_buffer_start = chunk_sample;
+    codec->sample_buffer_end   = chunk_sample;
+
+    /* Decode frames until we have enough */
+      
+    while(track_map->current_position + samples > codec->sample_buffer_end)
+      {
+      if(!decode_frame(file, track))
+        break;
+      }
+#if 0
+    fprintf(stderr, "Cur: %lld, start: %lld end: %lld\n", track_map->current_position,
+            codec->sample_buffer_start, codec->sample_buffer_end);
+#endif
     }
+  
 
   /* Flush unneeded samples */
   
@@ -466,16 +520,19 @@ static int decode(quicktime_t *file,
     }
   samples_decoded = (codec->sample_buffer_end - codec->sample_buffer_start) < samples ?
     (codec->sample_buffer_end - codec->sample_buffer_start) : samples;
-  
-  samples_copied = lqt_copy_audio(output_i,                // int16_t ** dst_i
-                                  output_f,                // float ** dst_f
-                                  (int16_t**)0,            // int16_t ** src_i
-                                  codec->sample_buffer,    // float ** src_f
-                                  0,                       // int dst_pos
-                                  0,                       // int src_pos
-                                  samples,                 // int dst_size
-                                  samples_decoded,         // int src_size
-                                  file->atracks[track].channels); // int num_channels
+
+  samples_copied = samples;
+  if(samples_copied > samples_decoded)
+    samples_copied = samples_decoded;
+
+  output = (float*)_output;
+  for(i = 0; i < samples_copied; i++)
+    {
+    for(j = 0; j < track_map->channels; j++)
+      {
+      *(output++) = codec->sample_buffer[j][i];
+      }
+    }
   
   file->atracks[track].last_position = file->atracks[track].current_position + samples_copied;
   
@@ -496,7 +553,6 @@ static int flush_header(quicktime_t * file, int track)
     {
     //    fprintf(stderr, "Got header page %ld + %ld bytes\n",
     //            codec->enc_og.header_len, codec->enc_og.body_len);
-
     codec->enc_header = realloc(codec->enc_header, codec->enc_header_len +
                                 codec->enc_og.header_len + codec->enc_og.body_len);
     
@@ -508,12 +564,21 @@ static int flush_header(quicktime_t * file, int track)
     
     codec->enc_header_len += codec->enc_og.header_len + codec->enc_og.body_len;
     }
-  
+
+  if(codec->write_OVHS)
+    {
+    fprintf(stderr, "Writing Vorbis header %d bytes\n", codec->enc_header_len);
+    quicktime_wave_set_user_atom(track_map->track, "OVHS",
+                                 codec->enc_header, codec->enc_header_len);
+    
+    codec->header_written = 1;
+    }
   return 0;
   }
 
 static int flush_data(quicktime_t * file, int track)
   {
+  int64_t new_encoded_samples;
   int result = 0;
   quicktime_audio_map_t *track_map = &(file->atracks[track]);
   quicktime_trak_t *trak = track_map->track;
@@ -536,7 +601,6 @@ static int flush_data(quicktime_t * file, int track)
       
       }
     }
-  
   while(!result)
     {
     //        ogg_stream_flush(&codec->enc_os, &codec->enc_og);
@@ -553,24 +617,31 @@ static int flush_data(quicktime_t * file, int track)
     if(!codec->chunk_started)
       {
       codec->chunk_started = 1;
+      lqt_start_audio_vbr_chunk(file, track);
       quicktime_write_chunk_header(file, trak, &codec->chunk_atom);
       //      fprintf(stderr, "Starting chunk\n");
-
-      if(!codec->header_written)
-        {
-        codec->header_written = 1;
-        result = !quicktime_write_data(file, codec->enc_header, codec->enc_header_len);
-        }
-
+      
+      }
+    lqt_start_audio_vbr_frame(file, track);
+    if(!codec->header_written)
+      {
+      codec->header_written = 1;
+      result = !quicktime_write_data(file, codec->enc_header, codec->enc_header_len);
       }
-    result = !quicktime_write_data(file, codec->enc_og.header, codec->enc_og.header_len);
     
+    result = !quicktime_write_data(file, codec->enc_og.header, codec->enc_og.header_len);
     if(!result)
       {
       result = !quicktime_write_data(file, codec->enc_og.body, codec->enc_og.body_len);
       }
     // fprintf(stderr, "Granulepos: %lld\n", ogg_page_granulepos(&(codec->enc_og)));
 
+    new_encoded_samples = codec->enc_os.granulepos;
+
+    lqt_finish_audio_vbr_frame(file, track, new_encoded_samples - codec->encoded_samples);
+    codec->encoded_samples = new_encoded_samples;
+    
+    
 #if 0
     fprintf(stderr, "Wrote page %ld + %ld bytes\n",
             codec->enc_og.header_len, codec->enc_og.body_len);
@@ -602,144 +673,128 @@ static void flush_audio(quicktime_t * file, int track)
   }
 
 static int encode(quicktime_t *file, 
-							int16_t **input_i, 
-							float **input_f, 
-							int track, 
-							long samples)
-{
-        int i, j;
-        int samples_copied, samples_to_copy;
-        int result = 0;
-	quicktime_audio_map_t *track_map = &(file->atracks[track]);
-	quicktime_trak_t *trak = track_map->track;
-	quicktime_vorbis_codec_t *codec = ((quicktime_codec_t*)track_map->codec)->priv;
-	int samplerate = trak->mdia.minf.stbl.stsd.table[0].sample_rate;
+                  void *_input,
+                  long samples,
+                  int track)
+  {
+  int i, j;
+  int samples_copied, samples_to_copy;
+  int result = 0;
+  quicktime_audio_map_t *track_map = &(file->atracks[track]);
+  quicktime_trak_t *trak = track_map->track;
+  quicktime_vorbis_codec_t *codec = ((quicktime_codec_t*)track_map->codec)->priv;
+  int samplerate = track_map->samplerate;
+  float * input;
+  
+  //        fprintf(stderr, "Encode vorbis %ld\n", samples);
         
-        //        fprintf(stderr, "Encode vorbis %ld\n", samples);
-        
-	if(!codec->encode_initialized)
-	{
-    	ogg_packet header;
-    	ogg_packet header_comm;
-    	ogg_packet header_code;
+  if(!codec->encode_initialized)
+    {
+    ogg_packet header;
+    ogg_packet header_comm;
+    ogg_packet header_code;
 
  
-        codec->encode_initialized = 1;
-        if(file->use_avi)
-          trak->mdia.minf.stbl.stsd.table[0].sample_size = 0;
-        vorbis_info_init(&codec->enc_vi);
+    codec->encode_initialized = 1;
 
-        if(codec->use_vbr)
-          {
-          result = vorbis_encode_setup_managed(&codec->enc_vi,
-                                               track_map->channels, 
-                                               samplerate, 
-                                               codec->max_bitrate, 
-                                               codec->nominal_bitrate, 
-                                               codec->min_bitrate);
-          result |= vorbis_encode_ctl(&codec->enc_vi, OV_ECTL_RATEMANAGE_AVG, NULL);
-          result |= vorbis_encode_setup_init(&codec->enc_vi);
-          }
-        else
-          {
-          vorbis_encode_init(&codec->enc_vi,
-                             track_map->channels,
-                             samplerate, 
-                             codec->max_bitrate, 
-                             codec->nominal_bitrate, 
-                             codec->min_bitrate);
-          }
+    lqt_init_vbr_audio(file, track);
 
 
-        vorbis_comment_init(&codec->enc_vc);
-        vorbis_analysis_init(&codec->enc_vd, &codec->enc_vi);
-        vorbis_block_init(&codec->enc_vd, &codec->enc_vb);
-        //    	srand(time(NULL));
-        ogg_stream_init(&codec->enc_os, rand());
+    if(file->file_type == LQT_FILE_AVI)
+      trak->mdia.minf.stbl.stsd.table[0].sample_size = 0;
+    vorbis_info_init(&codec->enc_vi);
 
-    	vorbis_analysis_headerout(&codec->enc_vd, 
-                                  &codec->enc_vc,
-                                  &header,
-                                  &header_comm,
-                                  &header_code);
-
-    	ogg_stream_packetin(&codec->enc_os, &header); 
-    	ogg_stream_packetin(&codec->enc_os, &header_comm);
-    	ogg_stream_packetin(&codec->enc_os, &header_code);
-
-        //        FLUSH_OGG1
-        flush_header(file, track);
-
-        codec->sample_buffer = alloc_sample_buffer(codec->sample_buffer, track_map->channels,
-                                                   ENCODE_SAMPLES,
-                                                   &codec->sample_buffer_alloc);
-        }
-        
-    samples_copied = 0;
-
-    while(samples_copied < samples)
+    if(codec->use_vbr)
       {
-      samples_to_copy = samples - samples_copied;
-      if(samples_to_copy > ENCODE_SAMPLES - codec->enc_samples_in_buffer)
-        samples_to_copy = ENCODE_SAMPLES - codec->enc_samples_in_buffer;
-
-      if(input_i)
-	{
-        for(i = 0; i < track_map->channels; i++)
-          {
-          for(j = 0; j < samples_to_copy; j++)
-            {
-            codec->sample_buffer[i][codec->enc_samples_in_buffer + j] =
-              (float)input_i[i][samples_copied + j] / (float)32768;
-            }
-          }
-	}
-      else
-	if(input_f)
-          {
-          int i;
-          for(i = 0; i < track_map->channels; i++)
-            {
-            memcpy(codec->sample_buffer[i] + codec->enc_samples_in_buffer,
-                   input_f[i] + samples_copied,
-                   sizeof(float) * samples_to_copy);
-            }
-          }
-      samples_copied += samples_to_copy;
-      codec->enc_samples_in_buffer += samples_to_copy;
-
-      if(codec->enc_samples_in_buffer >= ENCODE_SAMPLES)
-        flush_audio(file, track);
+      result = vorbis_encode_setup_managed(&codec->enc_vi,
+                                           track_map->channels, 
+                                           samplerate, 
+                                           codec->max_bitrate, 
+                                           codec->nominal_bitrate, 
+                                           codec->min_bitrate);
+      result |= vorbis_encode_ctl(&codec->enc_vi, OV_ECTL_RATEMANAGE_AVG, NULL);
+      result |= vorbis_encode_setup_init(&codec->enc_vi);
+      }
+    else
+      {
+      vorbis_encode_init(&codec->enc_vi,
+                         track_map->channels,
+                         samplerate, 
+                         codec->max_bitrate, 
+                         codec->nominal_bitrate, 
+                         codec->min_bitrate);
       }
-    
-    result = 0;
 
-    // FLUSH_OGG2
-    //    flush_data(file, track);
-    //    fprintf(stderr, "Encoded %lld samples\n",
-    //            codec->enc_os.granulepos - codec->encoded_samples);
-    
 
-// Wrote a chunk.
-	if(codec->chunk_started)
-	{
-		int new_encoded_samples = codec->enc_os.granulepos;
+    vorbis_comment_init(&codec->enc_vc);
+    vorbis_analysis_init(&codec->enc_vd, &codec->enc_vi);
+    vorbis_block_init(&codec->enc_vd, &codec->enc_vb);
+    //    	srand(time(NULL));
+    ogg_stream_init(&codec->enc_os, rand());
+
+    vorbis_analysis_headerout(&codec->enc_vd, 
+                              &codec->enc_vc,
+                              &header,
+                              &header_comm,
+                              &header_code);
+
+    ogg_stream_packetin(&codec->enc_os, &header); 
+    ogg_stream_packetin(&codec->enc_os, &header_comm);
+    ogg_stream_packetin(&codec->enc_os, &header_code);
+
+    //        FLUSH_OGG1
+    flush_header(file, track);
+
+    codec->sample_buffer = alloc_sample_buffer(codec->sample_buffer, track_map->channels,
+                                               ENCODE_SAMPLES,
+                                               &codec->sample_buffer_alloc);
+    }
+        
+  samples_copied = 0;
+
+  while(samples_copied < samples)
+    {
+    samples_to_copy = samples - samples_copied;
+    if(samples_to_copy > ENCODE_SAMPLES - codec->enc_samples_in_buffer)
+      samples_to_copy = ENCODE_SAMPLES - codec->enc_samples_in_buffer;
+    input = ((float*)_input + samples_copied * track_map->channels);
+    
+    for(j = 0; j < samples_to_copy; j++)
+      {
+      for(i = 0; i < track_map->channels; i++)
+        codec->sample_buffer[i][codec->enc_samples_in_buffer + j] = *(input++);
+      
+      }
+    samples_copied += samples_to_copy;
+    codec->enc_samples_in_buffer += samples_to_copy;
+    
+    if(codec->enc_samples_in_buffer >= ENCODE_SAMPLES)
+      flush_audio(file, track);
+    }
+    
+  result = 0;
+
+  //    fprintf(stderr, "Encoded %lld samples\n",
+  //            codec->enc_os.granulepos - codec->encoded_samples);
+  
+  // Wrote a chunk.
+  if(codec->chunk_started)
+    {
 #if 0
-                fprintf(stderr, "WRITING CHUNK: %lld %lld\n",
-                        codec->enc_os.granulepos, codec->enc_vd.granulepos);
+    fprintf(stderr, "WRITING CHUNK: %lld %lld\n",
+            codec->enc_os.granulepos, codec->enc_vd.granulepos);
 #endif           
-		quicktime_write_chunk_footer(file, 
-						trak,
-						track_map->current_chunk,
-						&codec->chunk_atom, 
-						new_encoded_samples - codec->encoded_samples);
-		track_map->current_chunk++;
-		codec->encoded_samples = new_encoded_samples;
-                codec->chunk_started = 0;
-	}
+    quicktime_write_chunk_footer(file, 
+                                 trak,
+                                 track_map->current_chunk,
+                                 &codec->chunk_atom, 
+                                 track_map->vbr_num_frames);
+    track_map->current_chunk++;
+    codec->chunk_started = 0;
+    }
 
-	return result;
-}
+  return result;
+  }
 
 
 
@@ -763,47 +818,49 @@ static int set_parameter(quicktime_t *file,
 	else
 	if(!strcasecmp(key, "vorbis_min_bitrate"))
 		codec->min_bitrate = *(int*)value;
-	return 0;
+        return 0;
 }
 
 
-static void flush(quicktime_t *file, int track)
+static int flush(quicktime_t *file, int track)
 {
 	quicktime_audio_map_t *track_map = &(file->atracks[track]);
 	quicktime_vorbis_codec_t *codec = ((quicktime_codec_t*)track_map->codec)->priv;
 	quicktime_trak_t *trak = track_map->track;
 
 //printf("flush 1\n");
+        flush_audio(file, track);
 	vorbis_analysis_wrote(&codec->enc_vd,0);
 
-        flush_audio(file, track);
+        flush_data(file, track);
         
         //	FLUSH_OGG2
 	
 //printf("flush 2 %d\n", size);
 	if(codec->chunk_started)
 	{
-		int new_encoded_samples = codec->enc_os.granulepos;
 #if 0
                 fprintf(stderr, "WRITING CHUNK: %lld %lld\n",
                         (long long)codec->enc_os.granulepos,
                         (long long)codec->enc_vd.granulepos);
 #endif
                 quicktime_write_chunk_footer(file, 
-						trak,
-						track_map->current_chunk,
-						&codec->chunk_atom, 
-						new_encoded_samples - codec->encoded_samples);
+                                             trak,
+                                             track_map->current_chunk,
+                                             &codec->chunk_atom, 
+                                             track_map->vbr_num_frames);
 		track_map->current_chunk++;
                 codec->chunk_started = 0;
-
+                return 1;
 	}
+        return 0;
 }
 
 void quicktime_init_codec_vorbis(quicktime_audio_map_t *atrack)
 {
 	quicktime_codec_t *codec_base = (quicktime_codec_t*)atrack->codec;
 	quicktime_vorbis_codec_t *codec;
+	char *compressor = atrack->track->mdia.minf.stbl.stsd.table[0].format;
 
 /* Init public items */
 	codec_base->priv = calloc(1, sizeof(quicktime_vorbis_codec_t));
@@ -812,12 +869,30 @@ void quicktime_init_codec_vorbis(quicktime_audio_map_t *atrack)
 	codec_base->encode_audio = encode;
 	codec_base->set_parameter = set_parameter;
 	codec_base->flush = flush;
-	codec_base->fourcc = QUICKTIME_VORBIS;
-	codec_base->title = "OGG Vorbis";
-	codec_base->desc = "OGG Vorbis for video. (Not standardized)";
-
+        
 	codec = codec_base->priv;
 	codec->nominal_bitrate = 128000;
 	codec->max_bitrate = -1;
 	codec->min_bitrate = -1;
+        atrack->sample_format = LQT_SAMPLE_FLOAT;
+        if(quicktime_match_32(compressor, "OggV"))
+          {
+          codec->write_OVHS = 1;
+          }
+        /* Set Vorbis 5.1 channel mapping */
+        if(atrack->channels == 6)
+          {
+          if(!atrack->channel_setup)
+            {
+            atrack->channel_setup = calloc(6, sizeof(*atrack->channel_setup));
+            atrack->channel_setup[0] =  LQT_CHANNEL_FRONT_LEFT;
+            atrack->channel_setup[1] =  LQT_CHANNEL_FRONT_CENTER;
+            atrack->channel_setup[2] =  LQT_CHANNEL_FRONT_RIGHT;
+            atrack->channel_setup[3] =  LQT_CHANNEL_LFE;
+            atrack->channel_setup[4] =  LQT_CHANNEL_BACK_LEFT;
+            atrack->channel_setup[5] =  LQT_CHANNEL_BACK_RIGHT;
+
+            }
+          }
+
 }
