@@ -14,11 +14,21 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 
  * USA
  */
+
+#define HAVE_RGB_16_TO_RGB_24
+#define HAVE_RGB_TO_YUV
+#define HAVE_RGB_TO_YUVJ
+#define HAVE_YUV_TO_RGB
+#define HAVE_YUV_8_TO_YUVJ
+#define HAVE_YUVJ_TO_YUV_8
+
+#include <inttypes.h>
 #include <cmodel_permutation.h>
+#include <lqt.h>
+#include <lqt_funcprotos.h>
 
 
 #define TRANSFER_FRAME_DEFAULT(output, \
-	input, \
 	y_in_offset, \
 	u_in_offset, \
 	v_in_offset, \
@@ -31,24 +41,16 @@
 		case BC_YUV420P: \
 			switch(out_colormodel) \
 			{ \
-				case BC_RGB8: \
-					TRANSFER_YUV420P_IN_HEAD \
-					transfer_YUV422P_to_RGB8((output), \
-						input_y + (y_in_offset), \
-						input_u + (u_in_offset), \
-						input_v + (v_in_offset)); \
-					TRANSFER_FRAME_TAIL \
-					break; \
 				case BC_BGR565: \
-					TRANSFER_YUV420P_IN_HEAD \
-					transfer_YUV422P_to_BGR565((output), \
+					TRANSFER_YUV420P_IN_HEAD_16 \
+                                          transfer_YUV422P_to_BGR565((output), \
 						input_y + (y_in_offset), \
 						input_u + (u_in_offset), \
 						input_v + (v_in_offset)); \
 					TRANSFER_FRAME_TAIL \
 					break; \
 				case BC_RGB565: \
-					TRANSFER_YUV420P_IN_HEAD \
+					TRANSFER_YUV420P_IN_HEAD_16 \
 					transfer_YUV422P_to_RGB565((output), \
 						input_y + (y_in_offset), \
 						input_u + (u_in_offset), \
@@ -74,12 +76,12 @@
 				case BC_YUV420P: \
 					for(i = 0; i < out_h; i++) \
 					{ \
-						unsigned char *output_y = out_y_plane + i * total_out_w; \
-						unsigned char *output_u = out_u_plane + i / 2 * total_out_w / 2; \
-						unsigned char *output_v = out_v_plane + i / 2 * total_out_w / 2; \
-						unsigned char *input_y = in_y_plane + row_table[i] * total_in_w; \
-						unsigned char *input_u = in_u_plane + row_table[i] / 2 * total_in_w / 2; \
-						unsigned char *input_v = in_v_plane + row_table[i] / 2 * total_in_w / 2; \
+						unsigned char *output_y = output_rows[0] + i * out_rowspan; \
+						unsigned char *output_u = output_rows[1] + i / 2 * out_rowspan_uv; \
+						unsigned char *output_v = output_rows[2] + i / 2 * out_rowspan_uv; \
+						unsigned char *input_y = input_rows[0] + row_table[i] * in_rowspan; \
+						unsigned char *input_u = input_rows[1] + row_table[i] / 2 * in_rowspan_uv; \
+						unsigned char *input_v = input_rows[2] + row_table[i] / 2 * in_rowspan_uv; \
 						for(j = 0; j < out_w; j++) \
 						{ \
 							transfer_YUV422P_to_YUV420P(input_y + (y_in_offset), \
@@ -104,12 +106,12 @@
 				case BC_YUV422P: \
 					for(i = 0; i < out_h; i++) \
 					{ \
-						unsigned char *output_y = out_y_plane + i * total_out_w; \
-						unsigned char *output_u = out_u_plane + i * total_out_w / 2; \
-						unsigned char *output_v = out_v_plane + i * total_out_w / 2; \
-						unsigned char *input_y = in_y_plane + row_table[i] * total_in_w; \
-						unsigned char *input_u = in_u_plane + row_table[i] / 2 * total_in_w / 2; \
-						unsigned char *input_v = in_v_plane + row_table[i] / 2 * total_in_w / 2; \
+						unsigned char *output_y = output_rows[0] + i * out_rowspan; \
+						unsigned char *output_u = output_rows[1] + i * out_rowspan_uv; \
+						unsigned char *output_v = output_rows[2] + i * out_rowspan_uv; \
+						unsigned char *input_y = input_rows[0] + row_table[i] * in_rowspan; \
+						unsigned char *input_u = input_rows[1] + row_table[i] / 2 * in_rowspan_uv; \
+						unsigned char *input_v = input_rows[2] + row_table[i] / 2 * in_rowspan_uv; \
 						for(j = 0; j < out_w; j++) \
 						{ \
 							transfer_YUV422P_to_YUV420P(input_y + (y_in_offset), \
@@ -125,12 +127,12 @@
 				case BC_YUV444P: \
 					for(i = 0; i < out_h; i++) \
 					{ \
-						unsigned char *output_y = out_y_plane + i * total_out_w; \
-						unsigned char *output_u = out_u_plane + i * total_out_w; \
-						unsigned char *output_v = out_v_plane + i * total_out_w; \
-						unsigned char *input_y = in_y_plane + row_table[i] * total_in_w; \
-						unsigned char *input_u = in_u_plane + row_table[i] / 2 * total_in_w / 2; \
-						unsigned char *input_v = in_v_plane + row_table[i] / 2 * total_in_w / 2; \
+						unsigned char *output_y = output_rows[0] + i * out_rowspan; \
+						unsigned char *output_u = output_rows[1] + i * out_rowspan_uv; \
+						unsigned char *output_v = output_rows[2] + i * out_rowspan_uv; \
+						unsigned char *input_y = input_rows[0] + row_table[i] * in_rowspan; \
+						unsigned char *input_u = input_rows[1] + row_table[i] / 2 * in_rowspan_uv; \
+						unsigned char *input_v = input_rows[2] + row_table[i] / 2 * in_rowspan_uv; \
 						for(j = 0; j < out_w; j++) \
 						{ \
 							transfer_YUV422P_to_YUV444P(input_y + (y_in_offset), \
@@ -151,22 +153,6 @@
 						input_v + (v_in_offset)); \
 					TRANSFER_FRAME_TAIL \
 					break; \
-				case BC_ARGB8888:      \
-					TRANSFER_YUV420P_IN_HEAD \
-					transfer_YUV422P_to_ARGB8888((output), \
-						input_y + (y_in_offset), \
-						input_u + (u_in_offset), \
-						input_v + (v_in_offset)); \
-					TRANSFER_FRAME_TAIL \
-					break; \
-				case BC_ABGR8888:      \
-					TRANSFER_YUV420P_IN_HEAD \
-					transfer_YUV422P_to_ABGR8888((output), \
-						input_y + (y_in_offset), \
-						input_u + (u_in_offset), \
-						input_v + (v_in_offset)); \
-					TRANSFER_FRAME_TAIL \
-					break; \
 				case BC_RGBA8888:      \
 					TRANSFER_YUV420P_IN_HEAD \
 					transfer_YUV422P_to_RGBA8888((output), \
@@ -191,14 +177,6 @@
 						input_v + (v_in_offset)); \
 					TRANSFER_FRAME_TAIL \
 					break; \
-				case BC_YUV888: \
-					TRANSFER_YUV420P_IN_HEAD \
-					transfer_YUV422P_to_YUV888((output), \
-						input_y + (y_in_offset), \
-						input_u + (u_in_offset), \
-						input_v + (v_in_offset)); \
-					TRANSFER_FRAME_TAIL \
-					break; \
 				case BC_YUVA8888: \
 					TRANSFER_YUV420P_IN_HEAD \
 					transfer_YUV422P_to_YUVA8888((output), \
@@ -207,17 +185,14 @@
 						input_v + (v_in_offset)); \
 					TRANSFER_FRAME_TAIL \
 					break; \
-				case BC_YUV161616: \
-					TRANSFER_YUV420P_IN_HEAD_16 \
-                                        transfer_YUV422P_to_YUV161616((output), \
-						input_y + (y_in_offset), \
-						input_u + (u_in_offset), \
-						input_v + (v_in_offset)); \
-					TRANSFER_FRAME_TAIL \
-					break; \
-				case BC_YUVA16161616: \
-					TRANSFER_YUV420P_IN_HEAD_16 \
-					transfer_YUV422P_to_YUVA16161616((output), \
+			} \
+			break; \
+		case BC_YUV411P: \
+			switch(out_colormodel) \
+			{ \
+				case BC_RGB888:      \
+					TRANSFER_YUV411P_IN_HEAD \
+					transfer_YUV411P_to_RGB888((output), \
 						input_y + (y_in_offset), \
 						input_u + (u_in_offset), \
 						input_v + (v_in_offset)); \
@@ -226,19 +201,167 @@
 			} \
 			break; \
  \
-		case BC_YUV422P: \
+		case BC_YUVJ444P: \
 			switch(out_colormodel) \
 			{ \
-				case BC_RGB8: \
-					TRANSFER_YUV422P_IN_HEAD \
-					transfer_YUV422P_to_RGB8((output), \
+				case BC_RGB888:      \
+					TRANSFER_YUV444P_IN_HEAD \
+					transfer_YUVJ444P_to_RGB888((output), \
 						input_y + (y_in_offset), \
 						input_u + (u_in_offset), \
 						input_v + (v_in_offset)); \
 					TRANSFER_FRAME_TAIL \
 					break; \
-				case BC_BGR565: \
+				case BC_YUVJ444P:      \
+					for(i = 0; i < out_h; i++) \
+					{ \
+						unsigned char *output_y = output_rows[0] + i * out_rowspan; \
+						unsigned char *output_u = output_rows[1] + i * out_rowspan_uv; \
+						unsigned char *output_v = output_rows[2] + i * out_rowspan_uv; \
+						unsigned char *input_y = input_rows[0] + row_table[i] * in_rowspan; \
+						unsigned char *input_u = input_rows[1] + row_table[i] * in_rowspan_uv; \
+						unsigned char *input_v = input_rows[2] + row_table[i] * in_rowspan_uv; \
+						for(j = 0; j < out_w; j++) \
+						{ \
+							transfer_YUV444P_to_YUV444P(input_y + (y_in_offset), \
+								input_u + (u_in_offset), \
+								input_v + (v_in_offset), \
+								output_y, \
+								output_u, \
+								output_v, \
+								j); \
+						} \
+					} \
+					break; \
+			} \
+			break; \
+ \
+		case BC_YUVJ422P: \
+			switch(out_colormodel) \
+			{ \
+				case BC_RGB888:      \
 					TRANSFER_YUV422P_IN_HEAD \
+					transfer_YUVJ422P_to_RGB888((output), \
+						input_y + (y_in_offset), \
+						input_u + (u_in_offset), \
+						input_v + (v_in_offset)); \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_YUVJ422P: \
+					for(i = 0; i < out_h; i++) \
+					{ \
+						unsigned char *output_y = output_rows[0] + i * out_rowspan; \
+						unsigned char *output_u = output_rows[1] + i * out_rowspan_uv; \
+						unsigned char *output_v = output_rows[2] + i * out_rowspan_uv; \
+						unsigned char *input_y = input_rows[0] + row_table[i] * in_rowspan; \
+						unsigned char *input_u = input_rows[1] + row_table[i] * in_rowspan_uv; \
+						unsigned char *input_v = input_rows[2] + row_table[i] * in_rowspan_uv; \
+						for(j = 0; j < out_w; j++) \
+						{ \
+							transfer_YUV422P_to_YUV420P(input_y + (y_in_offset), \
+								input_u + (u_in_offset), \
+								input_v + (v_in_offset), \
+								output_y, \
+								output_u, \
+								output_v, \
+								j); \
+						} \
+					} \
+					break; \
+                               case BC_YUV420P: \
+                                       for(i = 0; i < out_h; i++) \
+                                       { \
+                                                unsigned char *output_y = output_rows[0] + i * out_rowspan; \
+                                                unsigned char *output_u = output_rows[1] + i / 2 * out_rowspan_uv; \
+                                                unsigned char *output_v = output_rows[2] + i / 2 * out_rowspan_uv; \
+                                                unsigned char *input_y = input_rows[0] + row_table[i] * in_rowspan; \
+                                                unsigned char *input_u = input_rows[1] + row_table[i] * in_rowspan_uv; \
+                                                unsigned char *input_v = input_rows[2] + row_table[i] * in_rowspan_uv; \
+                                                for(j = 0; j < out_w; j++) \
+                                                { \
+                                                        transfer_YUVJ422P_to_YUV420P(input_y + (y_in_offset), \
+                                                                input_u + (u_in_offset), \
+                                                                input_v + (v_in_offset), \
+                                                                output_y, \
+                                                                output_u, \
+                                                                output_v, \
+                                                                j); \
+                                                } \
+                                        } \
+                                        break; \
+				case BC_YUV422P: \
+					for(i = 0; i < out_h; i++) \
+					{ \
+						unsigned char *output_y = output_rows[0] + i * out_rowspan; \
+						unsigned char *output_u = output_rows[1] + i * out_rowspan_uv; \
+						unsigned char *output_v = output_rows[2] + i * out_rowspan_uv; \
+						unsigned char *input_y = input_rows[0] + row_table[i] * in_rowspan; \
+						unsigned char *input_u = input_rows[1] + row_table[i] * in_rowspan_uv; \
+						unsigned char *input_v = input_rows[2] + row_table[i] * in_rowspan_uv; \
+						for(j = 0; j < out_w; j++) \
+						{ \
+							transfer_YUVJ422P_to_YUV420P(input_y + (y_in_offset), \
+								input_u + (u_in_offset), \
+								input_v + (v_in_offset), \
+								output_y, \
+								output_u, \
+								output_v, \
+								j); \
+						} \
+					} \
+					break; \
+				case BC_YUV422:      \
+					TRANSFER_YUV422_IN_HEAD \
+					transfer_YUVJ422P_to_YUV422((output), \
+						input_y + (y_in_offset), \
+						input_u + (u_in_offset), \
+						input_v + (v_in_offset), \
+						j); \
+					TRANSFER_FRAME_TAIL \
+					break; \
+			} \
+			break; \
+ \
+		case BC_YUVJ420P: \
+			switch(out_colormodel) \
+			{ \
+				case BC_RGB888:      \
+					TRANSFER_YUV420P_IN_HEAD \
+					transfer_YUVJ420P_to_RGB888((output), \
+						input_y + (y_in_offset), \
+						input_u + (u_in_offset), \
+						input_v + (v_in_offset)); \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_YUVJ420P: \
+					for(i = 0; i < out_h; i++) \
+					{ \
+						unsigned char *output_y = output_rows[0] + i * out_rowspan; \
+						unsigned char *output_u = output_rows[1] + i / 2 * out_rowspan_uv; \
+						unsigned char *output_v = output_rows[2] + i / 2 * out_rowspan_uv; \
+						unsigned char *input_y = input_rows[0] + row_table[i] * in_rowspan; \
+						unsigned char *input_u = input_rows[1] + row_table[i] / 2 * in_rowspan_uv; \
+						unsigned char *input_v = input_rows[2] + row_table[i] / 2 * in_rowspan_uv; \
+						for(j = 0; j < out_w; j++) \
+						{ \
+							transfer_YUV422P_to_YUV420P(input_y + (y_in_offset), \
+								input_u + (u_in_offset), \
+								input_v + (v_in_offset), \
+								output_y, \
+								output_u, \
+								output_v, \
+								j); \
+						} \
+					} \
+					break; \
+			} \
+			break; \
+ \
+		case BC_YUV422P: \
+			switch(out_colormodel) \
+			{ \
+				case BC_BGR565: \
+					TRANSFER_YUV422P_IN_HEAD_16 \
 					transfer_YUV422P_to_BGR565((output), \
 						input_y + (y_in_offset), \
 						input_u + (u_in_offset), \
@@ -246,7 +369,7 @@
 					TRANSFER_FRAME_TAIL \
 					break; \
 				case BC_RGB565: \
-					TRANSFER_YUV422P_IN_HEAD \
+					TRANSFER_YUV422P_IN_HEAD_16 \
 					transfer_YUV422P_to_RGB565((output), \
 						input_y + (y_in_offset), \
 						input_u + (u_in_offset), \
@@ -277,22 +400,6 @@
 						input_v + (v_in_offset)); \
 					TRANSFER_FRAME_TAIL \
 					break; \
-				case BC_ARGB8888:      \
-					TRANSFER_YUV422P_IN_HEAD \
-					transfer_YUV422P_to_ARGB8888((output), \
-						input_y + (y_in_offset), \
-						input_u + (u_in_offset), \
-						input_v + (v_in_offset)); \
-					TRANSFER_FRAME_TAIL \
-					break; \
-				case BC_ABGR8888:      \
-					TRANSFER_YUV422P_IN_HEAD \
-					transfer_YUV422P_to_ABGR8888((output), \
-						input_y + (y_in_offset), \
-						input_u + (u_in_offset), \
-						input_v + (v_in_offset)); \
-					TRANSFER_FRAME_TAIL \
-					break; \
 				case BC_RGBA8888:      \
 					TRANSFER_YUV422P_IN_HEAD \
 					transfer_YUV422P_to_RGBA8888((output), \
@@ -317,14 +424,6 @@
 						input_v + (v_in_offset)); \
 					TRANSFER_FRAME_TAIL \
 					break; \
-				case BC_YUV888: \
-					TRANSFER_YUV422P_IN_HEAD \
-					transfer_YUV422P_to_YUV888((output), \
-						input_y + (y_in_offset), \
-						input_u + (u_in_offset), \
-						input_v + (v_in_offset)); \
-					TRANSFER_FRAME_TAIL \
-					break; \
 				case BC_YUVA8888: \
 					TRANSFER_YUV422P_IN_HEAD \
 					transfer_YUV422P_to_YUVA8888((output), \
@@ -333,31 +432,15 @@
 						input_v + (v_in_offset)); \
 					TRANSFER_FRAME_TAIL \
 					break; \
-				case BC_YUV161616: \
-					TRANSFER_YUV422P_IN_HEAD_16 \
-                                          transfer_YUV422P_to_YUV161616((output), \
-						input_y + (y_in_offset), \
-						input_u + (u_in_offset), \
-						input_v + (v_in_offset)); \
-					TRANSFER_FRAME_TAIL \
-					break; \
-				case BC_YUVA16161616: \
-					TRANSFER_YUV422P_IN_HEAD_16 \
-					transfer_YUV422P_to_YUVA16161616((output), \
-						input_y + (y_in_offset), \
-						input_u + (u_in_offset), \
-						input_v + (v_in_offset)); \
-					TRANSFER_FRAME_TAIL \
-					break; \
 				case BC_YUV420P: \
 					for(i = 0; i < out_h; i++) \
 					{ \
-						unsigned char *output_y = out_y_plane + i * total_in_w; \
-						unsigned char *output_u = out_u_plane + i / 2 * total_in_w / 2; \
-						unsigned char *output_v = out_v_plane + i / 2 * total_in_w / 2; \
-						unsigned char *input_y = in_y_plane + row_table[i] * total_in_w; \
-						unsigned char *input_u = in_u_plane + row_table[i] * total_in_w / 2; \
-						unsigned char *input_v = in_v_plane + row_table[i] * total_in_w / 2; \
+						unsigned char *output_y = output_rows[0] + i * out_rowspan; \
+						unsigned char *output_u = output_rows[1] + i / 2 * out_rowspan_uv; \
+						unsigned char *output_v = output_rows[2] + i / 2 * out_rowspan_uv; \
+						unsigned char *input_y = input_rows[0] + row_table[i] * in_rowspan; \
+						unsigned char *input_u = input_rows[1] + row_table[i] * in_rowspan_uv; \
+						unsigned char *input_v = input_rows[2] + row_table[i] * in_rowspan_uv; \
 						for(j = 0; j < out_w; j++) \
 						{ \
 							transfer_YUV422P_to_YUV420P(input_y + (y_in_offset), \
@@ -382,12 +465,12 @@
 				case BC_YUV422P: \
 					for(i = 0; i < out_h; i++) \
 					{ \
-						unsigned char *output_y = out_y_plane + i * total_in_w; \
-						unsigned char *output_u = out_u_plane + i * total_in_w / 2; \
-						unsigned char *output_v = out_v_plane + i * total_in_w / 2; \
-						unsigned char *input_y = in_y_plane + row_table[i] * total_in_w; \
-						unsigned char *input_u = in_u_plane + row_table[i] * total_in_w / 2; \
-						unsigned char *input_v = in_v_plane + row_table[i] * total_in_w / 2; \
+						unsigned char *output_y = output_rows[0] + i * out_rowspan; \
+						unsigned char *output_u = output_rows[1] + i * out_rowspan_uv; \
+						unsigned char *output_v = output_rows[2] + i * out_rowspan_uv; \
+						unsigned char *input_y = input_rows[0] + row_table[i] * in_rowspan; \
+						unsigned char *input_u = input_rows[1] + row_table[i] * in_rowspan_uv; \
+						unsigned char *input_v = input_rows[2] + row_table[i] * in_rowspan_uv; \
 						for(j = 0; j < out_w; j++) \
 						{ \
 							transfer_YUV422P_to_YUV420P(input_y + (y_in_offset), \
@@ -403,12 +486,12 @@
 				case BC_YUV444P: \
 					for(i = 0; i < out_h; i++) \
 					{ \
-						unsigned char *output_y = out_y_plane + i * total_in_w; \
-						unsigned char *output_u = out_u_plane + i * total_in_w; \
-						unsigned char *output_v = out_v_plane + i * total_in_w; \
-						unsigned char *input_y = in_y_plane + row_table[i] * total_in_w; \
-						unsigned char *input_u = in_u_plane + row_table[i] * total_in_w / 2; \
-						unsigned char *input_v = in_v_plane + row_table[i] * total_in_w / 2; \
+						unsigned char *output_y = output_rows[0] + i * out_rowspan; \
+						unsigned char *output_u = output_rows[1] + i * out_rowspan_uv; \
+						unsigned char *output_v = output_rows[2] + i * out_rowspan_uv; \
+						unsigned char *input_y = input_rows[0] + row_table[i] * in_rowspan; \
+						unsigned char *input_u = input_rows[1] + row_table[i] * in_rowspan_uv; \
+						unsigned char *input_v = input_rows[2] + row_table[i] * in_rowspan_uv; \
 						for(j = 0; j < out_w; j++) \
 						{ \
 							transfer_YUV422P_to_YUV444P(input_y + (y_in_offset), \
@@ -421,23 +504,145 @@
 						} \
 					} \
 					break; \
+				case BC_YUVJ422P: \
+					for(i = 0; i < out_h; i++) \
+					{ \
+						unsigned char *output_y = output_rows[0] + i * out_rowspan; \
+						unsigned char *output_u = output_rows[1] + i * out_rowspan_uv; \
+						unsigned char *output_v = output_rows[2] + i * out_rowspan_uv; \
+						unsigned char *input_y = input_rows[0] + row_table[i] * in_rowspan; \
+						unsigned char *input_u = input_rows[1] + row_table[i] * in_rowspan_uv; \
+						unsigned char *input_v = input_rows[2] + row_table[i] * in_rowspan_uv; \
+						for(j = 0; j < out_w; j++) \
+						{ \
+							transfer_YUV422P_to_YUVJ420P(input_y + (y_in_offset), \
+								input_u + (u_in_offset), \
+								input_v + (v_in_offset), \
+								output_y, \
+								output_u, \
+								output_v, \
+								j); \
+						} \
+					} \
+					break; \
 			} \
 			break; \
  \
- \
-		case BC_YUV444P: \
+		case BC_YUV422P16: \
 			switch(out_colormodel) \
 			{ \
-				case BC_RGB8: \
-					TRANSFER_YUV444P_IN_HEAD \
-					transfer_YUV422P_to_RGB8((output), \
+				case BC_RGB888:      \
+					TRANSFER_YUV422P16_IN_HEAD \
+					transfer_YUV422P16_to_RGB888((output), \
 						input_y + (y_in_offset), \
 						input_u + (u_in_offset), \
 						input_v + (v_in_offset)); \
 					TRANSFER_FRAME_TAIL \
 					break; \
+      				case BC_YUV422P16: \
+					for(i = 0; i < out_h; i++) \
+					{ \
+                                        uint16_t *output_y = (uint16_t *)(output_rows[0] + i * out_rowspan); \
+                                        uint16_t *output_u = (uint16_t *)(output_rows[1] + i * out_rowspan_uv); \
+                                        uint16_t *output_v = (uint16_t *)(output_rows[2] + i * out_rowspan_uv); \
+                                        uint16_t *input_y = (uint16_t *)(input_rows[0] + row_table[i] * in_rowspan); \
+                                        uint16_t *input_u = (uint16_t *)(input_rows[1] + row_table[i] * in_rowspan_uv); \
+                                        uint16_t *input_v = (uint16_t *)(input_rows[2] + row_table[i] * in_rowspan_uv); \
+						for(j = 0; j < out_w; j++) \
+						{ \
+							transfer_YUV422P16_to_YUV420P16(input_y + (y_in_offset), \
+								input_u + (u_in_offset), \
+								input_v + (v_in_offset), \
+								output_y, \
+								output_u, \
+								output_v, \
+								j); \
+						} \
+					} \
+					break; \
+      				case BC_YUV422P: \
+					for(i = 0; i < out_h; i++) \
+					{ \
+                                        uint8_t *output_y = (uint8_t *)(output_rows[0] + i * out_rowspan); \
+                                        uint8_t *output_u = (uint8_t *)(output_rows[1] + i * out_rowspan_uv); \
+                                        uint8_t *output_v = (uint8_t *)(output_rows[2] + i * out_rowspan_uv); \
+                                        uint16_t *input_y = (uint16_t *)(input_rows[0] + row_table[i] * in_rowspan); \
+                                        uint16_t *input_u = (uint16_t *)(input_rows[1] + row_table[i] * in_rowspan_uv); \
+                                        uint16_t *input_v = (uint16_t *)(input_rows[2] + row_table[i] * in_rowspan_uv); \
+						for(j = 0; j < out_w; j++) \
+						{ \
+							transfer_YUV422P16_to_YUV420P(input_y + (y_in_offset), \
+								input_u + (u_in_offset), \
+								input_v + (v_in_offset), \
+								output_y, \
+								output_u, \
+								output_v, \
+								j); \
+						} \
+					} \
+					break; \
+                        }                              \
+			break; \
+		case BC_YUV444P16: \
+			switch(out_colormodel) \
+			{ \
+				case BC_RGB888:      \
+					TRANSFER_YUV444P16_IN_HEAD \
+					transfer_YUV444P16_to_RGB888((output), \
+						input_y + (y_in_offset), \
+						input_u + (u_in_offset), \
+						input_v + (v_in_offset)); \
+					TRANSFER_FRAME_TAIL \
+					break; \
+                                case BC_YUV444P16:                      \
+					for(i = 0; i < out_h; i++) \
+					{ \
+                                        uint16_t *output_y = (uint16_t *)(output_rows[0] + i * out_rowspan); \
+                                        uint16_t *output_u = (uint16_t *)(output_rows[1] + i * out_rowspan_uv); \
+                                        uint16_t *output_v = (uint16_t *)(output_rows[2] + i * out_rowspan_uv); \
+                                        uint16_t *input_y = (uint16_t *)(input_rows[0] + row_table[i] * in_rowspan); \
+                                        uint16_t *input_u = (uint16_t *)(input_rows[1] + row_table[i] * in_rowspan_uv); \
+                                        uint16_t *input_v = (uint16_t *)(input_rows[2] + row_table[i] * in_rowspan_uv); \
+						for(j = 0; j < out_w; j++) \
+						{ \
+							transfer_YUV444P16_to_YUV444P16(input_y + (y_in_offset), \
+								input_u + (u_in_offset), \
+								input_v + (v_in_offset), \
+								output_y, \
+								output_u, \
+								output_v, \
+								j); \
+						} \
+					} \
+					break; \
+                                case BC_YUV444P:                      \
+					for(i = 0; i < out_h; i++) \
+					{ \
+                                        uint8_t *output_y = (uint8_t *)(output_rows[0] + i * out_rowspan); \
+                                        uint8_t *output_u = (uint8_t *)(output_rows[1] + i * out_rowspan_uv); \
+                                        uint8_t *output_v = (uint8_t *)(output_rows[2] + i * out_rowspan_uv); \
+                                        uint16_t *input_y = (uint16_t *)(input_rows[0] + row_table[i] * in_rowspan); \
+                                        uint16_t *input_u = (uint16_t *)(input_rows[1] + row_table[i] * in_rowspan_uv); \
+                                        uint16_t *input_v = (uint16_t *)(input_rows[2] + row_table[i] * in_rowspan_uv); \
+						for(j = 0; j < out_w; j++) \
+						{ \
+							transfer_YUV444P16_to_YUV444P(input_y + (y_in_offset), \
+								input_u + (u_in_offset), \
+								input_v + (v_in_offset), \
+								output_y, \
+								output_u, \
+								output_v, \
+								j); \
+						} \
+					} \
+					break; \
+			} \
+			break; \
+		case BC_YUV444P: \
+			switch(out_colormodel) \
+			{ \
 				case BC_BGR565: \
-					TRANSFER_YUV444P_IN_HEAD \
+					TRANSFER_YUV444P_IN_HEAD_16 \
 					transfer_YUV422P_to_BGR565((output), \
 						input_y + (y_in_offset), \
 						input_u + (u_in_offset), \
@@ -445,7 +650,7 @@
 					TRANSFER_FRAME_TAIL \
 					break; \
 				case BC_RGB565: \
-					TRANSFER_YUV444P_IN_HEAD \
+					TRANSFER_YUV444P_IN_HEAD_16 \
 					transfer_YUV422P_to_RGB565((output), \
 						input_y + (y_in_offset), \
 						input_u + (u_in_offset), \
@@ -476,22 +681,6 @@
 						input_v + (v_in_offset)); \
 					TRANSFER_FRAME_TAIL \
 					break; \
-				case BC_ARGB8888:      \
-					TRANSFER_YUV444P_IN_HEAD \
-					transfer_YUV422P_to_ARGB8888((output), \
-						input_y + (y_in_offset), \
-						input_u + (u_in_offset), \
-						input_v + (v_in_offset)); \
-					TRANSFER_FRAME_TAIL \
-					break; \
-				case BC_ABGR8888:      \
-					TRANSFER_YUV444P_IN_HEAD \
-					transfer_YUV422P_to_ABGR8888((output), \
-						input_y + (y_in_offset), \
-						input_u + (u_in_offset), \
-						input_v + (v_in_offset)); \
-					TRANSFER_FRAME_TAIL \
-					break; \
 				case BC_RGBA8888:      \
 					TRANSFER_YUV444P_IN_HEAD \
 					transfer_YUV422P_to_RGBA8888((output), \
@@ -516,14 +705,6 @@
 						input_v + (v_in_offset)); \
 					TRANSFER_FRAME_TAIL \
 					break; \
-				case BC_YUV888: \
-					TRANSFER_YUV444P_IN_HEAD \
-					transfer_YUV422P_to_YUV888((output), \
-						input_y + (y_in_offset), \
-						input_u + (u_in_offset), \
-						input_v + (v_in_offset)); \
-					TRANSFER_FRAME_TAIL \
-					break; \
 				case BC_YUVA8888: \
 					TRANSFER_YUV444P_IN_HEAD \
 					transfer_YUV422P_to_YUVA8888((output), \
@@ -532,31 +713,15 @@
 						input_v + (v_in_offset)); \
 					TRANSFER_FRAME_TAIL \
 					break; \
-				case BC_YUV161616: \
-					TRANSFER_YUV444P_IN_HEAD_16 \
-					transfer_YUV422P_to_YUV161616((output), \
-						input_y + (y_in_offset), \
-						input_u + (u_in_offset), \
-						input_v + (v_in_offset)); \
-					TRANSFER_FRAME_TAIL \
-					break; \
-				case BC_YUVA16161616: \
-					TRANSFER_YUV444P_IN_HEAD_16 \
-					transfer_YUV422P_to_YUVA16161616((output), \
-						input_y + (y_in_offset), \
-						input_u + (u_in_offset), \
-						input_v + (v_in_offset)); \
-					TRANSFER_FRAME_TAIL \
-					break; \
 				case BC_YUV420P: \
 					for(i = 0; i < out_h; i++) \
 					{ \
-						unsigned char *output_y = out_y_plane + i * total_in_w; \
-						unsigned char *output_u = out_u_plane + i / 2 * total_in_w / 2; \
-						unsigned char *output_v = out_v_plane + i / 2 * total_in_w / 2; \
-						unsigned char *input_y = in_y_plane + row_table[i] * total_in_w; \
-						unsigned char *input_u = in_u_plane + row_table[i] * total_in_w; \
-						unsigned char *input_v = in_v_plane + row_table[i] * total_in_w; \
+						unsigned char *output_y = output_rows[0] + i * out_rowspan; \
+						unsigned char *output_u = output_rows[1] + i / 2 * out_rowspan_uv; \
+						unsigned char *output_v = output_rows[2] + i / 2 * out_rowspan_uv; \
+						unsigned char *input_y = input_rows[0] + row_table[i] * in_rowspan; \
+						unsigned char *input_u = input_rows[1] + row_table[i] * in_rowspan_uv; \
+						unsigned char *input_v = input_rows[2] + row_table[i] * in_rowspan_uv; \
 						for(j = 0; j < out_w; j++) \
 						{ \
 							transfer_YUV422P_to_YUV420P(input_y + (y_in_offset), \
@@ -581,12 +746,12 @@
 				case BC_YUV422P: \
 					for(i = 0; i < out_h; i++) \
 					{ \
-						unsigned char *output_y = out_y_plane + i * total_in_w; \
-						unsigned char *output_u = out_u_plane + i * total_in_w / 2; \
-						unsigned char *output_v = out_v_plane + i * total_in_w / 2; \
-						unsigned char *input_y = in_y_plane + row_table[i] * total_in_w; \
-						unsigned char *input_u = in_u_plane + row_table[i] * total_in_w; \
-						unsigned char *input_v = in_v_plane + row_table[i] * total_in_w; \
+						unsigned char *output_y = output_rows[0] + i * out_rowspan; \
+						unsigned char *output_u = output_rows[1] + i * out_rowspan_uv; \
+						unsigned char *output_v = output_rows[2] + i * out_rowspan_uv; \
+						unsigned char *input_y = input_rows[0] + row_table[i] * in_rowspan; \
+						unsigned char *input_u = input_rows[1] + row_table[i] * in_rowspan_uv; \
+						unsigned char *input_v = input_rows[2] + row_table[i] * in_rowspan_uv; \
 						for(j = 0; j < out_w; j++) \
 						{ \
 							transfer_YUV422P_to_YUV420P(input_y + (y_in_offset), \
@@ -602,12 +767,12 @@
 				case BC_YUV444P: \
 					for(i = 0; i < out_h; i++) \
 					{ \
-						unsigned char *output_y = out_y_plane + i * total_in_w; \
-						unsigned char *output_u = out_u_plane + i * total_in_w; \
-						unsigned char *output_v = out_v_plane + i * total_in_w; \
-						unsigned char *input_y = in_y_plane + row_table[i] * total_in_w; \
-						unsigned char *input_u = in_u_plane + row_table[i] * total_in_w; \
-						unsigned char *input_v = in_v_plane + row_table[i] * total_in_w; \
+						unsigned char *output_y = output_rows[0] + i * out_rowspan; \
+						unsigned char *output_u = output_rows[1] + i * out_rowspan_uv; \
+						unsigned char *output_v = output_rows[2] + i * out_rowspan_uv; \
+						unsigned char *input_y = input_rows[0] + row_table[i] * in_rowspan; \
+						unsigned char *input_u = input_rows[1] + row_table[i] * in_rowspan_uv; \
+						unsigned char *input_v = input_rows[2] + row_table[i] * in_rowspan_uv; \
 						for(j = 0; j < out_w; j++) \
 						{ \
 							transfer_YUV444P_to_YUV444P(input_y + (y_in_offset), \
@@ -630,7 +795,6 @@ void cmodel_yuv420p(PERMUTATION_ARGS)
 	if(scale)
 	{
 		TRANSFER_FRAME_DEFAULT(&output_row, 
-			input_row + column_table[j] * in_pixelsize,
 			column_table[j],
 			column_table[j] / 2,
 			column_table[j] / 2,
@@ -639,7 +803,6 @@ void cmodel_yuv420p(PERMUTATION_ARGS)
 	else
 	{
 		TRANSFER_FRAME_DEFAULT(&output_row, 
-			input_row + j * in_pixelsize,
 			j,
 			j / 2,
 			j / 2,
@@ -647,12 +810,32 @@ void cmodel_yuv420p(PERMUTATION_ARGS)
 	}
 }
 
+void cmodel_yuv411p(PERMUTATION_ARGS)
+{
+	if(scale)
+	{
+		TRANSFER_FRAME_DEFAULT(&output_row, 
+			column_table[j],
+			column_table[j] / 4,
+			column_table[j] / 4,
+			0);
+	}
+	else
+	{
+		TRANSFER_FRAME_DEFAULT(&output_row, 
+			j,
+			j / 4,
+			j / 4,
+			0);
+	}
+}
+
+
 void cmodel_yuv444p(PERMUTATION_ARGS)
 {
 	if(scale)
 	{
 		TRANSFER_FRAME_DEFAULT(&output_row, 
-			input_row + column_table[j] * in_pixelsize,
 			column_table[j],
 			column_table[j],
 			column_table[j],
@@ -661,7 +844,6 @@ void cmodel_yuv444p(PERMUTATION_ARGS)
 	else
 	{
 		TRANSFER_FRAME_DEFAULT(&output_row, 
-			input_row + j * in_pixelsize,
 			j,
 			j,
 			j,
