@@ -15,13 +15,17 @@
  * USA
  */
 
+#define HAVE_RGB_16_TO_RGB_24
+#define HAVE_RGB_TO_YUV
+#define HAVE_RGB_TO_YUVJ
+#define HAVE_YUV_TO_RGB
+#define HAVE_YUV_8_TO_YUVJ
+#define HAVE_YUVJ_TO_YUV_8
 
-
-
+#include <inttypes.h>
 #include <cmodel_permutation.h>
-
-
-
+#include <lqt.h>
+#include <lqt_funcprotos.h>
 
 #define TRANSFER_FRAME_DEFAULT(output, \
 	input, \
@@ -34,15 +38,14 @@
  \
 	switch(out_colormodel) \
 	{ \
-		case BC_RGB8: \
-			TRANSFER_FRAME_HEAD \
-			transfer_YUV422_to_RGB8((output), (input), (input_column)); \
+		case BC_BGR565: \
+			TRANSFER_FRAME_HEAD_16 \
+                          transfer_YUV422_to_BGR565((output), (input), (input_column)); \
 			TRANSFER_FRAME_TAIL \
 			break; \
-		case BC_BGR565: \
 		case BC_RGB565: \
-			TRANSFER_FRAME_HEAD \
-			transfer_YUV422_to_RGB565((output), (input), (input_column)); \
+			TRANSFER_FRAME_HEAD_16 \
+                          transfer_YUV422_to_RGB565((output), (input), (input_column)); \
 			TRANSFER_FRAME_TAIL \
 			break; \
 		case BC_RGB888:      \
@@ -55,11 +58,6 @@
 			transfer_YUV422_to_RGBA8888((output), (input), (input_column)); \
 			TRANSFER_FRAME_TAIL \
 			break; \
-		case BC_YUV888:      \
-			TRANSFER_FRAME_HEAD \
-			transfer_YUV422_to_YUV888((output), (input), (input_column)); \
-			TRANSFER_FRAME_TAIL \
-			break; \
 		case BC_YUVA8888:      \
 			TRANSFER_FRAME_HEAD \
 			transfer_YUV422_to_YUVA8888((output), (input), (input_column)); \
@@ -75,16 +73,6 @@
 			transfer_YUV422_to_RGBA16161616((output), (input), (input_column)); \
 			TRANSFER_FRAME_TAIL \
 			break; \
-		case BC_YUV161616:      \
-			TRANSFER_FRAME_HEAD_16 \
-			transfer_YUV422_to_YUV161616((output), (input), (input_column)); \
-			TRANSFER_FRAME_TAIL \
-			break; \
-		case BC_YUVA16161616:      \
-			TRANSFER_FRAME_HEAD_16 \
-			transfer_YUV422_to_YUVA16161616((output), (input), (input_column)); \
-			TRANSFER_FRAME_TAIL \
-			break; \
 		case BC_BGR888:      \
 			TRANSFER_FRAME_HEAD \
 			transfer_YUV422_to_BGR888((output), (input), (input_column)); \
@@ -104,6 +92,15 @@
 				j); \
 			TRANSFER_FRAME_TAIL \
 			break; \
+		case BC_YUVJ422P: \
+			TRANSFER_YUV422P_OUT_HEAD \
+			transfer_YUV422_to_YUVJ422P(output_y, \
+				output_u, \
+				output_v, \
+				(input), \
+				j); \
+			TRANSFER_FRAME_TAIL \
+			break; \
 		case BC_YUV422: \
 			TRANSFER_FRAME_HEAD \
 			transfer_YUV422_to_YUV422((output), \
