@@ -56,6 +56,8 @@ static const char * fourccs_key         = "Fourccs: ";
 static const char * num_wav_ids_key     = "NumWavIds: ";
 static const char * wav_ids_key         = "WavIds: ";
 
+static const char * compatibility_key   = "Compatibility: ";
+
 /* Module filename and module index */
 
 static const char * module_filename_key  = "ModuleFilename: ";
@@ -65,10 +67,15 @@ static const char * module_file_time_key = "FileTime: ";
 /* Types for parameters */
 
 static const char * type_int            = "Integer";
+static const char * type_float          = "Float";
 static const char * type_string         = "String";
 static const char * type_stringlist     = "Stringlist";
 static const char * type_section        = "Section";
 
+static const char * help_string_key     = "HelpString: ";
+static const char * num_digits_key      = "NumDigits";
+
+
 static const char * num_encoding_parameters_key  = "NumEncodingParameters: ";
 static const char * num_decoding_parameters_key  = "NumDecodingParameters: ";
 
@@ -81,11 +88,6 @@ static const char * real_name_key       = "RealName: ";
 static const char * num_options_key     = "NumOptions: ";
 static const char * option_key          = "Options: ";
 
-/* Encoding colormodels */
-
-static const char * num_encoding_colormodels_key = "NumEncodingColormodels: ";
-static const char * encoding_colormodel_key =      "EncodingColormodel: ";
-static const char * decoding_colormodel_key =      "DecodingColormodel: ";
 
 /* Codec order */
 
@@ -128,6 +130,9 @@ static void read_parameter_value(char * pos,
     case LQT_PARAMETER_INT:
       ret->val_int = atoi(pos);
       break;
+    case LQT_PARAMETER_FLOAT:
+      ret->val_float = strtod(pos, (char**)0);
+      break;
     case LQT_PARAMETER_STRING:
     case LQT_PARAMETER_STRINGLIST:
       ret->val_string = __lqt_strdup(pos);
@@ -137,6 +142,36 @@ static void read_parameter_value(char * pos,
     }
   }
 
+static char * convert_help_string(char * str)
+  {
+  char * ret;
+  char * src_pos, *dst_pos;
+  
+  ret = malloc(strlen(str)+1);
+  
+  src_pos = str;
+  dst_pos = ret;
+
+  while(*src_pos != '\0')
+    {
+    if((src_pos[0] == '\\') && (src_pos[1] == 'n'))
+      {
+      *dst_pos = '\n';
+      src_pos += 2;
+      dst_pos++;
+      }
+    else
+      {
+      *dst_pos = *src_pos;
+      src_pos++;
+      dst_pos++;
+      }
+    }
+  *dst_pos = '\0';
+  free(str);
+  return ret;
+  }
+
 static void read_parameter_info(FILE * input,
                                 lqt_parameter_info_t * info,
                                 char * line)
@@ -174,8 +209,22 @@ static void read_parameter_info(FILE * input,
          */
 
         info->val_default.val_int = 0;
-        info->val_min = 0;
-        info->val_max = 0;
+        info->val_min.val_int = 0;
+        info->val_max.val_int = 0;
+        }
+      if(!strcmp(pos, type_float))
+        {
+        info->type = LQT_PARAMETER_FLOAT;
+
+        /*
+         *  We set them here for the case, they are not set after
+         *  (which can happen for min and max)
+         */
+
+        info->val_default.val_float = 0;
+        info->val_min.val_float = 0;
+        info->val_max.val_float = 0;
+        info->num_digits = 1;
         }
 
       /*
@@ -214,12 +263,19 @@ static void read_parameter_info(FILE * input,
     else if(CHECK_KEYWORD(min_value_key))
       {
       pos = line + strlen(min_value_key);
-      info->val_min = atoi(pos);
+      if(info->type == LQT_PARAMETER_INT)
+        info->val_min.val_int = atoi(pos);
+      else if(info->type == LQT_PARAMETER_FLOAT)
+        info->val_min.val_float = strtod(pos, (char**)0);
       }
+
     else if(CHECK_KEYWORD(max_value_key))
       {
       pos = line + strlen(max_value_key);
-      info->val_max = atoi(pos);
+      if(info->type == LQT_PARAMETER_INT)
+        info->val_max.val_int = atoi(pos);
+      else if(info->type == LQT_PARAMETER_FLOAT)
+        info->val_max.val_float = strtod(pos, (char**)0);
       }
     else if(CHECK_KEYWORD(num_options_key))
       {
@@ -234,6 +290,17 @@ static void read_parameter_info(FILE * input,
       info->stringlist_options[options_read] = __lqt_strdup(pos);
       options_read++;
       }
+    else if(CHECK_KEYWORD(help_string_key))
+      {
+      pos = line + strlen(help_string_key);
+      info->help_string = __lqt_strdup(pos);
+      info->help_string = convert_help_string(info->help_string);
+      }
+    else if(CHECK_KEYWORD(num_digits_key))
+      {
+      pos = line + strlen(num_digits_key);
+      info->num_digits = atoi(pos);
+      }
     else if(CHECK_KEYWORD(end_parameter_key))
       break;
     }
@@ -246,7 +313,6 @@ static void read_codec_info(FILE * input, lqt_codec_info_t * codec,
   int i;
   
   int encoding_parameters_read = 0;
-  int encoding_colormodels_read = 0;
   int decoding_parameters_read = 0;
   
   uint32_t tmp_fourcc;
@@ -304,6 +370,13 @@ static void read_codec_info(FILE * input, lqt_codec_info_t * codec,
       else if(!strcmp(pos, direction_both))
         codec->direction = LQT_DIRECTION_BOTH;
       }
+
+    /* Compatibility flags */
+    else if(CHECK_KEYWORD(compatibility_key))
+      {
+      pos = line + strlen(compatibility_key);
+      codec->compatibility_flags = strtoul(pos, (char**)0, 16);
+      }
     
     /* Module filename */
     
@@ -392,7 +465,7 @@ static void read_codec_info(FILE * input, lqt_codec_info_t * codec,
       codec->num_encoding_parameters = atoi(pos);
       if(codec->num_encoding_parameters)
         codec->encoding_parameters =
-          malloc(codec->num_encoding_parameters *
+          calloc(codec->num_encoding_parameters,
                  sizeof(lqt_parameter_info_t));
       else
         codec->encoding_parameters =
@@ -404,7 +477,7 @@ static void read_codec_info(FILE * input, lqt_codec_info_t * codec,
       codec->num_decoding_parameters = atoi(pos);
       if(codec->num_decoding_parameters)
         codec->decoding_parameters =
-          malloc(codec->num_decoding_parameters *
+          calloc(codec->num_decoding_parameters,
                  sizeof(lqt_parameter_info_t));
       else
         codec->decoding_parameters =
@@ -431,32 +504,6 @@ static void read_codec_info(FILE * input, lqt_codec_info_t * codec,
       }
     else if(CHECK_KEYWORD(end_codec_key))
       break;
-
-    /* Number of encoding colormodels */
-
-    else if(CHECK_KEYWORD(num_encoding_colormodels_key))
-      {
-      pos = line + strlen(num_encoding_colormodels_key);
-      codec->num_encoding_colormodels = atoi(pos);
-      if(codec->num_encoding_colormodels)
-        codec->encoding_colormodels =
-          malloc(codec->num_encoding_colormodels *
-                 sizeof(int));
-      else
-        codec->encoding_colormodels = (int*)0;
-      }
-    else if(CHECK_KEYWORD(encoding_colormodel_key))
-      {
-      pos = line + strlen(encoding_colormodel_key);
-      codec->encoding_colormodels[encoding_colormodels_read] =
-        lqt_string_to_colormodel(pos);
-      encoding_colormodels_read++;
-      }
-    else if(CHECK_KEYWORD(decoding_colormodel_key))
-      {
-      pos = line + strlen(decoding_colormodel_key);
-      codec->decoding_colormodel = lqt_string_to_colormodel(pos);
-      }
     }
   
   }
@@ -486,7 +533,6 @@ lqt_codec_info_t * lqt_registry_read(char ** audio_order, char ** video_order)
  
   if(!input)
     {
-    lqt_registry_unlock();
 
 #ifndef NDEBUG
     //    fprintf(stderr, "failed\n");
@@ -567,6 +613,22 @@ lqt_codec_info_t * lqt_registry_read(char ** audio_order, char ** video_order)
   return ret;
   }
 
+static void write_help_string(FILE * output, char * help_string)
+  {
+  int i, imax;
+  fprintf(output, "%s", help_string_key);
+
+  imax = strlen(help_string);
+  for(i = 0; i < imax; i++)
+    {
+    if(help_string[i] == '\n')
+      fprintf(output, "\\n");
+    else
+      fprintf(output, "%c", help_string[i]);
+    }
+  fprintf(output, "\n");
+  }
+
 static void write_parameter_info(FILE * output,
                                  const lqt_parameter_info_t * info,
                                  int encode)
@@ -583,6 +645,9 @@ static void write_parameter_info(FILE * output,
     case LQT_PARAMETER_INT:
       tmp = type_int;
       break;
+    case LQT_PARAMETER_FLOAT:
+      tmp = type_float;
+      break;
     case LQT_PARAMETER_STRING:
       tmp = type_string;
       break;
@@ -605,12 +670,22 @@ static void write_parameter_info(FILE * output,
     case LQT_PARAMETER_INT:
       fprintf(output, "%s%d\n", value_key, info->val_default.val_int);
 
-      if(info->val_min < info->val_max)
+      if(info->val_min.val_int < info->val_max.val_int)
         {
-        fprintf(output, "%s%d\n", min_value_key, info->val_min);
-        fprintf(output, "%s%d\n", max_value_key, info->val_max);
+        fprintf(output, "%s%d\n", min_value_key, info->val_min.val_int);
+        fprintf(output, "%s%d\n", max_value_key, info->val_max.val_int);
         }
 
+      break;
+    case LQT_PARAMETER_FLOAT:
+      fprintf(output, "%s%f\n", value_key, info->val_default.val_float);
+
+      if(info->val_min.val_float < info->val_max.val_float)
+        {
+        fprintf(output, "%s%f\n", min_value_key, info->val_min.val_float);
+        fprintf(output, "%s%f\n", max_value_key, info->val_max.val_float);
+        }
+      fprintf(output, "%s%d\n", num_digits_key, info->num_digits);
       break;
     case LQT_PARAMETER_STRING:
       fprintf(output, "%s%s\n", value_key, info->val_default.val_string);
@@ -630,6 +705,11 @@ static void write_parameter_info(FILE * output,
       break;
     }
 
+  if(info->help_string)
+    {
+    write_help_string(output, info->help_string);
+    }
+  
   fprintf(output, "%s\n", end_parameter_key);
     
   }
@@ -680,6 +760,9 @@ static int write_codec_info(const lqt_codec_info_t * info, FILE * output)
   if(tmp)
     fprintf(output, "%s%s\n", direction_key, tmp);
 
+  fprintf(output, "%s%08x\n", compatibility_key, info->compatibility_flags);
+  
+  
   if(info->num_fourccs)
     {
     fprintf(output, "%s%d\n", num_fourccs_key, info->num_fourccs);
@@ -715,29 +798,7 @@ static int write_codec_info(const lqt_codec_info_t * info, FILE * output)
     {
     write_parameter_info(output, &(info->decoding_parameters[i]), 0);
     }
-
-  if(info->type == LQT_CODEC_VIDEO)
-    {
-    if(info->direction != LQT_DIRECTION_DECODE)
-      {
-      fprintf(output, "%s%d\n", num_encoding_colormodels_key,
-              info->num_encoding_colormodels);
-      
-      for(i = 0; i < info->num_encoding_colormodels; i++)
-        {
-        fprintf(output, "%s%s\n", encoding_colormodel_key,
-                lqt_colormodel_to_string(info->encoding_colormodels[i]));
-        }
-      }
-    if(info->direction != LQT_DIRECTION_ENCODE)
-      {
-      fprintf(output, "%s%s\n", decoding_colormodel_key,
-              lqt_colormodel_to_string(info->decoding_colormodel));
-      
-      }
-    
-    }
-  
+ 
   
   /* Module filename and index */
   fprintf(output, "%s%s\n", module_filename_key, info->module_filename);
