@@ -62,8 +62,8 @@ int lqt_num_video_codecs = 0;
 lqt_codec_info_t * lqt_audio_codecs = (lqt_codec_info_t*)0;
 lqt_codec_info_t * lqt_video_codecs = (lqt_codec_info_t*)0;
 
-static int mutex_initialized = 0;
-pthread_mutex_t codecs_mutex;
+static int registry_init_done = 0;
+pthread_mutex_t codecs_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 /*
  *  Lock and unlock the codec registry
@@ -71,15 +71,6 @@ pthread_mutex_t codecs_mutex;
 
 void lqt_registry_lock()
   {
-  if(!mutex_initialized)
-    {
-    pthread_mutex_init(&codecs_mutex, (pthread_mutexattr_t *)0);
-    mutex_initialized = 1;
-
-    /* We initialize the registry also */
-    
-    lqt_registry_init();
-    }
   pthread_mutex_lock(&codecs_mutex);
   }
 
@@ -88,6 +79,17 @@ void lqt_registry_unlock()
   pthread_mutex_unlock(&codecs_mutex);
   }
 
+int lqt_registry_initialized()
+  {
+  int ret;
+  
+  lqt_registry_lock();
+  ret = registry_init_done;
+  lqt_registry_unlock();
+  
+  return ret;
+  }
+
 /* Free memory of parameter info */
 
 void destroy_parameter_info(lqt_parameter_info_t * p)
@@ -97,6 +99,8 @@ void destroy_parameter_info(lqt_parameter_info_t * p)
     free(p->name);
   if(p->real_name)
     free(p->real_name);
+  if(p->help_string)
+    free(p->help_string);
 
   switch(p->type)
     {
@@ -144,8 +148,6 @@ void destroy_codec_info(lqt_codec_info_t * ptr)
   if(ptr->module_filename)        /* Module filename       */
     free(ptr->module_filename);
 
-  if(ptr->encoding_colormodels)
-    free(ptr->encoding_colormodels);
   
   if(ptr->encoding_parameters)
     {
@@ -172,6 +174,9 @@ static void copy_parameter_value(lqt_parameter_value_t * dst,
     case LQT_PARAMETER_INT:
       dst->val_int = src->val_int;
       break;
+    case LQT_PARAMETER_FLOAT:
+      dst->val_float = src->val_float;
+      break;
     case LQT_PARAMETER_STRING:
     case LQT_PARAMETER_STRINGLIST: /* String with options */
 
@@ -199,14 +204,21 @@ copy_parameter_info(lqt_parameter_info_t * ret,
     ret->name = __lqt_strdup(info->name);
   if(info->real_name)
     ret->real_name = __lqt_strdup(info->real_name);
+  if(info->help_string)
+    ret->help_string = __lqt_strdup(info->help_string);
 
   ret->type = info->type;
 
   switch(ret->type)
     {
     case LQT_PARAMETER_INT:
-      ret->val_min = info->val_min;
-      ret->val_max = info->val_max;
+      ret->val_min.val_int = info->val_min.val_int;
+      ret->val_max.val_int = info->val_max.val_int;
+      break;
+    case LQT_PARAMETER_FLOAT:
+      ret->val_min.val_float = info->val_min.val_float;
+      ret->val_max.val_float = info->val_max.val_float;
+      ret->num_digits = info->num_digits;
       break;
     case LQT_PARAMETER_STRING:
       break;
@@ -274,16 +286,6 @@ copy_codec_info(const lqt_codec_info_t * info)
     
     }
   
-  ret->num_encoding_colormodels = info->num_encoding_colormodels;
-  if(ret->num_encoding_colormodels)
-    {
-    ret->encoding_colormodels =
-      malloc(ret->num_encoding_colormodels * sizeof(int));
-    for(i = 0; i < ret->num_encoding_colormodels; i++)
-      ret->encoding_colormodels[i] = info->encoding_colormodels[i];
-    }
-
-  ret->decoding_colormodel = info->decoding_colormodel;
   
   ret->num_encoding_parameters = info->num_encoding_parameters;
   
@@ -419,8 +421,8 @@ static lqt_codec_info_t * load_codec_info_from_plugin(char * plugin_filename,
 #ifndef NDEBUG  
     //    fprintf(stderr, "\n");
 #endif
-    //    fprintf(stderr, "dlopen failed for %s: %s\n",
-    //            plugin_filename, dlerror());
+    fprintf(stderr, "load_codec_info_from_plugin: dlopen failed for %s: %s\n",
+             plugin_filename, dlerror());
     return ret;
     }
 #ifndef NDEBUG  
@@ -436,7 +438,7 @@ static lqt_codec_info_t * load_codec_info_from_plugin(char * plugin_filename,
 
   if(!get_codec_api_version)
     {
-    fprintf(stderr, "module %s has to API version and is thus terribly old\n",
+    fprintf(stderr, "module %s has no API version and is thus terribly old\n",
             plugin_filename);
     dlclose(module);
     return ret;
@@ -562,7 +564,7 @@ static void register_codecs(lqt_codec_info_t * list,
     }
   }
 
-static int scan_for_plugins(char * plugin_dir, lqt_codec_info_t ** database)
+static int scan_for_plugins(const char * plugin_dir, lqt_codec_info_t ** database)
   {
   char * pos;
   int ret;
@@ -705,9 +707,18 @@ void lqt_registry_init()
   
   lqt_codec_info_t * file_codecs;
   lqt_codec_info_t * tmp_file_codecs;
+  const char* plugin_dir = PLUGIN_DIR;
 
+  /* Check for environment variable for plugin dir */
+  if(getenv("LIBQUICKTIME_PLUGIN_DIR"))
+    {
+    plugin_dir = getenv("LIBQUICKTIME_PLUGIN_DIR");
+    }
+  
   lqt_registry_lock();
   
+  registry_init_done = 1;
+  
   if(lqt_audio_codecs || lqt_video_codecs)
     {
     lqt_registry_unlock();
@@ -718,7 +729,7 @@ void lqt_registry_init()
 
   /* Scan for the plugins, use cached values if possible */
 
-  if(scan_for_plugins(PLUGIN_DIR, &file_codecs))
+  if(scan_for_plugins(plugin_dir, &file_codecs))
     do_write = 1;
 
   /*
@@ -815,6 +826,9 @@ lqt_codec_info_t * lqt_get_audio_codec_info_c(int index)
   const lqt_codec_info_t * info;
   lqt_codec_info_t * ret;
 
+  if (!lqt_registry_initialized()) /* also init registry */
+  	lqt_registry_init();
+	
   lqt_registry_lock();
 
   info = lqt_get_audio_codec_info(index);
@@ -832,6 +846,9 @@ lqt_codec_info_t * lqt_get_video_codec_info_c(int index)
   const lqt_codec_info_t * info;
   lqt_codec_info_t * ret;
 
+  if (!lqt_registry_initialized()) /* also init registry */
+  	lqt_registry_init();
+
   lqt_registry_lock();
 
   info = lqt_get_video_codec_info(index);
@@ -854,14 +871,23 @@ create_parameter_info(lqt_parameter_info_t * ret,
   ret->name = __lqt_strdup(info->name);           /* Parameter name  */
   ret->real_name = __lqt_strdup(info->real_name); /* Parameter name  */
 
+  if(info->help_string)
+    ret->help_string = __lqt_strdup(info->help_string);
+  
   ret->type = info->type;
 
   switch(ret->type)
     {
     case LQT_PARAMETER_INT:
       ret->val_default.val_int = info->val_default.val_int;
-      ret->val_min = info->val_min;
-      ret->val_max = info->val_max;
+      ret->val_min.val_int = info->val_min.val_int;
+      ret->val_max.val_int = info->val_max.val_int;
+      break;
+    case LQT_PARAMETER_FLOAT:
+      ret->val_default.val_float = info->val_default.val_float;
+      ret->val_min.val_float = info->val_min.val_float;
+      ret->val_max.val_float = info->val_max.val_float;
+      ret->num_digits = info->num_digits;
       break;
     case LQT_PARAMETER_STRING:
       ret->val_default.val_string = __lqt_strdup(info->val_default.val_string);
@@ -962,25 +988,6 @@ lqt_create_codec_info(const lqt_codec_info_static_t * template)
   
   /* Copy encoding colormodels */
     
-  ret->num_encoding_colormodels = 0;
-
-  if(template->encoding_colormodels)
-    {
-    while(1)
-      {
-      if(template->encoding_colormodels[ret->num_encoding_colormodels] !=
-         LQT_COLORMODEL_NONE)
-        ret->num_encoding_colormodels++;
-      else
-        break;
-      }
-    ret->encoding_colormodels =
-      malloc(ret->num_encoding_colormodels * sizeof(int));
-    for(i = 0; i < ret->num_encoding_colormodels; i++)
-      ret->encoding_colormodels[i] = template->encoding_colormodels[i];
-    }
-
-  ret->decoding_colormodel = template->decoding_colormodel;
     
   if(template->encoding_parameters)
     {
@@ -1054,9 +1061,19 @@ static void dump_codec_parameter(lqt_parameter_info_t * p)
       fprintf(stderr, "Integer, Default Value: %d ",
               p->val_default.val_int);
 
-      if(p->val_min < p->val_max)
+      if(p->val_min.val_int < p->val_max.val_int)
         fprintf(stderr, "(%d..%d)\n",
-                p->val_min, p->val_max);
+                p->val_min.val_int, p->val_max.val_int);
+      else
+        fprintf(stderr, "(unlimited)\n");
+      break;
+    case LQT_PARAMETER_FLOAT:
+      fprintf(stderr, "Float, Default Value: %f ",
+              p->val_default.val_float);
+
+      if(p->val_min.val_float < p->val_max.val_float)
+        fprintf(stderr, "(%f..%f)\n",
+                p->val_min.val_float, p->val_max.val_float);
       else
         fprintf(stderr, "(unlimited)\n");
       break;
@@ -1076,6 +1093,8 @@ static void dump_codec_parameter(lqt_parameter_info_t * p)
     case LQT_PARAMETER_SECTION:
       fprintf(stderr, "Section");
     }
+  if(p->help_string)
+    fprintf(stderr, "Help string: %s\n", p->help_string);
   }
 
 void lqt_dump_codec_info(const lqt_codec_info_t * info)
@@ -1127,27 +1146,6 @@ void lqt_dump_codec_info(const lqt_codec_info_t * info)
   fprintf(stderr, "Module filename: %s\nIndex inside module: %d\n",
           info->module_filename, info->module_index);
 
-  /* Dump supported colormodels */
-  
-  if(info->type == LQT_CODEC_VIDEO)
-    {
-    if(info->num_encoding_colormodels)
-      {
-      fprintf(stderr, "Supported encoder colormodels: ");
-      for(i = 0; i < info->num_encoding_colormodels - 1; i++)
-        fprintf(stderr, "%s, ",
-                lqt_colormodel_to_string(info->encoding_colormodels[i]));
-      
-      fprintf(stderr, "%s\n",
-              lqt_colormodel_to_string(info->encoding_colormodels[info->num_encoding_colormodels - 1]));
-      }
-
-    fprintf(stderr, "Decoder Colormodel: ");
-    if(info->decoding_colormodel != LQT_COLORMODEL_NONE)
-      fprintf(stderr, "%s\n", lqt_colormodel_to_string(info->decoding_colormodel));
-    else
-      fprintf(stderr, "Depends on stream\n");
-    }
   }
 
 #define MATCH_FOURCC(a, b) \
@@ -1168,7 +1166,10 @@ lqt_codec_info_t ** lqt_find_audio_codec(char * fourcc, int encode)
   lqt_codec_info_t * ptr;
 
   lqt_codec_info_t ** ret = (lqt_codec_info_t **)0;
-  
+
+  if (!lqt_registry_initialized()) /* also init registry */
+    lqt_registry_init();
+    
   lqt_registry_lock();
   
   ptr = lqt_audio_codecs;
@@ -1207,6 +1208,9 @@ lqt_codec_info_t ** lqt_find_audio_codec_by_wav_id(int wav_id, int encode)
   lqt_codec_info_t * ptr;
 
   lqt_codec_info_t ** ret = (lqt_codec_info_t **)0;
+
+  if (!lqt_registry_initialized()) /* also init registry */
+  	lqt_registry_init();
   
   lqt_registry_lock();
   
@@ -1248,6 +1252,9 @@ lqt_codec_info_t ** lqt_find_video_codec(char * fourcc, int encode)
 
   lqt_codec_info_t ** ret = (lqt_codec_info_t **)0;
   
+  if (!lqt_registry_initialized()) /* also init registry */
+     lqt_registry_init();
+  
   lqt_registry_lock();
   
   ptr = lqt_video_codecs;
@@ -1290,6 +1297,10 @@ lqt_codec_info_t ** lqt_query_registry(int audio, int video,
   lqt_codec_info_t ** ret;
   const lqt_codec_info_t * info;
   int num_codecs = 0, num_added = 0, i;
+
+  if (!lqt_registry_initialized()) /* also init registry */
+    lqt_registry_init();
+
   lqt_registry_lock();
 
   if(audio)
@@ -1358,6 +1369,9 @@ lqt_codec_info_t ** lqt_find_audio_codec_by_name(const char * name)
   if(!name)
     return ret;
   
+  if (!lqt_registry_initialized()) /* also init registry */
+     lqt_registry_init();
+
   lqt_registry_lock();
 
   info = lqt_get_audio_codec_info(0);
@@ -1396,6 +1410,9 @@ lqt_codec_info_t ** lqt_find_video_codec_by_name(const char * name)
   if(!name)
     return ret;
   
+  if (!lqt_registry_initialized()) /* also init registry */
+     lqt_registry_init();
+  
   lqt_registry_lock();
 
   info = lqt_get_video_codec_info(0);
@@ -1473,6 +1490,9 @@ void lqt_set_default_parameter(lqt_codec_type type, int encode,
   lqt_codec_info_t * codec_info;
   lqt_parameter_info_t * parameter_info;
   
+  if (!lqt_registry_initialized()) /* also init registry */
+     lqt_registry_init();
+
   lqt_registry_lock();
 
   if(type == LQT_CODEC_AUDIO)
@@ -1541,6 +1561,16 @@ void lqt_set_default_parameter(lqt_codec_type type, int encode,
           (encode ? "Encoding" : "Decoding"),
           parameter_name,
           codec_name, parameter_info[i].val_default.val_int);
+#endif
+      break;
+    case LQT_PARAMETER_FLOAT:
+      parameter_info[i].val_default.val_float = val->val_float;
+#ifndef NDEBUG
+  fprintf(stderr,
+          "%s parameter %s for codec %s (value: %f) stored in registry\n",
+          (encode ? "Encoding" : "Decoding"),
+          parameter_name,
+          codec_name, parameter_info[i].val_default.val_float);
 #endif
       break;
     case LQT_PARAMETER_STRING:
@@ -1590,7 +1620,11 @@ void lqt_restore_default_parameters(lqt_codec_info_t * codec_info,
   
   module = dlopen(codec_info->module_filename, RTLD_NOW);
   if(!module)
+    {
+    fprintf(stderr, "lqt_restore_default_parameters: dlopen failed for %s: %s\n",
+             codec_info->module_filename, dlerror());
     return;
+    }
   
   get_codec_info = (lqt_codec_info_static_t*(*)(int))(dlsym(module, "get_codec_info"));
   if(!get_codec_info)
@@ -1809,15 +1843,13 @@ void lqt_reorder_video_codecs(lqt_codec_info_t ** info)
  * should work everywhere now)
  ***************************************************************/
 
-#if defined(__GNUC__) && defined(__ELF__)
+#if defined(__GNUC__)
 
 static void __lqt_cleanup_codecinfo() __attribute__ ((destructor));
 
 static void __lqt_cleanup_codecinfo()
   {
-#ifndef NDEBUG
   //  fprintf(stderr, "Deleting quicktime codecs\n");
-#endif
   lqt_registry_destroy();
   }
 
