@@ -40,8 +40,7 @@ static int quicktime_encode_video_stub(quicktime_t *file,
 }
 
 static int quicktime_decode_audio_stub(quicktime_t *file, 
-                                       int16_t ** output_i, 
-                                       float ** output_f, 
+                                       void * output,
                                        long samples,
                                        int track)
 {
@@ -50,65 +49,17 @@ static int quicktime_decode_audio_stub(quicktime_t *file,
 }
 
 static int quicktime_encode_audio_stub(quicktime_t *file, 
-				int16_t **input_i, 
-				float **input_f, 
-				int track, 
-				long samples)
+                                       void * input,
+                                       long samples,
+                                       int track)
 {
 	printf("quicktime_encode_audio_stub called\n");
 	return 1;
 }
 
-
-static int quicktime_reads_colormodel_stub(quicktime_t *file, 
-		int colormodel, 
-		int track)
+static int quicktime_flush_codec_stub(quicktime_t *file, int track)
 {
-	fprintf(stderr, "quicktime_reads_colormodel_stub called\n");
-	return (colormodel == BC_RGB888);
-}
-
-static int quicktime_writes_colormodel_stub(quicktime_t *file, 
-		int colormodel, 
-		int track)
-{
-	return (colormodel == BC_RGB888);
-}
-
-static void quicktime_flush_codec_stub(quicktime_t *file, int track)
-{
-}
-
-/* Convert Microsoft audio id to codec */
-void quicktime_id_to_codec(char *result, int id)
-{
-        switch(id)
-        {
-                case 0x55:
-                        memcpy(result, QUICKTIME_MP3, 4);
-                        break;
-                case 0x161:
-                        memcpy(result, QUICKTIME_WMA, 4);
-                        break;
-                default:
-                        printf("quicktime_id_to_codec: unknown audio id: %p\n", (void*)id);
-                        break;
-        }
-}
-
-int quicktime_codec_to_id(char *codec)
-{
-        if(quicktime_match_32(codec, QUICKTIME_MP3))
-                return 0x55;
-        else
-        if(quicktime_match_32(codec, QUICKTIME_WMA))
-                return 0x161;
-        else
-          {
-                printf("quicktime_codec_to_id: unknown codec %c%c%c%c\n", codec[0], codec[1], codec[2], codec[3]);
-                return -1;
-          }
-        
+return 0;
 }
 
 
@@ -120,55 +71,9 @@ int quicktime_codec_defaults(quicktime_codec_t *codec)
 	codec->encode_video = quicktime_encode_video_stub;
 	codec->decode_audio = quicktime_decode_audio_stub;
 	codec->encode_audio = quicktime_encode_audio_stub;
-	codec->reads_colormodel = quicktime_reads_colormodel_stub;
-	codec->writes_colormodel = quicktime_writes_colormodel_stub;
 	codec->flush = quicktime_flush_codec_stub;
 	return 0;
 }
-#if 0 /* Not needed in libquicktime */
-static int get_vcodec_index(char *compressor)
-{
-	int index;
-/* Initialize internal codecs on the first call */
-	if(quicktime_vcodec_size() == 0)
-		quicktime_register_internal_vcodec();
-
-/* Try internal codec */
-	index = quicktime_find_vcodec(compressor);
-
-//printf("get_vcodec_index %d\n", index);
-/* Try external codec */
-	if(index < 0)
-	{
-		index = quicktime_register_external_vcodec(compressor);
-	}
-
-	if(index < 0)
-		return -1;
-	return index;
-}
-
-static int get_acodec_index(char *compressor)
-{
-	int index;
-/* Initialize internal codecs on the first call */
-	if(quicktime_acodec_size() == 0)
-		quicktime_register_internal_acodec();
-
-/* Try internal codec */
-	index = quicktime_find_acodec(compressor);
-
-/* Try external codec */
-	if(index < 0)
-	{
-		index = quicktime_register_external_acodec(compressor);
-	}
-
-	if(index < 0)
-		return -1;
-	return index;
-}
-#endif /* libquicktime */
 
 /*
  *  Original quicktime4linux function changed for dynamic loading
@@ -197,7 +102,7 @@ int quicktime_init_vcodec(quicktime_video_map_t *vtrack, int encode,
     {
     
 #ifndef NDEBUG
-    fprintf(stderr, "Trying to find %s for fourcc \"%s\"...",
+    fprintf(stderr, "Trying to find %s for fourcc \"%4s\"...",
             (encode ? "Encoder" : "Decoder"), compressor);
 #endif
     
@@ -271,6 +176,8 @@ int quicktime_init_vcodec(quicktime_video_map_t *vtrack, int encode,
   if(codec_array)
     lqt_destroy_codec_info(codec_array);
 
+  //  vtrack->stream_cmodel = lqt_get_decoder_colormodel(quicktime_t * file, int track);
+  
   return 0;
   
   }
@@ -382,9 +289,9 @@ int quicktime_init_acodec(quicktime_audio_map_t *atrack, int encode,
 
   /* We set the wav ids from our info structure, so we don't have to do this
      in the plugin sources */
-
+  
   if(codec_info->num_wav_ids)
-    ((quicktime_codec_t*)((quicktime_codec_t*)atrack->codec))->wav_id = codec_info->wav_ids[0];
+    atrack->wav_id = codec_info->wav_ids[0];
   
   if(codec_array)
     lqt_destroy_codec_info(codec_array);
@@ -464,6 +371,7 @@ int quicktime_supported_audio(quicktime_t *file, int track)
 
 void lqt_update_frame_position(quicktime_video_map_t * track)
   {
+  //  fprintf(stderr, "Timestamp: %lld\n", track->timestamp);
   track->timestamp +=
     track->track->mdia.minf.stbl.stts.table[track->stts_index].sample_duration;
 
@@ -478,35 +386,18 @@ void lqt_update_frame_position(quicktime_video_map_t * track)
   track->current_position++;
   }
 
-int quicktime_decode_video(quicktime_t *file,
-                           unsigned char **row_pointers, int track)
-{
-	int result;
-        int track_height;
-	int track_width;
+/* Set the io_rowspan for the case the user didn't. */
 
-        
-	track_height = quicktime_video_height(file, track);
-	track_width =  quicktime_video_width(file, track);
+static void set_default_rowspan(quicktime_t *file, int track)
+  {
+  if(file->vtracks[track].io_row_span)
+    return;
 
-//printf("quicktime_decode_video 1\n");
-// Fake scaling parameters
-	file->do_scaling = 0;
-                
-        file->vtracks[track].color_model = BC_RGB888;
-	file->in_x = 0;
-	file->in_y = 0;
-	file->in_w = track_width;
-	file->in_h = track_height;
-	file->out_w = track_width;
-	file->out_h = track_height;
-
-        //printf("quicktime_decode_video 1\n");
-	result = ((quicktime_codec_t*)file->vtracks[track].codec)->decode_video(file, row_pointers, track);
-        lqt_update_frame_position(&(file->vtracks[track]));
-//printf("quicktime_decode_video 2\n");
-	return result;
-}
+  lqt_get_default_rowspan(file->vtracks[track].io_cmodel,
+                          quicktime_video_width(file, track),
+                          &(file->vtracks[track].io_row_span),
+                          &(file->vtracks[track].io_row_span_uv));
+  }
 
 /*
  *  Same as quicktime_decode_video but doesn't force BC_RGB888
@@ -515,32 +406,77 @@ int quicktime_decode_video(quicktime_t *file,
 int lqt_decode_video(quicktime_t *file,
                      unsigned char **row_pointers, int track)
 {
-        int result;
-        int track_height;
-	int track_width;
-
+	int result;
+        int height;
+	int width;
+#if 0
+        fprintf(stderr, "lqt_decode_video: %s -> %s (io_span: %d, %d, stream_span: %d %d)\n",
+                lqt_colormodel_to_string(file->vtracks[track].stream_cmodel),
+                lqt_colormodel_to_string(file->vtracks[track].io_cmodel),
+                file->vtracks[track].io_row_span, file->vtracks[track].io_row_span_uv,
+                file->vtracks[track].stream_row_span, file->vtracks[track].stream_row_span_uv);
+#endif
+        //        fprintf(stderr, "Rowspan: [%d,%d]/", file->vtracks[track].io_row_span, file->vtracks[track].io_row_span_uv);
+        set_default_rowspan(file, track);
+        //        fprintf(stderr, "[%d,%d]\n", file->vtracks[track].io_row_span, file->vtracks[track].io_row_span_uv);
         
-	track_height = quicktime_video_height(file, track);
-	track_width =  quicktime_video_width(file, track);
-
-//printf("quicktime_decode_video 1\n");
-// Fake scaling parameters
-	file->do_scaling = 0;
-	file->in_x = 0;
-	file->in_y = 0;
-	file->in_w = track_width;
-	file->in_h = track_height;
-	file->out_w = track_width;
-	file->out_h = track_height;
-
-//printf("quicktime_decode_video 1\n");
-	result =
-          ((quicktime_codec_t*)file->vtracks[track].codec)->decode_video(file, row_pointers, track);
+        height = quicktime_video_height(file, track);
+	width =  quicktime_video_width(file, track);
+        
+        if(file->vtracks[track].io_cmodel != file->vtracks[track].stream_cmodel)
+          {
+          if(!file->vtracks[track].temp_frame)
+            {
+            file->vtracks[track].temp_frame =
+              lqt_rows_alloc(width, height, file->vtracks[track].stream_cmodel,
+                             &(file->vtracks[track].stream_row_span),
+                             &(file->vtracks[track].stream_row_span_uv));
+            }
+          result =
+            ((quicktime_codec_t*)file->vtracks[track].codec)->decode_video(file,
+                                                                           file->vtracks[track].temp_frame,
+                                                                           track);
+          cmodel_transfer(row_pointers,                    //    unsigned char **output_rows, /* Leave NULL if non existent */
+                          file->vtracks[track].temp_frame, //    unsigned char **input_rows,
+                          0, //                                  int in_x,        /* Dimensions to capture from input frame */
+                          0, //                                  int in_y, 
+                          width, //                              int in_w, 
+                          height, //                             int in_h,
+                          width, //                              int out_w, 
+                          height, //                             int out_h,
+                          file->vtracks[track].stream_cmodel, // int in_colormodel, 
+                          file->vtracks[track].io_cmodel,     // int out_colormodel,
+                          file->vtracks[track].stream_row_span,   /* For planar use the luma rowspan */
+                          file->vtracks[track].io_row_span,       /* For planar use the luma rowspan */
+                          file->vtracks[track].stream_row_span_uv, /* Chroma rowspan */
+                          file->vtracks[track].io_row_span_uv      /* Chroma rowspan */);
+         
+          }
+        else
+          {
+          file->vtracks[track].stream_row_span    = file->vtracks[track].io_row_span;
+          file->vtracks[track].stream_row_span_uv = file->vtracks[track].io_row_span_uv;
+          
+          result = ((quicktime_codec_t*)file->vtracks[track].codec)->decode_video(file, row_pointers, track);
+          
+          }
+        
+        //printf("quicktime_decode_video 1\n");
         lqt_update_frame_position(&(file->vtracks[track]));
 //printf("quicktime_decode_video 2\n");
 	return result;
 }
 
+/* The original function, which forces BG_RGB888 */
+int quicktime_decode_video(quicktime_t *file,
+                           unsigned char **row_pointers, int track)
+  {
+  
+  file->vtracks[track].io_cmodel = BC_RGB888;
+  return lqt_decode_video(file, row_pointers, track);
+  }
+
+
 long quicktime_decode_scaled(quicktime_t *file, 
 	int in_x,                    /* Location of input frame to take picture */
 	int in_y,
@@ -554,58 +490,289 @@ long quicktime_decode_scaled(quicktime_t *file,
 {
 	int result;
 
-	file->do_scaling = 1;
-	file->vtracks[track].color_model = color_model;
-	file->in_x = in_x;
-	file->in_y = in_y;
-	file->in_w = in_w;
-	file->in_h = in_h;
-	file->out_w = out_w;
-	file->out_h = out_h;
+        int height;
+	int width;
+        
+        set_default_rowspan(file, track);
+	height = quicktime_video_height(file, track);
+	width =  quicktime_video_width(file, track);
 
-	result = ((quicktime_codec_t*)file->vtracks[track].codec)->decode_video(file, row_pointers, track);
+        
+	file->vtracks[track].io_cmodel = color_model;
+
+        if(!file->vtracks[track].temp_frame)
+          {
+          file->vtracks[track].temp_frame =
+            lqt_rows_alloc(width, height, file->vtracks[track].stream_cmodel,
+                           &(file->vtracks[track].stream_row_span),
+                           &(file->vtracks[track].stream_row_span_uv));
+          }
+        result =
+          ((quicktime_codec_t*)file->vtracks[track].codec)->decode_video(file,
+                                                                         file->vtracks[track].temp_frame,
+                                                                         track);
+        cmodel_transfer(row_pointers,                    //    unsigned char **output_rows, /* Leave NULL if non existent */
+                        file->vtracks[track].temp_frame, //    unsigned char **input_rows,
+                        in_x, //                               int in_x,        /* Dimensions to capture from input frame */
+                        in_y, //                               int in_y, 
+                        in_w, //                               int in_w, 
+                        in_h, //                               int in_h,
+                        out_w, //                              int out_w, 
+                        out_h, //                              int out_h,
+                        file->vtracks[track].stream_cmodel, // int in_colormodel, 
+                        file->vtracks[track].io_cmodel,     // int out_colormodel,
+                        file->vtracks[track].stream_row_span,   /* For planar use the luma rowspan */
+                        file->vtracks[track].io_row_span,       /* For planar use the luma rowspan */
+                        file->vtracks[track].stream_row_span_uv, /* Chroma rowspan */
+                        file->vtracks[track].io_row_span_uv      /* Chroma rowspan */);
+        
         lqt_update_frame_position(&(file->vtracks[track]));
 	return result;
 }
 
+int lqt_set_video_pass(quicktime_t *file,
+                       int pass, int total_passes, 
+                       const char * stats_file, int track)
+  {
+  if(((quicktime_codec_t*)file->vtracks[track].codec)->set_pass)
+    return ((quicktime_codec_t*)file->vtracks[track].codec)->set_pass(file,
+                                                                      track,
+                                                                      pass,
+                                                                      total_passes,
+                                                                      stats_file);
+  else
+    return 0;
+  }
 
-int quicktime_encode_video(quicktime_t *file, 
-	unsigned char **row_pointers, 
-	int track)
-{
-	int result;
-//printf("quicktime_encode_video 1 %p\n", ((quicktime_codec_t*)file->vtracks[track].codec)->encode_video);
-	result = ((quicktime_codec_t*)file->vtracks[track].codec)->encode_video(file, row_pointers, track);
-//printf("quicktime_encode_video 2\n");
-        quicktime_update_stts(&file->vtracks[track].track->mdia.minf.stbl.stts,
-                              file->vtracks[track].current_position, 0);
-        file->vtracks[track].current_position++;
-	return result;
-}
+static void start_encoding(quicktime_t *file)
+  {
+  if(file->encoding_started)
+    return;
+
+  file->encoding_started = 1;
+
+  if(file->file_type & (LQT_FILE_AVI|LQT_FILE_AVI_ODML))
+    {
+    quicktime_set_position(file, 0);
+    // Write RIFF chunk
+    quicktime_init_riff(file);
+    }
+  }
+
+
+static int do_encode_video(quicktime_t *file, 
+                           unsigned char **row_pointers, 
+                           int track)
+  {
+  int result;
+
+  int height;
+  int width;
+
+  start_encoding(file);
+  
+  set_default_rowspan(file, track);
+  height = quicktime_video_height(file, track);
+  width =  quicktime_video_width(file, track);
+
+#if 0
+  fprintf(stderr, "lqt_encode_video: %s -> %s (io_span: %d, %d, stream_span: %d %d)\n",
+          lqt_colormodel_to_string(file->vtracks[track].io_cmodel),
+          lqt_colormodel_to_string(file->vtracks[track].stream_cmodel),
+          file->vtracks[track].io_row_span, file->vtracks[track].io_row_span_uv,
+          file->vtracks[track].stream_row_span, file->vtracks[track].stream_row_span_uv);
+#endif   
+  
+  if(file->vtracks[track].io_cmodel != file->vtracks[track].stream_cmodel)
+    {
+    if(!file->vtracks[track].temp_frame)
+      {
+      file->vtracks[track].temp_frame =
+        lqt_rows_alloc(width, height, file->vtracks[track].stream_cmodel,
+                       &(file->vtracks[track].stream_row_span),
+                       &(file->vtracks[track].stream_row_span_uv));
+      }
+    cmodel_transfer(file->vtracks[track].temp_frame, //    unsigned char **output_rows, /* Leave NULL if non existent */
+                    row_pointers,                    //    unsigned char **input_rows,
+                    0, //                                  int in_x,        /* Dimensions to capture from input frame */
+                    0, //                                  int in_y, 
+                    width, //                              int in_w, 
+                    height, //                             int in_h,
+                    width, //                              int out_w, 
+                    height, //                             int out_h,
+                    file->vtracks[track].io_cmodel, // int in_colormodel, 
+                    file->vtracks[track].stream_cmodel,     // int out_colormodel,
+                    file->vtracks[track].io_row_span,   /* For planar use the luma rowspan */
+                    file->vtracks[track].stream_row_span,       /* For planar use the luma rowspan */
+                    file->vtracks[track].io_row_span_uv, /* Chroma rowspan */
+                    file->vtracks[track].stream_row_span_uv      /* Chroma rowspan */);
+    result = ((quicktime_codec_t*)file->vtracks[track].codec)->encode_video(file, file->vtracks[track].temp_frame, track);
+    }
+  else
+    {
+    file->vtracks[track].stream_row_span    = file->vtracks[track].io_row_span;
+    file->vtracks[track].stream_row_span_uv = file->vtracks[track].io_row_span_uv;
+    result = ((quicktime_codec_t*)file->vtracks[track].codec)->encode_video(file, row_pointers, track);
+    }
+  return result;
+  }
 
 int lqt_encode_video(quicktime_t *file, 
                      unsigned char **row_pointers, 
                      int track, int64_t time)
-{
-	int result;
-//printf("quicktime_encode_video 1 %p\n", ((quicktime_codec_t*)file->vtracks[track].codec)->encode_video);
-	result = ((quicktime_codec_t*)file->vtracks[track].codec)->encode_video(file, row_pointers, track);
-//printf("quicktime_encode_video 2\n");
+  {
+  int64_t stts_index, stts_count, dts;
+  int result;
+  /* Must set valid timestamp for encoders */
+  int64_t last_time = file->vtracks[track].timestamp;
+  file->vtracks[track].timestamp = time;
 
-        if(file->vtracks[track].current_position)
-          quicktime_update_stts(&file->vtracks[track].track->mdia.minf.stbl.stts,
-                                file->vtracks[track].current_position - 1,
-                                time - file->vtracks[track].timestamp);
-        else
-          quicktime_update_stts(&file->vtracks[track].track->mdia.minf.stbl.stts,
-                                file->vtracks[track].current_position, 0);
+  /* B-Frame handling: B-Frame enabled codecs can have a delay */
+  /* Here, we use vtrack->current_chunk as index of the actually written
+     frame (implicitely assuming, that ther is never more than one frame per chunk),
+     vtrack->current_position is number of frames, which came from the user */
+    
+  result = do_encode_video(file, row_pointers, track);
 
-        file->vtracks[track].timestamp = time;
-        file->vtracks[track].current_position++;
-	return result;
-}
+  if(file->vtracks[track].has_b_frames)
+    {
+    file->vtracks[track].track->mdia.minf.stbl.has_ctts = 1;
 
+    if(file->vtracks[track].current_position)
+      quicktime_update_stts(&file->vtracks[track].track->mdia.minf.stbl.stts,
+                            file->vtracks[track].current_position - 1,
+                            time - last_time);
+    
+    if(file->vtracks[track].current_chunk > 1) /* Update ctts */
+      {
+      dts = quicktime_sample_to_time(&file->vtracks[track].track->mdia.minf.stbl.stts,
+                                     file->vtracks->current_chunk-2,
+                                     &stts_index, &stts_count);
 
+      quicktime_update_ctts(&file->vtracks[track].track->mdia.minf.stbl.ctts,
+                            file->vtracks[track].current_chunk - 2,
+                            file->vtracks[track].coded_timestamp - dts);
+#if 0      
+      fprintf(stderr, "pts: %lld, dts: %lld, diff: %lld, position: %ld, chunk: %ld\n",
+              file->vtracks[track].coded_timestamp, dts,
+              file->vtracks[track].coded_timestamp - dts,
+              file->vtracks[track].current_position,
+              file->vtracks[track].current_chunk-2);
+#endif      
+      }
+    }
+  else
+    {
+    if(file->vtracks[track].current_position)
+      quicktime_update_stts(&file->vtracks[track].track->mdia.minf.stbl.stts,
+                            file->vtracks[track].current_position - 1,
+                            time - last_time);
+    } 
+  
+  file->vtracks[track].current_position++;
+  return result;
+  }
+
+int quicktime_encode_video(quicktime_t *file, 
+	unsigned char **row_pointers, 
+	int track)
+  {
+  return lqt_encode_video(file, 
+                          row_pointers, 
+                          track, file->vtracks[track].timestamp +
+                          file->vtracks[track].track->mdia.minf.stbl.stts.default_duration);
+  }
+
+static int bytes_per_sample(lqt_sample_format_t format)
+  {
+  switch(format)
+    {
+    case LQT_SAMPLE_INT8:
+    case LQT_SAMPLE_UINT8:
+      return 1;
+      break;
+    case LQT_SAMPLE_INT16:
+      return 2;
+      break;
+    case LQT_SAMPLE_INT32:
+      return 4;
+      break;
+    case LQT_SAMPLE_FLOAT: /* Float is ALWAYS machine native */
+      return sizeof(float);
+      break;
+    case LQT_SAMPLE_UNDEFINED:
+      return 0;
+    }
+  return 0;
+  }
+
+/* Decode raw samples */
+
+int lqt_decode_audio_raw(quicktime_t *file,  void * output, long samples, int track)
+  {
+  int result;
+  quicktime_audio_map_t * atrack;
+  atrack = &(file->atracks[track]);
+  result = ((quicktime_codec_t*)(atrack->codec))->decode_audio(file, output, 
+                                                               samples,
+                                                               track);
+
+  file->atracks[track].current_position += samples;
+  return result;
+  }
+
+int lqt_encode_audio_raw(quicktime_t *file,  void * input, long samples, int track)
+  {
+  quicktime_audio_map_t * atrack;
+  if(!samples)
+    return 0;
+  atrack = &(file->atracks[track]);
+  start_encoding(file);
+  file->atracks[track].current_position += samples;
+  return ((quicktime_codec_t*)(atrack->codec))->encode_audio(file, input, 
+                                                             samples,
+                                                             track);
+  }
+
+/* Compatibility function for old decoding API */
+
+static int decode_audio_old(quicktime_t *file, 
+                            int16_t ** output_i, 
+                            float ** output_f, 
+                            long samples, 
+                            int track)
+  {
+  int result;
+  quicktime_audio_map_t * atrack;
+  atrack = &(file->atracks[track]);
+
+  if(atrack->sample_format == LQT_SAMPLE_UNDEFINED)
+    ((quicktime_codec_t*)(atrack->codec))->decode_audio(file, (void*)0, 
+                                                        0, track);
+  
+  
+  /* (Re)allocate sample buffer */
+
+  if(atrack->sample_buffer_alloc < samples)
+    {
+    atrack->sample_buffer_alloc = samples + 1024;
+    atrack->sample_buffer = realloc(atrack->sample_buffer,
+                                    atrack->sample_buffer_alloc *
+                                    atrack->channels *
+                                    bytes_per_sample(atrack->sample_format));
+    }
+  
+  /* Decode */
+
+  result = ((quicktime_codec_t*)(atrack->codec))->decode_audio(file, atrack->sample_buffer, 
+                                                               samples,
+                                                               track);
+  
+  /* Convert */
+  lqt_convert_audio_decode(atrack->sample_buffer, output_i, output_f, atrack->channels, samples,
+                           atrack->sample_format);
+  return result;
+  }
 
 int quicktime_decode_audio(quicktime_t *file, 
                            int16_t *output_i, 
@@ -641,11 +808,7 @@ int quicktime_decode_audio(quicktime_t *file,
         else
           channels_f = (float**)0;
         
-	result = ((quicktime_codec_t*)file->atracks[quicktime_track].codec)->decode_audio(file, 
-                                                                                          channels_i, 
-                                                                                          channels_f, 
-                                                                                          samples,
-                                                                                          quicktime_track);
+	result = decode_audio_old(file, channels_i, channels_f, samples, quicktime_track);
 	file->atracks[quicktime_track].current_position += result;
 
         if(channels_i)
@@ -703,12 +866,7 @@ int lqt_decode_audio(quicktime_t *file,
           if(file->atracks[i].eof)
             return 1;
                     
-          result = ((quicktime_codec_t*)file->atracks[i].codec)->decode_audio(
-                                                                              file, 
-                                                                              output_i, 
-                                                                              output_f, 
-                                                                              samples, 
-                                                                              i);
+          result = decode_audio_old(file, output_i, output_f, samples, i);
           if(output_f)
             output_f += track_channels;
           if(output_i)
@@ -734,18 +892,56 @@ int lqt_decode_audio_track(quicktime_t *file,
     return 1;
           
   
-  result = !(((quicktime_codec_t*)file->atracks[track].codec)->decode_audio(
-                                                                           file, 
-                                                                           poutput_i, 
-                                                                           poutput_f, 
-                                                                           samples, 
-                                                                           track));
+  result = !decode_audio_old(file, poutput_i, poutput_f, samples, track);
   
   file->atracks[track].current_position += samples;
   
   return result;
   }
 
+static int encode_audio_old(quicktime_t *file, 
+                     int16_t **input_i, 
+                     float **input_f, 
+                     long samples,
+                     int track)
+  {
+  quicktime_audio_map_t * atrack;
+  atrack = &(file->atracks[track]);
+  start_encoding(file);
+  
+  if(!samples)
+    return 0;
+  
+  if(atrack->sample_format == LQT_SAMPLE_UNDEFINED)
+    ((quicktime_codec_t*)(atrack->codec))->encode_audio(file, (void*)0, 
+                                                        0, track);
+
+  
+  /* (Re)allocate sample buffer */
+
+  if(atrack->sample_buffer_alloc < samples)
+    {
+    atrack->sample_buffer_alloc = samples + 1024;
+    atrack->sample_buffer = realloc(atrack->sample_buffer,
+                                    atrack->sample_buffer_alloc *
+                                    atrack->channels *
+                                    bytes_per_sample(atrack->sample_format));
+    }
+
+  /* Convert */
+
+  lqt_convert_audio_encode(input_i, input_f, atrack->sample_buffer, atrack->channels, samples,
+                           atrack->sample_format);
+
+  /* Encode */
+
+  file->atracks[track].current_position += samples;
+  
+  return ((quicktime_codec_t*)(atrack->codec))->encode_audio(file, atrack->sample_buffer,
+                                                             samples, track);
+  
+  }
+
 int lqt_encode_audio_track(quicktime_t *file, 
                            int16_t **input_i, 
                            float **input_f, 
@@ -754,11 +950,8 @@ int lqt_encode_audio_track(quicktime_t *file,
   {
 	int result = 1;
 
-	result = ((quicktime_codec_t*)file->atracks[track].codec)->encode_audio(file, input_i, 
-                                                                            input_f, track, samples);
-	file->atracks[track].current_position += samples;
-
-	return result;
+	result = encode_audio_old(file, input_i, input_f, samples, track);
+        return result;
   
   }
 
@@ -771,16 +964,14 @@ int quicktime_reads_cmodel(quicktime_t *file,
 	int colormodel, 
 	int track)
 {
-	int result = ((quicktime_codec_t*)file->vtracks[track].codec)->reads_colormodel(file, colormodel, track);
-/*         fprintf(stderr, "quicktime_reads_cmodel: %s\n", lqt_colormodel_to_string(colormodel)); */
-	return result;
+        return lqt_colormodel_has_conversion(file->vtracks[track].stream_cmodel, colormodel);
 }
 
 int quicktime_writes_cmodel(quicktime_t *file, 
 	int colormodel, 
 	int track)
 {
-	return ((quicktime_codec_t*)file->vtracks[track].codec)->writes_colormodel(file, colormodel, track);
+        return lqt_colormodel_has_conversion(colormodel, file->vtracks[track].stream_cmodel);
 }
 
 /* Compressors that can only encode a window at a time */
@@ -793,26 +984,26 @@ int quicktime_flush_acodec(quicktime_t *file, int track)
 };
 
 void quicktime_flush_vcodec(quicktime_t *file, int track)
-{
-	((quicktime_codec_t*)file->vtracks[track].codec)->flush(file, track);
-}
+  {
+  int64_t dts, stts_index, stts_count;
+  quicktime_stts_t * stts;
+  /* First, we set the final stts */
+  stts = &file->vtracks[track].track->mdia.minf.stbl.stts;
 
-#if 0
-int64_t quicktime_samples_to_bytes(quicktime_trak_t *track, long samples)
-{
-	char *compressor = track->mdia.minf.stbl.stsd.table[0].format;
-	int channels = track->mdia.minf.stbl.stsd.table[0].channels;
+  stts->table[stts->total_entries-1].sample_count++;
+  
+  while(((quicktime_codec_t*)file->vtracks[track].codec)->flush(file, track))
+    {
+    dts = quicktime_sample_to_time(&file->vtracks[track].track->mdia.minf.stbl.stts,
+                                   file->vtracks->current_chunk-2,
+                                   &stts_index, &stts_count);
+    
+    quicktime_update_ctts(&file->vtracks[track].track->mdia.minf.stbl.ctts,
+                          file->vtracks[track].current_chunk - 2,
+                          file->vtracks[track].coded_timestamp - dts);
+    }
+  }
 
-	if(quicktime_match_32(compressor, QUICKTIME_IMA4)) 
-		return samples * channels;
-
-	if(quicktime_match_32(compressor, QUICKTIME_ULAW)) 
-		return samples * channels;
-
-/* Default use the sample size specification for TWOS and RAW */
-	return samples * channels * track->mdia.minf.stbl.stsd.table[0].sample_size / 8;
-}
-#endif
 int quicktime_codecs_flush(quicktime_t *file)
 {
 	int result = 0;
@@ -896,3 +1087,4 @@ int lqt_copy_audio(int16_t ** dst_i, float ** dst_f,
     }
   return samples_to_copy;
   }
+
