@@ -6,6 +6,8 @@ void quicktime_minf_init(quicktime_minf_t *minf)
 {
 	minf->is_video = minf->is_audio = 0;
 	minf->is_panorama = 0;
+	minf->is_qtvr = 0;
+	minf->is_object = 0;
 	quicktime_vmhd_init(&(minf->vmhd));
 	quicktime_smhd_init(&(minf->smhd));
 	quicktime_gmhd_init(&(minf->gmhd));
@@ -14,6 +16,21 @@ void quicktime_minf_init(quicktime_minf_t *minf)
 	quicktime_stbl_init(&(minf->stbl));
 }
 
+
+void quicktime_minf_init_qtvr(quicktime_t *file,
+								quicktime_minf_t *minf,
+								int track_type,
+								int width,
+								int height,
+								int frame_duration)
+{
+	minf->is_qtvr = track_type;
+	quicktime_stbl_init_qtvr(file, &(minf->stbl), track_type, width, height, frame_duration);
+	quicktime_hdlr_init_data(&(minf->hdlr));
+	quicktime_dinf_init_all(&(minf->dinf));
+}
+
+
 void quicktime_minf_init_panorama(quicktime_t *file,
 								quicktime_minf_t *minf,
 								int width,
@@ -74,8 +91,9 @@ void quicktime_minf_delete(quicktime_minf_t *minf)
 void quicktime_minf_dump(quicktime_minf_t *minf)
 {
 	printf("   media info\n");
-	printf("    is_audio %d\n", minf->is_audio);
-	printf("    is_video %d\n", minf->is_video);
+	printf("    is_audio     %d\n", minf->is_audio);
+	printf("    is_audio_vbr %d\n", minf->is_audio_vbr);
+	printf("    is_video     %d\n", minf->is_video);
 	if(minf->is_audio) quicktime_smhd_dump(&(minf->smhd));
 	if(minf->is_video) quicktime_vmhd_dump(&(minf->vmhd));
 	if(minf->has_baseheader) quicktime_gmhd_dump(&(minf->gmhd));
@@ -84,7 +102,8 @@ void quicktime_minf_dump(quicktime_minf_t *minf)
 	quicktime_stbl_dump(minf, &(minf->stbl));
 }
 
-int quicktime_read_minf(quicktime_t *file, quicktime_minf_t *minf, quicktime_atom_t *parent_atom)
+int quicktime_read_minf(quicktime_t *file, quicktime_trak_t *trak,
+                        quicktime_minf_t *minf, quicktime_atom_t *parent_atom)
 {
 	quicktime_atom_t leaf_atom;
 	do
@@ -105,9 +124,7 @@ int quicktime_read_minf(quicktime_t *file, quicktime_minf_t *minf, quicktime_ato
 		if(quicktime_atom_is(&leaf_atom, "hdlr"))
 			{ 
 
-				quicktime_read_hdlr(file, &(minf->hdlr));
-				/* Main Actor doesn't write component name */
-				quicktime_atom_skip(file, &leaf_atom);
+				quicktime_read_hdlr(file, &(minf->hdlr), &leaf_atom);
 			}
 		else
 		if(quicktime_atom_is(&leaf_atom, "dinf"))
@@ -119,6 +136,12 @@ int quicktime_read_minf(quicktime_t *file, quicktime_minf_t *minf, quicktime_ato
 			quicktime_atom_skip(file, &leaf_atom);
 	}while(quicktime_position(file) < parent_atom->end);
 
+        /* Finalize stsd */
+        quicktime_finalize_stsd(file, trak, &(minf->stbl.stsd));
+          
+        
+        if(minf->is_audio && (minf->stbl.stsd.table[0].compression_id == -2))
+          minf->is_audio_vbr = 1;
 	return 0;
 }
 
@@ -130,6 +153,8 @@ void quicktime_write_minf(quicktime_t *file, quicktime_minf_t *minf)
 	if(minf->is_video) quicktime_write_vmhd(file, &(minf->vmhd));
 	if(minf->is_audio) quicktime_write_smhd(file, &(minf->smhd));
 	if(minf->is_panorama) quicktime_write_gmhd(file, &(minf->gmhd));
+	if(minf->is_qtvr != 0) quicktime_write_gmhd(file, &(minf->gmhd));
+	
 	quicktime_write_hdlr(file, &(minf->hdlr));
 	quicktime_write_dinf(file, &(minf->dinf));
 	quicktime_write_stbl(file, minf, &(minf->stbl));
