@@ -12,6 +12,7 @@ int quicktime_moov_init(quicktime_moov_t *moov)
 	quicktime_mvhd_init(&(moov->mvhd));
 	quicktime_udta_init(&(moov->udta));
 	quicktime_ctab_init(&(moov->ctab));
+	quicktime_iods_init(&(moov->iods));
 	return 0;
 }
 
@@ -21,6 +22,7 @@ int quicktime_moov_delete(quicktime_moov_t *moov)
 	quicktime_mvhd_delete(&(moov->mvhd));
 	quicktime_udta_delete(&(moov->udta));
 	quicktime_ctab_delete(&(moov->ctab));
+	quicktime_iods_delete(&(moov->iods));
 	return 0;
 }
 
@@ -30,9 +32,12 @@ void quicktime_moov_dump(quicktime_moov_t *moov)
 	printf("movie (moov)\n");
 	quicktime_mvhd_dump(&(moov->mvhd));
 	quicktime_udta_dump(&(moov->udta));
-	for(i = 0; i < moov->total_tracks; i++)
+        if(moov->has_iods)
+          quicktime_iods_dump(&(moov->iods));
+        for(i = 0; i < moov->total_tracks; i++)
 		quicktime_trak_dump(moov->trak[i]);
 	quicktime_ctab_dump(&(moov->ctab));
+        
 }
 
 static void* zalloc(void *opaque, unsigned int items, unsigned int size)
@@ -49,6 +54,12 @@ static int read_cmov(quicktime_t *file,
         quicktime_atom_t *parent_atom,
         quicktime_atom_t *moov_atom)
 {
+        int64_t old_preload_size;
+        uint8_t *old_preload_buffer;
+        int64_t old_preload_start;
+        int64_t old_preload_end;
+        int64_t old_preload_ptr;
+
         quicktime_atom_t leaf_atom;
         unsigned char *data_in, *data_out;
         z_stream zlib;
@@ -123,11 +134,11 @@ static int read_cmov(quicktime_t *file,
                         file->moov_size = moov_atom->size;
                         moov_atom->end = moov_atom->start + uncompressed_size;
                         moov_atom->size = uncompressed_size;
-                        file->old_preload_size = file->preload_size;
-                        file->old_preload_buffer = file->preload_buffer;
-                        file->old_preload_start = file->preload_start;
-                        file->old_preload_end = file->preload_end;
-                        file->old_preload_ptr = file->preload_ptr;
+                        old_preload_size = file->preload_size;
+                        old_preload_buffer = file->preload_buffer;
+                        old_preload_start = file->preload_start;
+                        old_preload_end = file->preload_end;
+                        old_preload_ptr = file->preload_ptr;
                         file->preload_size = uncompressed_size;
                         file->preload_buffer = data_out;
                         file->preload_start = moov_atom->start;
@@ -142,11 +153,11 @@ static int read_cmov(quicktime_t *file,
 /* Exit the compressed state */
                         moov_atom->size = file->moov_size;
                         moov_atom->end = file->moov_end;
-                        file->preload_size = file->old_preload_size;
-                        file->preload_buffer = file->old_preload_buffer;
-                        file->preload_start = file->old_preload_start;
-                        file->preload_end = file->old_preload_end;
-                        file->preload_ptr = file->old_preload_ptr;
+                        file->preload_size = old_preload_size;
+                        file->preload_buffer = old_preload_buffer;
+                        file->preload_start = old_preload_start;
+                        file->preload_end = old_preload_end;
+                        file->preload_ptr = old_preload_ptr;
                         quicktime_set_position(file, moov_atom->end);
                 }
                 else
@@ -213,6 +224,13 @@ int quicktime_read_moov(quicktime_t *file, quicktime_moov_t *moov, quicktime_ato
 			quicktime_read_ctab(file, &(moov->ctab));
 		}
 		else
+		if(quicktime_atom_is(&leaf_atom, "iods"))
+		{
+			quicktime_read_iods(file, &(moov->iods));
+			quicktime_atom_skip(file, &leaf_atom);
+                        moov->has_iods = 1;
+		}
+		else
 		quicktime_atom_skip(file, &leaf_atom);
 //printf("quicktime_read_moov %llx %llx\n", quicktime_position(file), parent_atom->end);
 	}while(quicktime_position(file) < parent_atom->end);
@@ -220,13 +238,36 @@ int quicktime_read_moov(quicktime_t *file, quicktime_moov_t *moov, quicktime_ato
 	return 0;
 }
 
+void quicktime_finalize_moov(quicktime_t *file, quicktime_moov_t *moov)
+  {
+  int i;
+  long longest_duration = 0;
+  long duration, timescale;
+  /* get the duration from the longest track in the mvhd's timescale */
+  for(i = 0; i < moov->total_tracks; i++)
+    {
+    quicktime_trak_fix_counts(file, moov->trak[i]);
+    quicktime_trak_duration(moov->trak[i], &duration, &timescale);
+    
+    duration = (long)((float)duration / timescale * moov->mvhd.time_scale);
+    
+    if(duration > longest_duration)
+      {
+      longest_duration = duration;
+      }
+
+    /* Add tracks to iods */
+    quicktime_iods_add_track(&(moov->iods), moov->trak[i]);
+        
+    }
+  moov->mvhd.duration = longest_duration;
+  moov->mvhd.selection_duration = longest_duration;
+  }
+
 void quicktime_write_moov(quicktime_t *file, quicktime_moov_t *moov)
 {
 	quicktime_atom_t atom;
-	int i;
-	long longest_duration = 0;
-	long duration, timescale;
-	int result;
+	int result, i;
 
 
 // Try moov header immediately
@@ -240,24 +281,9 @@ void quicktime_write_moov(quicktime_t *file, quicktime_moov_t *moov)
 		file->mdat.atom.end = quicktime_position(file);
 		quicktime_atom_write_header(file, &atom, "moov");
 	}
-
-/* get the duration from the longest track in the mvhd's timescale */
-	for(i = 0; i < moov->total_tracks; i++)
-	{
-		quicktime_trak_fix_counts(file, moov->trak[i]);
-		quicktime_trak_duration(moov->trak[i], &duration, &timescale);
-
-		duration = (long)((float)duration / timescale * moov->mvhd.time_scale);
-
-		if(duration > longest_duration)
-		{
-			longest_duration = duration;
-		}
-	}
-	moov->mvhd.duration = longest_duration;
-	moov->mvhd.selection_duration = longest_duration;
-
-	quicktime_write_mvhd(file, &(moov->mvhd));
+        quicktime_write_mvhd(file, &(moov->mvhd));
+        if(moov->has_iods)
+          quicktime_write_iods(file, moov);
 	quicktime_write_udta(file, &(moov->udta));
 	for(i = 0; i < moov->total_tracks; i++)
 	{
@@ -266,8 +292,6 @@ void quicktime_write_moov(quicktime_t *file, quicktime_moov_t *moov)
 	/*quicktime_write_ctab(file, &(moov->ctab)); */
 
 	quicktime_atom_write_footer(file, &atom);
-// Rewind to end of mdat
-	quicktime_set_position(file, file->mdat.atom.end);
 }
 
 void quicktime_update_durations(quicktime_moov_t *moov)
