@@ -14,6 +14,27 @@ void quicktime_stbl_init(quicktime_stbl_t *stbl)
 }
 
 
+void quicktime_stbl_init_qtvr(quicktime_t *file,
+								quicktime_stbl_t *stbl,
+								int track_type,
+								int width,
+								int height,
+								int frame_duration)
+{
+//printf("quicktime_stbl_init_qtvr 1\n");
+	quicktime_stsd_init_qtvr(file, &(stbl->stsd), track_type, width, height);
+//printf("quicktime_stbl_init_video 1 %d %f\n", time_scale, frame_rate);
+	quicktime_stts_init_video(file, &(stbl->stts), frame_duration);
+//printf("quicktime_stbl_init_video 1\n");
+	quicktime_stsc_init_video(file, &(stbl->stsc));
+//printf("quicktime_stbl_init_video 1\n");
+	quicktime_stsz_init_video(file, &(stbl->stsz));
+//printf("quicktime_stbl_init_video 1\n");
+	quicktime_stco_init_common(file, &(stbl->stco));
+//printf("quicktime_stbl_init_video 2\n");
+}
+
+
 void quicktime_stbl_init_panorama(quicktime_t *file,
 								quicktime_stbl_t *stbl,
 								int width,
@@ -23,7 +44,7 @@ void quicktime_stbl_init_panorama(quicktime_t *file,
 //printf("quicktime_stbl_init_panorama 1\n");
 	quicktime_stsd_init_panorama(file, &(stbl->stsd), width, height);
 //printf("quicktime_stbl_init_panorama 1 %d %f\n", time_scale, frame_rate);
-	quicktime_stts_init_panorama(file, &(stbl->stts), frame_duration);
+	quicktime_stts_init_video(file, &(stbl->stts), frame_duration);
 //printf("quicktime_stbl_init_video 1\n");
 	quicktime_stsc_init_video(file, &(stbl->stsc));
 //printf("quicktime_stbl_init_video 1\n");
@@ -93,48 +114,70 @@ void quicktime_stbl_dump(void *minf_ptr, quicktime_stbl_t *stbl)
 	quicktime_stsc_dump(&(stbl->stsc));
 	quicktime_stsz_dump(&(stbl->stsz));
 	quicktime_stco_dump(&(stbl->stco));
+        if(stbl->has_ctts)
+          quicktime_ctts_dump(&(stbl->ctts));
+        
 }
 
-int quicktime_read_stbl(quicktime_t *file, quicktime_minf_t *minf, quicktime_stbl_t *stbl, quicktime_atom_t *parent_atom)
-{
-	quicktime_atom_t leaf_atom;
+int quicktime_read_stbl(quicktime_t *file, quicktime_minf_t *minf,
+                        quicktime_stbl_t *stbl, quicktime_atom_t *parent_atom)
+  {
+  quicktime_atom_t leaf_atom;
+  
+  do
+    {
+    quicktime_atom_read_header(file, &leaf_atom);
 
-	do
-	{
-		quicktime_atom_read_header(file, &leaf_atom);
+    //printf("quicktime_read_stbl 1\n");
+    /* mandatory */
+    if(quicktime_atom_is(&leaf_atom, "stsd"))
+      { 
+      quicktime_read_stsd(file, &(stbl->stsd)); 
+      /* Some codecs store extra information at the end of this */
+      quicktime_atom_skip(file, &leaf_atom);
+      }
+    else if(quicktime_atom_is(&leaf_atom, "stts"))
+      {
+      quicktime_read_stts(file, &(stbl->stts));
+      quicktime_atom_skip(file, &leaf_atom);
+      }
+    else if(quicktime_atom_is(&leaf_atom, "ctts"))
+      {
+      quicktime_read_ctts(file, &(stbl->ctts));
+      quicktime_atom_skip(file, &leaf_atom);
+      stbl->has_ctts = 1;
+      }
+    else if(quicktime_atom_is(&leaf_atom, "stss"))
+      {
+      quicktime_read_stss(file, &(stbl->stss));
+      quicktime_atom_skip(file, &leaf_atom);
+      }
+    else if(quicktime_atom_is(&leaf_atom, "stsc"))
+      {
+      quicktime_read_stsc(file, &(stbl->stsc));
+      quicktime_atom_skip(file, &leaf_atom);
+      }
+    else if(quicktime_atom_is(&leaf_atom, "stsz"))
+      {
+      quicktime_read_stsz(file, &(stbl->stsz));
+      quicktime_atom_skip(file, &leaf_atom);
+      }
+    else if(quicktime_atom_is(&leaf_atom, "co64"))
+      {
+      quicktime_read_stco64(file, &(stbl->stco));
+      quicktime_atom_skip(file, &leaf_atom);
+      }
+    else if(quicktime_atom_is(&leaf_atom, "stco"))
+      {
+      quicktime_read_stco(file, &(stbl->stco));
+      quicktime_atom_skip(file, &leaf_atom);
+      }
+    else
+      quicktime_atom_skip(file, &leaf_atom);
+    }while(quicktime_position(file) < parent_atom->end);
 
-//printf("quicktime_read_stbl 1\n");
-/* mandatory */
-		if(quicktime_atom_is(&leaf_atom, "stsd"))
-		{ 
-			quicktime_read_stsd(file, minf, &(stbl->stsd)); 
-/* Some codecs store extra information at the end of this */
-			quicktime_atom_skip(file, &leaf_atom);
-		}
-		else
-		if(quicktime_atom_is(&leaf_atom, "stts"))
-			{ quicktime_read_stts(file, &(stbl->stts)); }
-		else
-		if(quicktime_atom_is(&leaf_atom, "stss"))
-			{ quicktime_read_stss(file, &(stbl->stss)); }
-		else
-		if(quicktime_atom_is(&leaf_atom, "stsc"))
-			{ quicktime_read_stsc(file, &(stbl->stsc)); }
-		else
-		if(quicktime_atom_is(&leaf_atom, "stsz"))
-			{ quicktime_read_stsz(file, &(stbl->stsz)); }
-		else
-		if(quicktime_atom_is(&leaf_atom, "co64"))
-			{ quicktime_read_stco64(file, &(stbl->stco)); }
-		else
-		if(quicktime_atom_is(&leaf_atom, "stco"))
-			{ quicktime_read_stco(file, &(stbl->stco)); }
-		else
-			quicktime_atom_skip(file, &leaf_atom);
-	}while(quicktime_position(file) < parent_atom->end);
-
-	return 0;
-}
+  return 0;
+  }
 
 void quicktime_write_stbl(quicktime_t *file, quicktime_minf_t *minf, quicktime_stbl_t *stbl)
 {
@@ -147,7 +190,9 @@ void quicktime_write_stbl(quicktime_t *file, quicktime_minf_t *minf, quicktime_s
 	quicktime_write_stsc(file, &(stbl->stsc));
 	quicktime_write_stsz(file, &(stbl->stsz));
 	quicktime_write_stco(file, &(stbl->stco));
-
+        if(stbl->has_ctts)
+          quicktime_write_ctts(file, &(stbl->ctts));
+        
 	quicktime_atom_write_footer(file, &atom);
 }
 
