@@ -9,6 +9,8 @@ int quicktime_trak_init(quicktime_trak_t *trak)
 	quicktime_tkhd_init(&(trak->tkhd));
 	quicktime_edts_init(&(trak->edts));
 	quicktime_mdia_init(&(trak->mdia));
+	quicktime_tref_init(&(trak->tref));
+	trak->has_tref = 0;
 	return 0;
 }
 
@@ -36,6 +38,22 @@ int quicktime_trak_init_video(quicktime_t *file,
 	return 0;
 }
 
+
+int quicktime_trak_init_qtvr(quicktime_t *file, quicktime_trak_t *trak, int track_type, int width, int height, int frame_duration, int timescale)
+{
+
+	quicktime_tkhd_init_video(file, 
+		&(trak->tkhd), 
+		height, 
+		height/2);
+	quicktime_mdia_init_qtvr(file, 
+		&(trak->mdia), track_type, width, height, timescale, frame_duration);
+	quicktime_edts_init_table(&(trak->edts));
+	quicktime_tref_init_qtvr(&(trak->tref), track_type);
+
+	return 0;
+}
+
 int quicktime_trak_init_panorama(quicktime_t *file, quicktime_trak_t *trak, int width, int height, int frame_duration, int timescale)
 {
 
@@ -73,6 +91,7 @@ int quicktime_trak_delete(quicktime_trak_t *trak)
 	quicktime_mdia_delete(&(trak->mdia));
 	quicktime_edts_delete(&(trak->edts));
 	quicktime_tkhd_delete(&(trak->tkhd));
+	quicktime_tref_delete(&(trak->tref));
 
         if(trak->chunk_sizes)
           free(trak->chunk_sizes);
@@ -85,6 +104,8 @@ int quicktime_trak_dump(quicktime_trak_t *trak)
 	printf(" track\n");
 	quicktime_tkhd_dump(&(trak->tkhd));
 	quicktime_edts_dump(&(trak->edts));
+	if (trak->has_tref)
+	    quicktime_tref_dump(&(trak->tref));
 	quicktime_mdia_dump(&(trak->mdia));
 
 	return 0;
@@ -129,7 +150,7 @@ int quicktime_read_trak(quicktime_t *file, quicktime_trak_t *trak, quicktime_ato
 			{ quicktime_read_tkhd(file, &(trak->tkhd)); }
 		else
 		if(quicktime_atom_is(&leaf_atom, "mdia"))
-			{ quicktime_read_mdia(file, &(trak->mdia), &leaf_atom); }
+                  { quicktime_read_mdia(file, trak, &(trak->mdia), &leaf_atom); }
 		else
 /* optional */
 		if(quicktime_atom_is(&leaf_atom, "clip"))
@@ -144,14 +165,14 @@ int quicktime_read_trak(quicktime_t *file, quicktime_trak_t *trak, quicktime_ato
 		if(quicktime_atom_is(&leaf_atom, "load"))
 			{ quicktime_atom_skip(file, &leaf_atom); }
 		else
-		if(quicktime_atom_is(&leaf_atom, "tref"))
-			{ quicktime_atom_skip(file, &leaf_atom); }
-		else
 		if(quicktime_atom_is(&leaf_atom, "imap"))
 			{ quicktime_atom_skip(file, &leaf_atom); }
 		else
 		if(quicktime_atom_is(&leaf_atom, "udta"))
 			{ quicktime_atom_skip(file, &leaf_atom); }
+		else
+		if(quicktime_atom_is(&leaf_atom, "tref"))
+			{ trak->has_tref = 1; quicktime_read_tref(file, &(trak->tref)); }
 		else
 			quicktime_atom_skip(file, &leaf_atom);
 //printf("quicktime_read_trak %llx %llx\n", quicktime_position(file), leaf_atom.end);
@@ -180,61 +201,49 @@ int quicktime_write_trak(quicktime_t *file,
 	if (trak->mdia.minf.is_panorama) trak->edts.elst.total_entries = 1;
 	quicktime_write_edts(file, &(trak->edts), trak->tkhd.duration);
 	quicktime_write_mdia(file, &(trak->mdia));
-
+	
+	if (trak->mdia.minf.is_qtvr) 
+	    quicktime_write_tref(file, &(trak->tref));
+//	printf("isobj: %i",trak->mdia.minf.is_object );
+//	if (trak->mdia.minf.is_object) 
+//	    quicktime_write_tref(file, &(trak->tref));
+	
 	quicktime_atom_write_footer(file, &atom);
 
 	return 0;
 }
 
-int64_t quicktime_track_end(quicktime_trak_t *trak)
+int64_t quicktime_track_samples(quicktime_t *file, quicktime_trak_t *trak)
 {
-/* get the byte endpoint of the track in the file */
-	int64_t size = 0;
-	int64_t chunk, chunk_offset, chunk_samples, sample;
-	quicktime_stsz_t *stsz = &(trak->mdia.minf.stbl.stsz);
-	quicktime_stsc_t *stsc = &(trak->mdia.minf.stbl.stsc);
-	quicktime_stco_t *stco;
+		quicktime_stts_t *stts = &(trak->mdia.minf.stbl.stts);
+		int i;
+		int64_t total = 0;
 
-/* get the last chunk offset */
-/* the chunk offsets contain the HEADER_LENGTH themselves */
-	stco = &(trak->mdia.minf.stbl.stco);
-	chunk = stco->total_entries;
-	size = chunk_offset = stco->table[chunk - 1].offset;
-
-/* get the number of samples in the last chunk */
-	chunk_samples = stsc->table[stsc->total_entries - 1].samples;
-
-/* get the size of last samples */
-	if(stsz->sample_size)
-	{
-/* assume audio so calculate the sample size */
-		size += chunk_samples * stsz->sample_size
-			* trak->mdia.minf.stbl.stsd.table[0].channels 
-			* trak->mdia.minf.stbl.stsd.table[0].sample_size / 8;
-	}
-	else
-	{
-/* assume video */
-		for(sample = stsz->total_entries - chunk_samples; 
-			sample < stsz->total_entries; sample++)
-		{
-			size += stsz->table[sample].size;
-		}
-	}
-
-	return size;
-}
-
-long quicktime_track_samples(quicktime_t *file, quicktime_trak_t *trak)
-{
 /*printf("file->rd %d file->wr %d\n", file->rd, file->wr); */
 	if(file->wr)
 	{
-/* get the sample count when creating a new file */
+        if(trak->mdia.minf.is_audio)
+          {
+          for(i = 0; i < stts->total_entries; i++)
+            {
+            total += stts->table[i].sample_count *
+              stts->table[i].sample_duration;
+            }
+          }
+        else
+          {
+          for(i = 0; i < stts->total_entries; i++)
+            {
+            total += stts->table[i].sample_count;
+            }
+          }
+        return total;
+#if 0
+        /* get the sample count when creating a new file */
  		quicktime_stsc_table_t *table = trak->mdia.minf.stbl.stsc.table;
 		long total_entries = trak->mdia.minf.stbl.stsc.total_entries;
 		long chunk = trak->mdia.minf.stbl.stco.total_entries;
-		long sample;
+		int64_t sample;
 
 		if(chunk)
 		{
@@ -245,20 +254,18 @@ long quicktime_track_samples(quicktime_t *file, quicktime_trak_t *trak)
 			sample = 0;
 
 		return sample;
+#endif
 	}
 	else
 	{
 /* get the sample count when reading only */
-		quicktime_stts_t *stts = &(trak->mdia.minf.stbl.stts);
-		int i;
-		long total = 0;
                 /* Get this from the AVI header */
                 //                if(trak->strl)
                 //                  {
                 //                  total = 
                 //                  }
                 /* LQT: Make this correct for VBR files */
-                if(trak->mdia.minf.stbl.stsd.table[0].compression_id == -2)
+                if(trak->mdia.minf.is_audio)
                   {
                   for(i = 0; i < stts->total_entries; i++)
                     {
@@ -329,52 +336,52 @@ int quicktime_avg_chunk_samples(quicktime_t *file, quicktime_trak_t *trak)
 }
 
 int quicktime_chunk_of_sample(int64_t *chunk_sample, 
-	int64_t *chunk, 
-	quicktime_trak_t *trak, 
-	long sample)
-{
-	quicktime_stsc_table_t *table = trak->mdia.minf.stbl.stsc.table;
-	long total_entries = trak->mdia.minf.stbl.stsc.total_entries;
-	long chunk2entry;
-	long chunk1, chunk2, chunk1samples, range_samples, total = 0;
+                              int64_t *chunk, 
+                              quicktime_trak_t *trak, 
+                              int64_t sample)
+  {
+  quicktime_stsc_table_t *table = trak->mdia.minf.stbl.stsc.table;
+  long total_entries = trak->mdia.minf.stbl.stsc.total_entries;
+  long chunk2entry;
+  long chunk1, chunk2, chunk1samples, range_samples, total = 0;
+  
+  chunk1 = 1;
+  chunk1samples = 0;
+  chunk2entry = 0;
+  
+  if(!total_entries)
+    {
+    *chunk_sample = 0;
+    *chunk = 0;
+    return 0;
+    }
 
-	chunk1 = 1;
-	chunk1samples = 0;
-	chunk2entry = 0;
+  do
+    {
+    chunk2 = table[chunk2entry].chunk;
+    *chunk = chunk2 - chunk1;
+    range_samples = *chunk * chunk1samples;
 
-	if(!total_entries)
-	{
-		*chunk_sample = 0;
-		*chunk = 0;
-		return 0;
-	}
+    if(sample < total + range_samples) break;
 
-	do
-	{
-		chunk2 = table[chunk2entry].chunk;
-		*chunk = chunk2 - chunk1;
-		range_samples = *chunk * chunk1samples;
+    chunk1samples = table[chunk2entry].samples;
+    chunk1 = chunk2;
 
-		if(sample < total + range_samples) break;
+    if(chunk2entry < total_entries)
+      {
+      chunk2entry++;
+      total += range_samples;
+      }
+    }while(chunk2entry < total_entries);
 
-		chunk1samples = table[chunk2entry].samples;
-		chunk1 = chunk2;
+  if(chunk1samples)
+    *chunk = (sample - total) / chunk1samples + chunk1;
+  else
+    *chunk = 1;
 
-		if(chunk2entry < total_entries)
-		{
-			chunk2entry++;
-			total += range_samples;
-		}
-	}while(chunk2entry < total_entries);
-
-	if(chunk1samples)
-		*chunk = (sample - total) / chunk1samples + chunk1;
-	else
-		*chunk = 1;
-
-	*chunk_sample = total + (*chunk - chunk1) * chunk1samples;
-	return 0;
-}
+  *chunk_sample = total + (*chunk - chunk1) * chunk1samples;
+  return 0;
+  }
 
 int64_t quicktime_chunk_to_offset(quicktime_t *file,
                                   quicktime_trak_t *trak, long chunk)
@@ -391,13 +398,6 @@ int64_t quicktime_chunk_to_offset(quicktime_t *file,
 	else
 		result = HEADER_LENGTH * 2;
 
-// Skip chunk header for AVI.  Skip it here instead of in read_chunk because some
-// codecs can't use read_chunk
-	if(file->use_avi)
-	{
-//printf("quicktime_chunk_to_offset 1 %llx %d\n", result, file->mdat.atom.start);
-		result += 8 + file->mdat.atom.start;
-	}
 	return result;
 }
 
@@ -420,17 +420,6 @@ long quicktime_offset_to_chunk(int64_t *chunk_offset,
 	return 1;
 }
 
-int quicktime_chunk_bytes(quicktime_t *file, 
-	int64_t *chunk_offset,
-	int chunk, 
-	quicktime_trak_t *trak)
-{
-	int result;
-	*chunk_offset = quicktime_chunk_to_offset(file, trak, chunk);
-	quicktime_set_position(file, *chunk_offset - 4);
-	result = quicktime_read_int32_le(file);
-	return result;
-}
 
 int64_t quicktime_sample_range_size(quicktime_trak_t *trak, 
 	long chunk_sample, 
@@ -462,71 +451,52 @@ int64_t i, total;
 	return total;
 }
 
-int64_t quicktime_sample_to_offset(quicktime_t *file, quicktime_trak_t *trak, long sample)
+int64_t quicktime_sample_to_offset(quicktime_t *file,
+                                   quicktime_trak_t *trak, long sample)
 {
 	int64_t chunk, chunk_sample, chunk_offset1, chunk_offset2;
 
 	quicktime_chunk_of_sample(&chunk_sample, &chunk, trak, sample);
 	chunk_offset1 = quicktime_chunk_to_offset(file, trak, chunk);
-	chunk_offset2 = chunk_offset1 + quicktime_sample_range_size(trak, chunk_sample, sample);
+	chunk_offset2 = chunk_offset1 +
+          quicktime_sample_range_size(trak, chunk_sample, sample);
 	return chunk_offset2;
 }
 
-long quicktime_offset_to_sample(quicktime_trak_t *trak, int64_t offset)
-{
-	int64_t chunk_offset;
-	int64_t chunk = quicktime_offset_to_chunk(&chunk_offset, trak, offset);
-	int64_t chunk_sample = quicktime_sample_of_chunk(trak, chunk);
-	int64_t sample, sample_offset;
-	quicktime_stsz_table_t *table = trak->mdia.minf.stbl.stsz.table;
-	int64_t total_samples = trak->mdia.minf.stbl.stsz.total_entries;
-
-	if(trak->mdia.minf.stbl.stsz.sample_size)
-	{
-		sample = chunk_sample + (offset - chunk_offset) / 
-			trak->mdia.minf.stbl.stsz.sample_size;
-	}
-	else
-	for(sample = chunk_sample, sample_offset = chunk_offset; 
-		sample_offset < offset && sample < total_samples; )
-	{
-		sample_offset += table[sample].size;
-		if(sample_offset < offset) sample++;
-	}
-	
-	return sample;
-}
-
 void quicktime_write_chunk_header(quicktime_t *file, 
 	quicktime_trak_t *trak, 
 	quicktime_atom_t *chunk)
-{
-	if(file->use_avi)
-	{
-/* Get tag from first riff strl */
-		quicktime_riff_t *first_riff = file->riff[0];
-		quicktime_hdrl_t *hdrl = &first_riff->hdrl;
-		quicktime_strl_t *strl = hdrl->strl[trak->tkhd.track_id - 1];
-		char *tag = strl->tag;
+  {
+  if(file->file_type & (LQT_FILE_AVI|LQT_FILE_AVI_ODML))
+    {
+    /* Get tag from first riff strl */
+    quicktime_riff_t *first_riff = file->riff[0];
+    quicktime_hdrl_t *hdrl = &first_riff->hdrl;
+    quicktime_strl_t *strl = hdrl->strl[trak->tkhd.track_id - 1];
+    char *tag = strl->tag;
+    
+    /* Create new RIFF object at 1 Gig mark */
 
-/* Create new RIFF object at 1 Gig mark */
-		quicktime_riff_t *riff = file->riff[file->total_riffs - 1];
-		if(quicktime_position(file) - riff->atom.start > 0x40000000)
-		{
-			quicktime_finalize_riff(file, riff);
-			quicktime_init_riff(file);
-		}
-
-		
-
-/* Write AVI header */
-		quicktime_atom_write_header(file, chunk, tag);
-	}
-	else
-	{
-		chunk->start = quicktime_position(file);
-	}
-}
+    if(file->file_type == LQT_FILE_AVI_ODML)
+      {
+      quicktime_riff_t *riff = file->riff[file->total_riffs - 1];
+      //      fprintf(stderr, "Write chunk header %lld %d\n",
+      //              quicktime_position(file) - riff->atom.start, file->max_riff_size);
+      if(quicktime_position(file) - riff->atom.start > file->max_riff_size)
+        {
+        quicktime_finalize_riff(file, riff);
+        quicktime_init_riff(file);
+        }
+      }
+    
+    /* Write AVI header */
+    quicktime_atom_write_header(file, chunk, tag);
+    }
+  else
+    {
+    chunk->start = quicktime_position(file);
+    }
+  }
 
 void quicktime_write_chunk_footer(quicktime_t *file, 
 	quicktime_trak_t *trak,
@@ -538,27 +508,30 @@ void quicktime_write_chunk_footer(quicktime_t *file,
 	int sample_size = quicktime_position(file) - offset;
 
 // Write AVI footer
-	if(file->use_avi)
-	{
-		quicktime_atom_write_footer(file, chunk);
-
-// Save original index entry for first RIFF only
-		if(file->total_riffs < 2)
-		{
-			quicktime_update_idx1table(file, 
-				trak, 
-				offset, 
-				sample_size);
-		}
-
-// Save partial index entry
-		quicktime_update_ixtable(file, 
-			trak, 
-			offset, 
-			sample_size);
-	}
-
-	if(offset + sample_size > file->mdat.atom.size)
+	if(file->file_type & (LQT_FILE_AVI|LQT_FILE_AVI_ODML))
+          {
+          quicktime_atom_write_footer(file, chunk);
+          
+          // Save original index entry for first RIFF only
+          if(file->total_riffs < 2)
+            {
+            quicktime_update_idx1table(file, 
+                                       trak, 
+                                       offset, 
+                                       sample_size);
+            }
+          
+          // Save partial index entry
+          if(file->file_type == LQT_FILE_AVI_ODML)
+            quicktime_update_ixtable(file, trak, offset, sample_size);
+          
+          //          fprintf(stderr, "quicktime_write_chunk_footer: %d %d\n",
+          //                  sample_size, trak->strl->strh.dwSuggestedBufferSize);
+          
+          if(sample_size > trak->strl->strh.dwSuggestedBufferSize)
+            trak->strl->strh.dwSuggestedBufferSize = ((sample_size+15)/16)*16;
+          }
+        if(offset + sample_size > file->mdat.atom.size)
 		file->mdat.atom.size = offset + sample_size;
 
 	quicktime_update_stco(&(trak->mdia.minf.stbl.stco), 
@@ -569,45 +542,28 @@ void quicktime_write_chunk_footer(quicktime_t *file,
 		quicktime_update_stsz(&(trak->mdia.minf.stbl.stsz), 
 		current_chunk - 1, 
 		sample_size);
-
+        /* Need to increase sample count (the VBR routines to it
+           themselves) */
+	if(trak->mdia.minf.is_audio && !trak->mdia.minf.is_audio_vbr)
+          trak->mdia.minf.stbl.stts.table->sample_count += samples;
+        
 	if(trak->mdia.minf.is_panorama) {
 		quicktime_update_stsz(&(trak->mdia.minf.stbl.stsz), 
 		current_chunk - 1, 
 		sample_size);	
 	}
 	
-	quicktime_update_stsc(&(trak->mdia.minf.stbl.stsc), 
-		current_chunk, 
-		samples);
+	if(trak->mdia.minf.is_qtvr) {
+		quicktime_update_stsz(&(trak->mdia.minf.stbl.stsz), 
+		current_chunk - 1, 
+		sample_size);
+	}
+        quicktime_update_stsc(&(trak->mdia.minf.stbl.stsc), 
+                              current_chunk, 
+                              samples);
+        
 }
 
-
-/*
- * int quicktime_update_tables(quicktime_t *file, 
- * 							quicktime_trak_t *trak, 
- * 							int64_t offset, 
- * 							int64_t chunk, 
- * 							int64_t sample, 
- * 							int64_t samples, 
- * 							int64_t sample_size)
- * {
- * //if(file->use_avi)
- * printf("quicktime_update_tables: replaced by quicktime_write_chunk_header and quicktime_write_chunk_footer\n");
- * 
- * 	if(offset + sample_size > file->mdat.atom.size) 
- * 		file->mdat.atom.size = offset + sample_size;
- * 
- * 	quicktime_update_stco(&(trak->mdia.minf.stbl.stco), chunk, offset);
- * 
- * 	if(trak->mdia.minf.is_video)
- * 		quicktime_update_stsz(&(trak->mdia.minf.stbl.stsz), sample, sample_size);
- * 
- * 
- * 	quicktime_update_stsc(&(trak->mdia.minf.stbl.stsc), chunk, samples);
- * 	return 0;
- * }
- */
-
 int quicktime_trak_duration(quicktime_trak_t *trak, 
 	long *duration, 
 	long *timescale)
@@ -636,12 +592,16 @@ int quicktime_trak_fix_counts(quicktime_t *file, quicktime_trak_t *trak)
           if(trak->mdia.minf.stbl.stts.total_entries == 1)
             trak->mdia.minf.stbl.stts.table[0].sample_count = samples;
           }
+        else if(trak->mdia.minf.is_audio_vbr)
+          {
+          quicktime_compress_stts(&(trak->mdia.minf.stbl.stts));
+          }
         else
           trak->mdia.minf.stbl.stts.table[0].sample_count = samples;
-
+        
         if(!trak->mdia.minf.stbl.stsz.total_entries)
 	{
-		trak->mdia.minf.stbl.stsz.sample_size = 1;
+        // trak->mdia.minf.stbl.stsz.sample_size = 1;
 		trak->mdia.minf.stbl.stsz.total_entries = samples;
 	}
 
