@@ -1,6 +1,7 @@
 #include "config.h"
 
 #include <fcntl.h>
+#include <math.h>
 #include <stdio.h>
 #include <sys/ioctl.h>
 #include <sys/stat.h>
@@ -13,9 +14,14 @@
 #include <workarounds.h>
 #include <string.h>
 
+
+#ifndef HAVE_LRINT
+#define lrint(x) ((long int)(x))
+#endif
+
 /* Disk I/O */
 
-int64_t quicktime_get_file_length(char *path)
+int64_t quicktime_get_file_length(const char *path)
 {
       struct stat status;
       if(stat(path, &status))
@@ -23,7 +29,7 @@ int64_t quicktime_get_file_length(char *path)
       return status.st_size;
 }
 
-int quicktime_file_open(quicktime_t *file, char *path, int rd, int wr)
+int quicktime_file_open(quicktime_t *file, const char *path, int rd, int wr)
 {
 	int exists = 0;
 	char flags[10];
@@ -144,9 +150,9 @@ int quicktime_read_data(quicktime_t *file, uint8_t *data, int64_t size)
 		if(selection_end - selection_start > file->preload_size)
 		{
 /* Size is larger than preload size.  Should never happen. */
-printf("read data Size is larger than preload size. size=%llx preload_size=%llx\n",
-       (long long)(selection_end - selection_start),
-       (long long)(file->preload_size));
+// printf("read data Size is larger than preload size. size=%llx preload_size=%llx\n",
+//       (long long)(selection_end - selection_start),
+//       (long long)(file->preload_size));
 			quicktime_fseek(file, file->file_position);
 			result = fread(data, size, 1, file->stream);
 			file->ftell_position += size;
@@ -319,6 +325,7 @@ float quicktime_read_fixed32(quicktime_t *file)
 		return a;
 }
 
+
 int quicktime_write_fixed32(quicktime_t *file, float number)
 {
 	unsigned char data[4];
@@ -329,11 +336,178 @@ int quicktime_write_fixed32(quicktime_t *file, float number)
 	data[0] = a >> 8;
 	data[1] = a & 0xff;
 	data[2] = b >> 8;
+	
 	data[3] = b & 0xff;
 
 	return quicktime_write_data(file, data, 4);
 }
 
+static float
+float32_be_read (unsigned char *cptr)
+{       int             exponent, mantissa, negative ;
+        float   fvalue ;
+
+        negative = cptr [0] & 0x80 ;
+        exponent = ((cptr [0] & 0x7F) << 1) | ((cptr [1] & 0x80) ? 1 : 0) ;
+        mantissa = ((cptr [1] & 0x7F) << 16) | (cptr [2] << 8) | (cptr [3]) ;
+
+        if (! (exponent || mantissa))
+                return 0.0 ;
+
+        mantissa |= 0x800000 ;
+        exponent = exponent ? exponent - 127 : 0 ;
+
+        fvalue = mantissa ? ((float) mantissa) / ((float) 0x800000) : 0.0 ;
+
+        if (negative)
+                fvalue *= -1 ;
+
+        if (exponent > 0)
+                fvalue *= (1 << exponent) ;
+        else if (exponent < 0)
+                fvalue /= (1 << abs (exponent)) ;
+
+        return fvalue ;
+} /* float32_be_read */
+
+static double
+double64_be_read (unsigned char *cptr)
+{       int             exponent, negative ;
+        double  dvalue ;
+
+        negative = (cptr [0] & 0x80) ? 1 : 0 ;
+        exponent = ((cptr [0] & 0x7F) << 4) | ((cptr [1] >> 4) & 0xF) ;
+
+        /* Might not have a 64 bit long, so load the mantissa into a double. */
+        dvalue = (((cptr [1] & 0xF) << 24) | (cptr [2] << 16) | (cptr [3] << 8) | cptr [4]) ;
+        dvalue += ((cptr [5] << 16) | (cptr [6] << 8) | cptr [7]) / ((double) 0x1000000) ;
+
+        if (exponent == 0 && dvalue == 0.0)
+                return 0.0 ;
+
+        dvalue += 0x10000000 ;
+
+        exponent = exponent - 0x3FF ;
+
+        dvalue = dvalue / ((double) 0x10000000) ;
+
+        if (negative)
+                dvalue *= -1 ;
+
+        if (exponent > 0)
+                dvalue *= (1 << exponent) ;
+        else if (exponent < 0)
+                dvalue /= (1 << abs (exponent)) ;
+
+        return dvalue ;
+} /* double64_be_read */
+
+static void
+float32_be_write (float in, unsigned char *out)
+{       int             exponent, mantissa, negative = 0 ;
+
+        memset (out, 0, sizeof (int)) ;
+
+        if (in == 0.0)
+                return ;
+
+        if (in < 0.0)
+        {       in *= -1.0 ;
+                negative = 1 ;
+                } ;
+
+        in = frexp (in, &exponent) ;
+
+        exponent += 126 ;
+
+        in *= (float) 0x1000000 ;
+        mantissa = (((int) in) & 0x7FFFFF) ;
+
+        if (negative)
+                out [0] |= 0x80 ;
+
+        if (exponent & 0x01)
+                out [1] |= 0x80 ;
+
+        out [3] = mantissa & 0xFF ;
+        out [2] = (mantissa >> 8) & 0xFF ;
+        out [1] |= (mantissa >> 16) & 0x7F ;
+        out [0] |= (exponent >> 1) & 0x7F ;
+
+        return ;
+} /* float32_be_write */
+
+
+static void
+double64_be_write (double in, unsigned char *out)
+{       int             exponent, mantissa ;
+
+        memset (out, 0, sizeof (double)) ;
+
+        if (in == 0.0)
+                return ;
+
+        if (in < 0.0)
+        {       in *= -1.0 ;
+                out [0] |= 0x80 ;
+                } ;
+
+        in = frexp (in, &exponent) ;
+
+        exponent += 1022 ;
+
+        out [0] |= (exponent >> 4) & 0x7F ;
+        out [1] |= (exponent << 4) & 0xF0 ;
+
+        in *= 0x20000000 ;
+        mantissa = lrint (floor (in)) ;
+
+        out [1] |= (mantissa >> 24) & 0xF ;
+        out [2] = (mantissa >> 16) & 0xFF ;
+        out [3] = (mantissa >> 8) & 0xFF ;
+        out [4] = mantissa & 0xFF ;
+
+        in = fmod (in, 1.0) ;
+        in *= 0x1000000 ;
+        mantissa = lrint (floor (in)) ;
+
+        out [5] = (mantissa >> 16) & 0xFF ;
+        out [6] = (mantissa >> 8) & 0xFF ;
+        out [7] = mantissa & 0xFF ;
+
+        return ;
+} /* double64_be_write */
+
+
+float quicktime_read_float32(quicktime_t *file)
+  {
+  unsigned char b[4];
+  quicktime_read_data(file, b, 4);
+  return float32_be_read(b);
+  }
+
+double quicktime_read_double64(quicktime_t *file)
+  {
+  unsigned char b[8];
+  quicktime_read_data(file, b, 8);
+  return double64_be_read(b);
+  }
+
+
+int quicktime_write_float32(quicktime_t *file, float value)
+  {
+  unsigned char b[4];
+  float32_be_write(value, b);
+  return quicktime_write_data(file, b, 4);
+  }
+
+int quicktime_write_double64(quicktime_t *file, double value)
+  {
+  unsigned char b[8];
+  double64_be_write(value, b);
+  return quicktime_write_data(file, b, 8);
+  }
+
 int quicktime_write_int64(quicktime_t *file, int64_t value)
 {
 	unsigned char data[8];
@@ -669,3 +843,38 @@ int quicktime_match_24(char *input, char *output)
 	else 
 		return 0;
 }
+
+void do_hexdump(uint8_t * data, int len, int linebreak, FILE * f)
+  {
+  int i;
+  int bytes_written = 0;
+  int imax;
+
+  while(bytes_written < len)
+    {
+    imax = (bytes_written + linebreak > len) ? len - bytes_written : linebreak;
+    for(i = 0; i < imax; i++)
+      fprintf(f, "%02x ", data[bytes_written + i]);
+    for(i = imax; i < linebreak; i++)
+      fprintf(f, "   ");
+    for(i = 0; i < imax; i++)
+      {
+      if(!(data[bytes_written + i] & 0x80) && (data[bytes_written + i] >= 32))
+        fprintf(f, "%c", data[bytes_written + i]);
+      else
+        fprintf(f, ".");
+      }
+    bytes_written += imax;
+    fprintf(f, "\n");
+    }
+  }
+
+void lqt_hexdump(uint8_t * data, int len, int linebreak)
+  {
+  do_hexdump(data, len, linebreak, stderr);  
+  }
+
+void lqt_hexdump_stdout(uint8_t * data, int len, int linebreak)
+  {
+  do_hexdump(data, len, linebreak, stdout);  
+  }
