@@ -5,7 +5,7 @@
  *
  */
 
-#define USE_GL
+// #define USE_GL
 
 #include "config.h"
 
@@ -15,6 +15,8 @@
 #include <strings.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <time.h>
+#include <math.h>
 
 #include <sys/types.h>
 #include <sys/time.h>
@@ -41,7 +43,7 @@
 #include <X11/extensions/Xv.h>
 #include <X11/extensions/Xvlib.h>
 
-#ifdef USE_GL
+#ifdef HAVE_GL
 
 #include <GL/gl.h>
 #include <GL/glu.h>
@@ -84,10 +86,14 @@ static int no_mitshm    = 0;
 static int pixmap_bytes = 0;
 static int x11_byteswap = 0;
 
-#ifdef USE_GL
+#ifdef HAVE_GL
 static int use_gl       = 0;
 #endif
 
+static int xpos = 0;
+static int ypos = 0;
+static int overhead_frames = 0;
+
 static unsigned long   lut_red[256];
 static unsigned long   lut_green[256];
 static unsigned long   lut_blue[256];
@@ -448,7 +454,7 @@ static void xv_blit(Window win, GC gc, XvImage *xi,
 
 /* ------------------------------------------------------------------------ */
 /* OpenGL code                                                              */
-#ifdef USE_GL
+#ifdef HAVE_GL
 static int gl_texture_width,gl_texture_height;
 static GLuint gl_texture;
 static int gl_attrib[] = { GLX_RGBA,
@@ -553,7 +559,7 @@ static void gl_init(Widget widget, int iw, int ih)
     use_gl = 1;
 }
 
-#endif // USE_GL
+#endif // HAVE_GL
 
 static int oss_sr,oss_hr;
 
@@ -780,13 +786,14 @@ static int16_t **qt_audion = (int16_t**)0;
 static int qt_channels,qt_sample_rate;
 static int qt_audio_eof = 0; /* No more samples can be decoded */
 
-static int qt_hasvideo,qt_hasaudio;
+static int qt_hasvideo,qt_hasaudio,qt_isqtvr;
 
-static int qt_width = 320, qt_height = 32, qt_drop = 0, qt_droptotal = 0;
+static int qt_width = 320, qt_height = 32, qt_drop = 0, qt_droptotal = 0, qtvr_dwidth = 0, qtvr_dheight = 0;
 static int64_t qt_frame_time; /* Timestamp of the decoded frame */
 static int qt_timescale = 0;
 
 static unsigned char *qt_frame,**qt_rows;
+static unsigned char *qt_panorama_buffer;
 static XImage *qt_ximage;
 static XvImage *qt_xvimage;
 static GC qt_gc;
@@ -795,14 +802,16 @@ static GC qt_gc;
 static void qt_init(FILE *fp, char *filename)
 {
     char *str;
-    int i;
-
+    int i, j;
+    int ipos;
+    float minpan, maxpan;
+    const lqt_channel_t * channel_setup;
+    int num_channels;
     /* audio device */
     char *adev_name;
     
     /* default */
     adev_name = strdup("plughw");
-//    if (getenv("LQTPLAY_ALSA_DEV")) adev_name = getenv("LQTPLAY_ALSA_DEV");
 
     /* open file */
     qt = quicktime_open(filename,1,0);
@@ -847,6 +856,7 @@ static void qt_init(FILE *fp, char *filename)
     if (str)
 	fprintf(fp,"  comment:   %s\n",str);
     
+    
     /* print video info */
     if (quicktime_has_video(qt)) {
 	fprintf(fp,"  video: %d track(s)\n",quicktime_video_tracks(qt));
@@ -866,25 +876,71 @@ static void qt_init(FILE *fp, char *filename)
 		    quicktime_video_compressor(qt,i));
 	}
     }
-
+    
+    qt_isqtvr = lqt_is_qtvr(qt);
+    if (qt_isqtvr) {
+	if (qt_isqtvr != QTVR_OBJ && qt_isqtvr != QTVR_PAN) {
+	    fprintf(stderr, "'%s' is no QTVR file or an unsupported version.\n"
+			    "Currently only QTVR V1.0 object and panorama files are supported.\n", filename);
+	    exit(1);
+	}
+	
+	ipos = lqt_qtvr_initial_position(qt);
+	lqt_qtvr_get_pan(qt, &(minpan), &(maxpan), NULL);
+	qtvr_dwidth = lqt_qtvr_get_display_width(qt);
+	qtvr_dheight = lqt_qtvr_get_display_height(qt);
+	fprintf(stderr, "startpos :%i\n", ipos);
+	//fprintf(stderr, "fov :%f\n", lqt_qtvr_get_fov(qt));
+	fprintf(stderr, "movietype :%i\n", lqt_qtvr_get_movietype(qt));
+	fprintf(stderr, "panning :%f %f\n", minpan, maxpan);
+	fprintf(stderr, "rows :%i\n", lqt_qtvr_get_rows(qt));
+	fprintf(stderr, "colums :%i\n", lqt_qtvr_get_columns(qt));
+	fprintf(stderr, "disp width :%i\n", qtvr_dwidth);
+	fprintf(stderr, "disp height :%i\n", qtvr_dheight);
+	fprintf(stderr, "width :%i\n", lqt_qtvr_get_width(qt));
+	fprintf(stderr, "height :%i\n", lqt_qtvr_get_height(qt));
+	fprintf(stderr, "depth :%i\n", lqt_qtvr_get_depth(qt));
+    
+	if (qt_isqtvr == QTVR_PAN) {
+	    overhead_frames = (float)lqt_qtvr_get_height(qt) * ((float)qtvr_dwidth / (float)qtvr_dheight);
+	    overhead_frames = floor((float)overhead_frames / (float)quicktime_video_height(qt, 0)) * (float)quicktime_video_height(qt, 0) + quicktime_video_height(qt, 0);
+	    fprintf(stderr, "overhead_frames :%i\n",overhead_frames);
+	}
+    }
     /* print audio info */
     if (quicktime_has_audio(qt)) {
 	fprintf(fp,"  audio: %d track(s)\n",quicktime_audio_tracks(qt));
 	for (i = 0; i < quicktime_audio_tracks(qt); i++) {
-	    fprintf(fp,
-		    "    track #%d\n"
-		    "      rate  : %ld Hz\n"
-		    "      bits  : %d\n"
-		    "      chans : %d\n"
-		    "      codec : %s\n",
-		    i+1,
-		    quicktime_sample_rate(qt,i),
-		    quicktime_audio_bits(qt,i),
-		    quicktime_track_channels(qt,i),
-		    quicktime_audio_compressor(qt,i));
-	}
+          {
+          num_channels = quicktime_track_channels(qt,i);
+          fprintf(fp,
+                  "    track #%d\n"
+                  "      rate  : %ld Hz\n"
+                  "      bits  : %d\n"
+                  "      chans : %d\n"
+                  "      codec : %s\n",
+                  i+1,
+                  quicktime_sample_rate(qt,i),
+                  quicktime_audio_bits(qt,i),
+                  num_channels,
+                  quicktime_audio_compressor(qt,i));
+          channel_setup = lqt_get_channel_setup(qt, i);
+          fprintf(fp, "    channel_setup : ");
+          if(channel_setup)
+            {
+            for(j = 0; j < num_channels; j++)
+              {
+              fprintf(fp, "%s", lqt_channel_to_string(channel_setup[j]));
+              if(j < num_channels - 1)
+                fprintf(fp, ", ");
+              }
+            fprintf(fp, "\n");
+            }
+          else
+            fprintf(fp, "Not available\n");
+          }
+        }
     }
-
     /* sanity checks */
     if (!quicktime_has_video(qt)) {
 	fprintf(stderr,"WARNING: no video stream\n");
@@ -933,19 +989,34 @@ static int qt_init_video(void)
 {
         int i;
         /* init */
-	qt_frame = malloc(qt_width * qt_height * 4);
+	
 	qt_rows = malloc(qt_height * sizeof(char*));
+	if (qt_isqtvr == QTVR_PAN) {
+	    qt_frame = malloc(qtvr_dwidth * qtvr_dheight * 4);
+	    qt_panorama_buffer = malloc((lqt_qtvr_get_width(qt)+overhead_frames)*
+					 lqt_qtvr_get_height(qt)*
+					 3);
+	}
+	else
+	    qt_frame = malloc(qt_width * qt_height * 4);
+
+
 	qt_gc = XCreateGC(dpy,XtWindow(simple),0,NULL);
 	switch (qt_cmodel) {
 	case BC_RGB888:
-#ifdef USE_GL
+#ifdef HAVE_GL
           fprintf(stderr,"INFO: using BC_RGB888 + %s\n",
 		    use_gl ? "OpenGL" : "plain X11");
 #else
           fprintf(stderr,"INFO: using BC_RGB888 + %s\n",
 		    "plain X11");
 #endif
-	    qt_ximage = x11_create_ximage(dpy,qt_width,qt_height);
+	    if (qt_isqtvr == QTVR_PAN) {
+		qt_ximage = x11_create_ximage(dpy,qtvr_dwidth,qtvr_dheight);
+	    }
+	    else
+		qt_ximage = x11_create_ximage(dpy,qt_width,qt_height);
+	    
 	    for (i = 0; i < qt_height; i++)
 		qt_rows[i] = qt_frame + qt_width * 3 * i;
 	    break;
@@ -988,28 +1059,95 @@ static int qt_init_video(void)
 		    __FILE__,__LINE__);
 	    exit(1);
 	}
+	if (qt_isqtvr) {
+	    /* has to be done here to set initial pov */ // look into this
+	    quicktime_set_video_position(qt, lqt_qtvr_initial_position(qt),0);
+	    /*XSetInputFocus(dpy, XtWindow(simple), RevertToPointerRoot, CurrentTime);
+	    */
+	    if (qt_isqtvr == QTVR_PAN) {
+		int i,l,j,k;
+		long buffer_size;
+      
+		buffer_size = (lqt_qtvr_get_width(qt) + overhead_frames) * lqt_qtvr_get_height(qt) * 3;
+		for ( l = 0; l < lqt_qtvr_get_columns(qt) + (int)(overhead_frames/qt_height); l++) {
+		    /* set position in video file */
+		    /* if position larger than panorama, start from the beginning (overhead)*/
+		    quicktime_set_video_position(qt,
+						 (l>=lqt_qtvr_get_columns(qt))?l-lqt_qtvr_get_columns(qt):l,
+						 0);
+		    /* decode a frame */
+		    lqt_decode_video(qt, qt_rows, 0);
+		    
+		    /* set position in the output buffer */
+		    /* to the upper left corner of the desired frame */
+		    j = l * qt_height * 3;
+		    
+		    /* put frame, rotated into buffer */
+		    for (k = 0; k < qt_width; k++) {
+			for (i = qt_height - 1; i >= 0; i--) {
+			    qt_panorama_buffer[buffer_size - j + 0] = qt_frame[((qt_height - i) * qt_width - k) * 3 + 0];
+			    qt_panorama_buffer[buffer_size - j + 1] = qt_frame[((qt_height - i) * qt_width - k) * 3 + 1];
+			    qt_panorama_buffer[buffer_size - j + 2] = qt_frame[((qt_height - i) * qt_width - k) * 3 + 2];
+			    j = j + 3;
+			}
+			j += (lqt_qtvr_get_width(qt) + overhead_frames - qt_height) * 3;
+		    }
+		}
+	    }
+	}
         return 0;
 }
 
 static int qt_frame_decode(void)
   {
   int i;
-  if (quicktime_video_position(qt,0) >= quicktime_video_length(qt,0))
-      return -1;
-
-  if (qt_drop) {
-        qt_droptotal += qt_drop;
-	fprintf(stderr,"dropped %d frame(s)\r",qt_droptotal);
-	for (i = 0; i < qt_drop; i++)
-	    quicktime_read_frame(qt,qt_frame,0);
-	qt_drop = 0;
+  
+  if (qt_isqtvr == QTVR_OBJ) {
+	/* sanity check and decode  */
+	if (quicktime_video_position(qt, 0) >= 0 &&
+	    quicktime_video_position(qt, 0) < quicktime_video_length(qt,0))
+	    lqt_decode_video(qt, qt_rows, 0);
+	//    quicktime_decode_scaled(qt,0,0,qt_width,qt_height,qt_width,qt_height,
+    //			    qt_cmodel,qt_rows,0);
     }
-    qt_frame_time = lqt_frame_time(qt, 0);
-//    fprintf(stderr, "Frame time: %lld\n", qt_frame_time);
-    lqt_decode_video(qt, qt_rows, 0);
-//    quicktime_decode_scaled(qt,0,0,qt_width,qt_height,qt_width,qt_height,
+    else if (qt_isqtvr == QTVR_PAN) {
+	int i, j, k = 0;
+	i = (ypos * (lqt_qtvr_get_width(qt) + overhead_frames) + xpos) * 3;
+	do {
+	    for (j = 0; j < qtvr_dwidth * 3; j++) {
+		qt_frame[k] = qt_panorama_buffer[i];
+		i++; k++;
+	    }
+	    i+= (lqt_qtvr_get_width(qt) + overhead_frames - qtvr_dwidth) * 3;
+	} while (k < (qtvr_dwidth * qtvr_dheight) * 3);
+
+	do {
+	    for (j = 0; j < qtvr_dwidth * 3; j++) {
+		qt_frame[k] = qt_panorama_buffer[i];
+		i++; k++;
+	    }
+	    i+= (lqt_qtvr_get_width(qt) + overhead_frames - qtvr_dwidth) * 3;
+	} while (k < (qtvr_dwidth * qtvr_dheight) * 3);
+    }	
+    else {
+	if (quicktime_video_position(qt,0) >= quicktime_video_length(qt,0))
+	    return -1;
+    
+	if (qt_drop) {
+	    	qt_droptotal += qt_drop;
+		fprintf(stderr,"dropped %d frame(s)\r",qt_droptotal);
+	    for (i = 0; i < qt_drop; i++)
+		quicktime_read_frame(qt,qt_frame,0);
+	    qt_drop = 0;
+	  }
+	qt_frame_time = lqt_frame_time(qt, 0);
+//        fprintf(stderr, "Frame time: %lld\n", qt_frame_time);
+	lqt_decode_video(qt, qt_rows, 0);
+//        quicktime_decode_scaled(qt,0,0,qt_width,qt_height,qt_width,qt_height,
 //			    qt_cmodel,qt_rows,0);
-    return 0;
+
+    }
+  return 0;
   }
 
 static int qt_frame_blit(void)
@@ -1017,21 +1155,30 @@ static int qt_frame_blit(void)
 
     switch (qt_cmodel) {
     case BC_RGB888:
-#ifdef USE_GL
+#ifdef HAVE_GL
       if (use_gl) {
-	    gl_blit(simple,qt_frame,qt_width,qt_height,swidth,sheight);
+	  if (qt_isqtvr == QTVR_PAN) {
+	    gl_blit(simple,qt_frame,qtvr_dwidth,qtvr_dheight,swidth,sheight);
+	  } else gl_blit(simple,qt_frame,qt_width,qt_height,swidth,sheight);
 	} else
-#endif // USE_GL
+#endif // HAVE_GL
 
         {
 	    switch (pixmap_bytes) {
 	    case 2:
-              rgb_to_lut2((uint8_t*)(qt_ximage->data),qt_frame,qt_width*qt_height);
+                if (qt_isqtvr == QTVR_PAN) {
+		    rgb_to_lut2((uint8_t*)(qt_ximage->data),qt_frame,qtvr_dwidth*qtvr_dheight);
+		} else rgb_to_lut2((uint8_t*)(qt_ximage->data),qt_frame,qt_width*qt_height);
 		break;
 	    case 4:
-              rgb_to_lut4((uint8_t*)(qt_ximage->data),qt_frame,qt_width*qt_height);
+            if (qt_isqtvr == QTVR_PAN) {
+		rgb_to_lut4((uint8_t*)(qt_ximage->data),qt_frame,qtvr_dwidth*qtvr_dheight);
+	    } else  rgb_to_lut4((uint8_t*)(qt_ximage->data),qt_frame,qt_width*qt_height);
 		break;
 	    }
+	    if (qt_isqtvr == QTVR_PAN) {
+		x11_blit(XtWindow(simple),qt_gc,qt_ximage,qtvr_dwidth,qtvr_dheight);
+	    } else
 	    x11_blit(XtWindow(simple),qt_gc,qt_ximage,qt_width,qt_height);
 	}
 	break;
@@ -1127,7 +1274,8 @@ static int decode_audio()
   qt_audio_ptr = qt_audio;
   if(samples_decoded < AUDIO_BLOCK_SIZE)
     {
-    fprintf(stderr, "Audio track finished\n");
+    fprintf(stderr, "Audio track finished (got %d samples, wanted %d)\n",
+            samples_decoded, AUDIO_BLOCK_SIZE);
     qt_audio_eof = 1;
     }
   return samples_decoded;
@@ -1269,6 +1417,39 @@ static void usage(FILE *fp, char *prog)
 	    prog);
 }
 
+static void decode_qtvr()
+{
+    XEvent event;
+    int startpos;
+
+    startpos = quicktime_video_position(qt, 0);
+
+    XFlush(dpy);
+    if (qt_hasvideo) {
+	if (0 != qt_frame_decode()) exit(0);
+	if (0 != qt_frame_blit()) exit(0);
+    }
+
+    for (;;) {
+	struct timespec time;
+	
+//      printf("loop pos :%ld\n",quicktime_video_position(qt,0));
+	if (quicktime_video_position(qt,0) >= startpos + lqt_qtvr_get_loop_frames(qt))
+	    quicktime_set_video_position(qt, startpos, 0);
+
+	if (True == XCheckMaskEvent(dpy, ~0, &event)) {
+	    XtDispatchEvent(&event);
+	} else {
+//	printf("loop pos before decode :%ld\n",quicktime_video_position(qt,0));
+	    XFlush(dpy);
+	    XtAppProcessEvent(app_context, KeyPressMask||KeyReleaseMask);
+	    time.tv_nsec = 10000000;
+	    time.tv_sec = 0;
+	    nanosleep(&time, NULL);
+	}
+    }
+}
+
 static void quit_ac(Widget widget, XEvent *event,
 		    String *params, Cardinal *num_params)
 {
@@ -1283,22 +1464,135 @@ static void resize_ev(Widget widget, XtPointer client_data,
     case ConfigureNotify:
 	XtVaGetValues(widget,XtNheight,&sheight,XtNwidth,&swidth,NULL);
 	fprintf(stderr,"INFO: window size is %dx%d\n",swidth,sheight);
-#ifdef USE_GL
+#ifdef HAVE_GL
 	if (use_gl)
 	    gl_resize(widget,swidth,sheight);
 #endif
+	if (qt_isqtvr != 0) decode_qtvr();
+	break;
+    case UnmapNotify:
+    case DestroyNotify:
+    	exit(0);
 	break;
     }
 }
 
+/* event handlers */
+
+static void left_ac(Widget widget, XEvent *event,
+		    String *params, Cardinal *num_params)
+{
+    if (qt_isqtvr == QTVR_PAN) {
+	if (xpos - 10 >= 0 ) xpos -= 10;
+	if (xpos - 10 < 0) {
+	    xpos = xpos-10 + lqt_qtvr_get_width(qt);
+	}
+    }
+    else if (qt_isqtvr == QTVR_OBJ) {
+	int vpos = quicktime_video_position(qt, 0);
+	int lfxcolumns = lqt_qtvr_get_loop_frames(qt) * lqt_qtvr_get_columns(qt);
+	
+	if (vpos % lfxcolumns == lfxcolumns - lqt_qtvr_get_loop_frames(qt) && vpos != 0) {
+	    quicktime_set_video_position(qt, vpos - (lqt_qtvr_get_columns(qt)-1), 0); 
+	} 
+	else {	
+	    quicktime_set_video_position(qt, vpos + lqt_qtvr_get_loop_frames(qt), 0);
+	}
+    }
+    else
+	return;
+//    printf("pos %ld\n",quicktime_video_position(qt,0));
+    decode_qtvr();
+}
+
+
+static void right_ac(Widget widget, XEvent *event,
+		    String *params, Cardinal *num_params)
+{
+    if (qt_isqtvr == QTVR_PAN) {
+	if (xpos + 10 <= lqt_qtvr_get_width(qt)+overhead_frames - qtvr_dwidth) xpos += 10;
+	if (xpos + 10 > lqt_qtvr_get_width(qt)) {
+	    xpos = xpos+10 - lqt_qtvr_get_width(qt);
+	}
+    }
+    else if (qt_isqtvr == QTVR_OBJ) {
+	int vpos = quicktime_video_position(qt, 0);
+	int lfxcolumns = lqt_qtvr_get_loop_frames(qt) * lqt_qtvr_get_columns(qt);
+    
+	if (vpos % lfxcolumns == 0) { 
+	    quicktime_set_video_position(qt, vpos + (lqt_qtvr_get_loop_frames(qt) * lqt_qtvr_get_columns(qt)-1), 0);
+	} 
+	else {
+	    quicktime_set_video_position(qt, vpos - lqt_qtvr_get_loop_frames(qt), 0);
+	}
+    }
+    else
+	return;
+//    printf("pos %ld\n",quicktime_video_position(qt,0));
+    decode_qtvr();
+}
+
+
+static void up_ac(Widget widget, XEvent *event,
+		    String *params, Cardinal *num_params)
+{
+    if (qt_isqtvr == QTVR_PAN) {
+	if (ypos - 10 >= 0 ) ypos -= 10;
+    }
+    else if (qt_isqtvr == QTVR_OBJ) {
+	int vpos = quicktime_video_position(qt, 0);
+
+	if (vpos + lqt_qtvr_get_columns(qt) < lqt_qtvr_get_loop_frames(qt) * lqt_qtvr_get_rows(qt) * lqt_qtvr_get_columns(qt)) {
+	    quicktime_set_video_position(qt ,vpos + lqt_qtvr_get_columns(qt), 0);
+	}
+//  printf("pos %ld\n",quicktime_video_position(qt,0));
+    }
+    else 
+	return;
+    decode_qtvr();
+}
+
+
+static void down_ac(Widget widget, XEvent *event,
+		    String *params, Cardinal *num_params)
+{
+    if (qt_isqtvr == QTVR_PAN) {
+	if (ypos + 10 <= lqt_qtvr_get_height(qt) - lqt_qtvr_get_display_height(qt)) ypos += 10;
+    }
+    else if (qt_isqtvr == QTVR_OBJ) {
+	if (quicktime_video_position(qt, 0) - lqt_qtvr_get_columns(qt) >= 0) {
+	    quicktime_set_video_position(qt ,quicktime_video_position(qt, 0) - lqt_qtvr_get_columns(qt), 0);
+	}
+    }
+    else 
+	return;
+//  printf("pos %ld\n",quicktime_video_position(qt,0));    
+    decode_qtvr();
+}
+
+static void expose_ev(Widget widget, XtPointer client_data,
+		      XEvent *event, Boolean *d)
+{
+    if (event->type == Expose)
+	decode_qtvr();
+}
+
 static XtActionsRec action_table[] = {
     { "Quit",  quit_ac },
+    { "Right", right_ac },
+    { "Left",  left_ac },
+    { "Up",    up_ac },
+    { "Down",  down_ac },
 };
 
 static String res[] = {
     "lqtplay.playback.translations:  #override \\n"
     "        <Key>Q:                 Quit()    \\n"
-    "        <Key>Escape:            Quit()",
+    "        <Key>Escape:            Quit()    \\n"
+    "        <Key>Right:             Right()    \\n"
+    "        <Key>Left:              Left()   \\n"
+    "        <Key>Up:                Up() \\n"
+    "        <Key>Down:              Down()",
     "lqtplay.playback.background:    black",
     NULL
 };
@@ -1329,30 +1623,52 @@ int main(int argc, char *argv[])
 
     /* init x11 stuff */
     dpy = XtDisplay(app_shell);
+    
     XtAppAddActions(app_context,action_table,
 		    sizeof(action_table)/sizeof(XtActionsRec));
     XtVaSetValues(app_shell, XtNtitle,argv[1],NULL);
-    simple = XtVaCreateManagedWidget("playback",simpleWidgetClass,app_shell,
-				     XtNwidth,  qt_width,
-				     XtNheight, qt_height,
-				     NULL);
+    if (qt_isqtvr) {
+	simple = XtVaCreateManagedWidget("playback",simpleWidgetClass,app_shell,
+			     		 XtNwidth,  qtvr_dwidth,
+					 XtNheight, qtvr_dheight,
+					 NULL);
+	XtAddEventHandler(simple,ExposureMask, True, expose_ev, NULL);
+    }
+    else
+	simple = XtVaCreateManagedWidget("playback",simpleWidgetClass,app_shell,
+			     		 XtNwidth,  qt_width,
+					 XtNheight, qt_height,
+					 NULL);
+
+
     XtAddEventHandler(simple,StructureNotifyMask, True, resize_ev, NULL);
 
     x11_init();
     if (args.xv && qt_hasvideo)
 	xv_init();
     if(qt_hasvideo) {
-        /* Decide about the colormodel */
-        qt_cmodel = lqt_get_best_colormodel(qt, 0, qt_cmodels);
-        fprintf(stderr, "Using colormodel %s\n", lqt_colormodel_to_string(qt_cmodel));
-        /* Set decoding colormodel */
-        lqt_set_cmodel(qt, 0, qt_cmodel);
+        if (qt_isqtvr == QTVR_PAN) {
+	    /* cmodel hardcoded for panos */
+	    lqt_set_cmodel(qt, 0, BC_RGB888);
+	}
+	else {
+	    /* Decide about the colormodel */
+	    fprintf(stderr, "Stream colormodel %s, ", lqt_colormodel_to_string(lqt_get_cmodel(qt, 0)));
+	    qt_cmodel = lqt_get_best_colormodel(qt, 0, qt_cmodels);
+	    fprintf(stderr, "using %s\n", lqt_colormodel_to_string(qt_cmodel));
+	    /* Set decoding colormodel */
+	    lqt_set_cmodel(qt, 0, qt_cmodel);
+	}
     }
     /* use OpenGL? */
     XtRealizeWidget(app_shell);
-#ifdef USE_GL
-    if (BC_RGB888 == qt_cmodel && args.gl && qt_hasvideo)
-	gl_init(simple,qt_width,qt_height);
+#ifdef HAVE_GL
+    if (BC_RGB888 == qt_cmodel && args.gl && qt_hasvideo) {
+    	if (qt_isqtvr == QTVR_PAN) {
+	    gl_init(simple,qtvr_dwidth,qtvr_dheight);
+	} else 
+	    gl_init(simple,qt_width,qt_height);
+    }
 #endif    
     /* frames per chunk for alsa */
 #if 0
@@ -1363,74 +1679,82 @@ int main(int argc, char *argv[])
 #endif
     /* Initialize video */
     if(qt_hasvideo)
-      qt_init_video();
-    
-    /* enter main loop */
-    gettimeofday(&start,NULL);
-    for (;;) {
-	int rc,max;
-	fd_set rd,wr;
-	XEvent event;
+	qt_init_video();
 
-	if (True == XCheckMaskEvent(dpy, ~0, &event)) {
-	    XtDispatchEvent(&event);
-	} else {
-	    XFlush(dpy);
-	    FD_ZERO(&rd);
-	    FD_ZERO(&wr);
-	    FD_SET(ConnectionNumber(dpy),&rd);
-	    max = ConnectionNumber(dpy);
-	    if (qt_hasaudio) {
-		if (use_alsa == 0) {
-		    FD_SET(oss_fd,&wr);
-		    if (oss_fd > max)
-			max = oss_fd;
-		}
-	    }
-	    if (qt_hasvideo) {
-                if(!has_frame) {
-                    if(0 != qt_frame_decode()) {
-                        qt_hasvideo = 0;
-                        wait.tv_sec  = 0;
-                        wait.tv_usec = 1000;
-                    } else {
-                    has_frame = 1;
-                    }
-                }
-                    
-            qt_frame_delay(&start,&wait);
+    if (qt_isqtvr) {
+	decode_qtvr();
+    }
+    else {
+	/* enter main loop */
+	gettimeofday(&start,NULL);
+	for (;;) {
+	    int rc,max;
+	    fd_set rd,wr;
+	    XEvent event;
 
-            /* "wait" is the time, we would have to wait.
-               If it's longer, than 2 ms, we'll continue feeding the
-               soundcard. This prevents audio underruns for frames with a
-               VERY long duration */
-             if(wait.tv_sec || (wait.tv_usec > 2000)) {
-                 wait.tv_sec  = 0;
-                 wait.tv_usec = 2000;
-                 blit_frame = 0;
-             } else
-                 blit_frame = 1;
-                
-	 } else {
-		wait.tv_sec  = 0;
-		wait.tv_usec = 1000;
-	    }
-	    rc = select(max+1,&rd,&wr,NULL,&wait);
-	    if (qt_hasaudio) {
-		if (use_alsa == 1) {
-		    if (0 != qt_alsa_audio_write()) qt_hasaudio = 0;
+	    if (True == XCheckMaskEvent(dpy, ~0, &event)) {
+		XtDispatchEvent(&event);
+	    } 
+	    else {
+		XFlush(dpy);
+		FD_ZERO(&rd);
+		FD_ZERO(&wr);
+		FD_SET(ConnectionNumber(dpy),&rd);
+		max = ConnectionNumber(dpy);
+		if (qt_hasaudio) {
+		    if (use_alsa == 0) {
+			FD_SET(oss_fd,&wr);
+			if (oss_fd > max)
+			    max = oss_fd;
+		    }
 		}
-		else if (FD_ISSET(oss_fd,&wr)) { 
-		    if (0 != qt_oss_audio_write()) qt_hasaudio = 0;
+
+		if (qt_hasvideo) {
+		    if(!has_frame) {
+			if(0 != qt_frame_decode()) {
+			    qt_hasvideo = 0;
+			    wait.tv_sec  = 0;
+			    wait.tv_usec = 1000;
+			} else {
+			has_frame = 1;
+			}
+		    }
+			
+		qt_frame_delay(&start,&wait);
+		
+
+		/* "wait" is the time, we would have to wait.
+		   If it's longer, than 2 ms, we'll continue feeding the
+		   soundcard. This prevents audio underruns for frames with a
+		   VERY long duration */
+		 if(wait.tv_sec || (wait.tv_usec > 2000)) {
+		     wait.tv_sec  = 0;
+		     wait.tv_usec = 2000;
+		     blit_frame = 0;
+		 } else
+		     blit_frame = 1;
+		    
+	     } else {
+		    wait.tv_sec  = 0;
+		    wait.tv_usec = 1000;
+		}
+		rc = select(max+1,&rd,&wr,NULL,&wait);
+		if (qt_hasaudio) {
+		    if (use_alsa == 1) {
+			if (0 != qt_alsa_audio_write()) qt_hasaudio = 0;
+		    }
+		    else if (FD_ISSET(oss_fd,&wr)) { 
+			if (0 != qt_oss_audio_write()) qt_hasaudio = 0;
+		    }
+		}
+		if (qt_hasvideo && 0 == rc && blit_frame) {
+		    qt_frame_blit();
+		    has_frame = 0;
 		}
 	    }
-	    if (qt_hasvideo && 0 == rc && blit_frame) {
-		qt_frame_blit();
-                has_frame = 0;
-	    }
+	if(!qt_hasvideo && !qt_hasaudio)
+	  break;
 	}
-    if(!qt_hasvideo && !qt_hasaudio)
-      break;
     }
     qt_cleanup();
     fprintf(stderr, "Decoded %lld samples\n",
